@@ -21,7 +21,6 @@
  */
 #include <gnumeric-config.h>
 #include <gnumeric.h>
-#include "excel-xml-read.h"
 #include "xml-io-version.h"
 #include "sheet-view.h"
 #include "sheet-style.h"
@@ -58,6 +57,9 @@
 
 /*****************************************************************************/
 
+/* A fake value of the right type, different from all the real enum values.  */
+#define VALUE_FAKE_DATETIME ((GnmValueType)42)
+
 typedef struct {
 	GnumericXMLVersion version;
 	IOContext 	*context;	/* The IOcontext managing things */
@@ -66,7 +68,7 @@ typedef struct {
 	Sheet		*sheet;		/* The current sheet */
 	GnmCellPos	 pos;
 	GnmValueType	 val_type;
-	GnmExpr const   *expr;
+	GnmExprTop const*texpr;
 	GnmRange	 array_range;
 	char		*style_name;
 	GnmStyle	*style;
@@ -246,11 +248,11 @@ attr_color (GsfXMLIn *xin, xmlChar const * const *attrs,
 	return parse_color (xin, attrs[1], name);
 }
 
-static GnmExpr const *
+static GnmExprTop const *
 xl_xml_parse_expr (GsfXMLIn *xin, xmlChar const *expr_str,
 		   GnmParsePos const *pp)
 {
-	GnmExpr const *expr;
+	GnmExprTop const *texpr;
 	GnmParseError err;
 	if (*expr_str != '=') {
 		xl_xml_warning (xin, "Invalid formula '%s' does not begin with '='", expr_str);
@@ -263,13 +265,14 @@ xl_xml_parse_expr (GsfXMLIn *xin, xmlChar const *expr_str,
 	 * optimizations in * xls ? */
 	while (' ' == *(++expr_str))
 		;
-	expr = gnm_expr_parse_str (expr_str, pp,
+	texpr = gnm_expr_parse_str (expr_str, pp,
 		GNM_EXPR_PARSE_DEFAULT, gnm_expr_conventions_r1c1,
 		parse_error_init (&err));
-	if (NULL == expr)
+	if (NULL == texpr)
 		xl_xml_warning (xin, "'%s' %s", expr_str, err.err->message);
 	parse_error_free (&err);
-	return expr;
+
+	return texpr;
 }
 
 static void
@@ -384,7 +387,6 @@ xl_xml_cell_start (GsfXMLIn *xin, xmlChar const **attrs)
 {
 	ExcelXMLReadState *state = (ExcelXMLReadState *)xin->user_state;
 	GnmStyle *style = NULL;
-	GnmExpr const *expr;
 	int across = 0, down = 0, tmp;
 	GnmParsePos pp;
 
@@ -397,11 +399,11 @@ xl_xml_cell_start (GsfXMLIn *xin, xmlChar const **attrs)
 			if (tmp > 0)
 				state->pos.col = tmp -1;
 		} else if (gsf_xml_in_namecmp (xin, attrs[0], XL_NS_SS, "Formula")) {
-			expr = xl_xml_parse_expr (xin, attrs[1], &pp);
-			if (NULL != expr) {
-				if (NULL != state->expr)
-					gnm_expr_unref (state->expr);
-				state->expr = expr;
+			GnmExprTop const *texpr = xl_xml_parse_expr (xin, attrs[1], &pp);
+			if (NULL != texpr) {
+				if (NULL != state->texpr)
+					gnm_expr_top_unref (state->texpr);
+				state->texpr = texpr;
 			}
 		} else if (gsf_xml_in_namecmp (xin, attrs[0], XL_NS_SS, "ArrayRange")) {
 			GnmRangeRef rr;
@@ -445,7 +447,7 @@ xl_xml_data_start (GsfXMLIn *xin, xmlChar const **attrs)
 		{ "Number",	VALUE_FLOAT },
 		{ "Boolean",	VALUE_BOOLEAN },
 		{ "Error",	VALUE_ERROR },
-		{ "DateTime",	0x42 /* some cheesy magic */ },
+		{ "DateTime",	VALUE_FAKE_DATETIME },
 		{ NULL, 0 }
 	};
 	ExcelXMLReadState *state = (ExcelXMLReadState *)xin->user_state;
@@ -464,8 +466,8 @@ xl_xml_data_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
 	ExcelXMLReadState *state = (ExcelXMLReadState *)xin->user_state;
 	GnmCell *cell = sheet_cell_fetch (state->sheet, state->pos.col, state->pos.row);
 	GnmValue *v;
-	
-	if (state->val_type == 0x42) { /* see cheesy magic above */
+
+	if (state->val_type == VALUE_FAKE_DATETIME) {
 		GDate date;
 		unsigned y, mo, d, h, mi;
 		double s;
@@ -482,13 +484,13 @@ xl_xml_data_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
 			v = value_new_string (xin->content->str);
 	} else
 		v = value_new_from_string (state->val_type, xin->content->str, NULL, FALSE);
-	if (NULL != state->expr) {
+	if (NULL != state->texpr) {
 		if (NULL != v)
-			cell_set_expr_and_value (cell, state->expr, v, TRUE);
+			cell_set_expr_and_value (cell, state->texpr, v, TRUE);
 		else
-			cell_set_expr (cell, state->expr);
-		gnm_expr_unref (state->expr);
-		state->expr = NULL;
+			cell_set_expr (cell, state->texpr);
+		gnm_expr_top_unref (state->texpr);
+		state->texpr = NULL;
 	} else if (NULL != v)
 		cell_set_value (cell, v);
 	else
@@ -651,7 +653,9 @@ xl_xml_border (GsfXMLIn *xin, xmlChar const **attrs)
 	    line_type != STYLE_BORDER_MAX) {
 		border = style_border_fetch (line_type,
 			color, style_border_get_orientation (location));
-		gnm_style_set_border (state->style, MSTYLE_BORDER_TOP + location, border);
+		gnm_style_set_border (state->style,
+				      STYLE_BORDER_LOCATION_TO_STYLE_ELEMENT (location),
+				      border);
 	} else if (color)
 		    style_color_unref (color);
 }
@@ -766,11 +770,11 @@ xl_xml_named_range (GsfXMLIn *xin, xmlChar const **attrs)
 
 	if (NULL != name && NULL != expr_str) {
 		GnmParsePos pp;
-		GnmExpr const *expr = xl_xml_parse_expr (xin, expr_str,
+		GnmExprTop const *texpr = xl_xml_parse_expr (xin, expr_str,
 			parse_pos_init (&pp, state->wb, NULL, 0, 0));
 		g_warning ("%s = %s", name, expr_str);
-		if (NULL != expr)
-			expr_name_add (&pp, name, expr, NULL, TRUE, NULL);
+		if (NULL != texpr)
+			expr_name_add (&pp, name, texpr, NULL, TRUE, NULL);
 	}
 }
 
@@ -857,9 +861,7 @@ xl_xml_selection (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
 		end = rangeref_parse (&rr, ptr, &pp, gnm_expr_conventions_r1c1);
 		if (end != ptr) {
 			range_init_rangeref (&r, &rr);
-			fprintf (stderr, "%s = ", ptr);
-			range_dump (&r, "\n");
-			sv_selection_add_range (sv,
+			sv_selection_add_full (sv,
 				state->pos.col, state->pos.row,
 				r.start.col, r.start.row,
 				r.end.col, r.end.row);
@@ -962,7 +964,8 @@ GSF_XML_IN_NODE_FULL (START, WORKBOOK, XL_NS_SS, "Workbook", GSF_XML_NO_CONTENT,
       GSF_XML_IN_NODE (COND_FMT, COND,		XL_NS_XL, "Condition", GSF_XML_NO_CONTENT, NULL, NULL),
         GSF_XML_IN_NODE (COND, COND_VALUE1,	XL_NS_XL, "Value1", GSF_XML_NO_CONTENT, NULL, NULL),
         GSF_XML_IN_NODE (COND, COND_STYLE,	XL_NS_XL, "Format", GSF_XML_NO_CONTENT, NULL, NULL),
-  { NULL }
+
+  GSF_XML_IN_NODE_END
 };
 
 G_MODULE_EXPORT void
@@ -978,11 +981,11 @@ excel_xml_file_open (GOFileOpener const *fo, IOContext *io_context,
 
 	state.context	= io_context;
 	state.wb_view	= wb_view;
-	state.wb	= wb_view_workbook (wb_view);
+	state.wb	= wb_view_get_workbook (wb_view);
 	state.sheet	= NULL;
 	state.style	= NULL;
 	state.def_style	= NULL;
-	state.expr	= NULL;
+	state.texpr	= NULL;
 	state.style_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
 		(GDestroyNotify)g_free, (GDestroyNotify) gnm_style_unref);
 
