@@ -76,35 +76,34 @@ excel_write_prep_sheet (ExcelWriteState *ewb, Sheet const *sheet)
 /**
  * excel_write_prep_expr :
  * @ewb :
- * @expr :
+ * @texpr :
  *
  *  Searches for interesting functions, names, or sheets.
  * and builds a database of things to write out later.
  **/
-void
-excel_write_prep_expr (ExcelWriteState *ewb, GnmExpr const *expr)
+static void
+do_excel_write_prep_expr (ExcelWriteState *ewb, GnmExpr const *expr)
 {
-	switch (expr->any.oper) {
+	switch (GNM_EXPR_GET_OPER (expr)) {
 
 	case GNM_EXPR_OP_RANGE_CTOR:
 	case GNM_EXPR_OP_INTERSECT:
 	case GNM_EXPR_OP_ANY_BINARY:
-		excel_write_prep_expr (ewb, expr->binary.value_a);
-		excel_write_prep_expr (ewb, expr->binary.value_b);
+		do_excel_write_prep_expr (ewb, expr->binary.value_a);
+		do_excel_write_prep_expr (ewb, expr->binary.value_b);
 		break;
 
 	case GNM_EXPR_OP_ANY_UNARY:
-		excel_write_prep_expr (ewb, expr->unary.value);
+		do_excel_write_prep_expr (ewb, expr->unary.value);
 		break;
 
 	case GNM_EXPR_OP_FUNCALL: {
 		GnmFunc *func = expr->func.func;
-		GnmExprList *l;
 		ExcelFunc *ef = g_hash_table_lookup (ewb->function_map, func);
 		int i;
 
-		for (l = expr->func.arg_list; l; l = l->next)
-			excel_write_prep_expr (ewb, l->data);
+		for (i = 0; i < expr->func.argc; i++)
+			do_excel_write_prep_expr (ewb, expr->func.argv[i]);
 
 		if (ef != NULL)
 			return;
@@ -135,16 +134,17 @@ excel_write_prep_expr (ExcelWriteState *ewb, GnmExpr const *expr)
 		g_hash_table_insert (ewb->function_map, func, ef);
 		break;
 	}
-	case GNM_EXPR_OP_ARRAY:
-		if (expr->array.x == 0 && expr->array.y == 0)
-			excel_write_prep_expr (ewb, expr->array.corner.expr);
+	case GNM_EXPR_OP_ARRAY_CORNER:
+		do_excel_write_prep_expr (ewb, expr->array_corner.expr);
+		break;
+	case GNM_EXPR_OP_ARRAY_ELEM:
 		break;
 
 	case GNM_EXPR_OP_SET: {
-		GnmExprList *l;
+		int i;
 
-		for (l = expr->set.set; l; l = l->next)
-			excel_write_prep_expr (ewb, l->data);
+		for (i = 0; i < expr->set.argc; i++)
+			do_excel_write_prep_expr (ewb, expr->set.argv[i]);
 		break;
 	}
 
@@ -172,6 +172,12 @@ excel_write_prep_expr (ExcelWriteState *ewb, GnmExpr const *expr)
 	}
 }
 
+void
+excel_write_prep_expr (ExcelWriteState *ewb, GnmExprTop const *texpr)
+{
+	do_excel_write_prep_expr (ewb, texpr->expr);
+}
+
 /**
  * excel_write_prep_expressions :
  * @ewb: state
@@ -221,7 +227,7 @@ static void write_node (PolishData *pd, GnmExpr const *expr,
 #define CLASS_REF	0x00
 #define CLASS_VAL	0x20
 #define CLASS_ARRAY	0x40
-static guint8 xl_op_class[NUM_CTXTS][NUM_XL_TYPES][NUM_XL_TYPES+1] = {
+static const guint8 xl_op_class[NUM_CTXTS][NUM_XL_TYPES][NUM_XL_TYPES+1] = {
 	{ /* CELL | Wants REF	Wants VAL	Wants ARR 	From Root */
 /* From REF */	 { CLASS_REF,	CLASS_VAL,	CLASS_ARRAY, 	CLASS_VAL },
 /* From VAL */	 { CLASS_VAL,	CLASS_VAL,	CLASS_VAL, 	CLASS_VAL },
@@ -487,11 +493,10 @@ write_funcall (PolishData *pd, GnmExpr const *expr,
 {
 	static guint8 const zeros [12];
 
-	int      num_args = 0;
+	int arg;
 	gboolean prompt   = FALSE;
 	gboolean cmdequiv = FALSE;
 	char const *arg_types = NULL;
-	GnmExprList *ptr;
 	GnmFunc *func = expr->func.func;
 	ExcelFunc *ef = g_hash_table_lookup (pd->ewb->function_map, func);
 	XLOpType arg_type = XL_VAL; /* default */
@@ -524,11 +529,11 @@ write_funcall (PolishData *pd, GnmExpr const *expr,
 	} else
 		arg_types = ef->efunc->known_args;
 
-	for (ptr = expr->func.arg_list ; ptr != NULL; ptr = ptr->next, num_args++)
-		if (ef->efunc != NULL && num_args >= ef->efunc->max_args) {
+	for (arg = 0; arg < expr->func.argc; arg++)
+		if (ef->efunc != NULL && arg >= ef->efunc->max_args) {
 			gnm_io_warning (pd->ewb->io_context, 
 				_("Too many arguments for function '%s', MS Excel can only handle %d not %d"),
-				ef->efunc->name, ef->efunc->max_args, num_args);
+				ef->efunc->name, ef->efunc->max_args, arg);
 			break;
 		} else { /* convert the args */
 			if (arg_types != NULL && *arg_types) {
@@ -536,7 +541,7 @@ write_funcall (PolishData *pd, GnmExpr const *expr,
 				if (arg_types[1])
 					arg_types++;
 			}
-			write_node (pd, ptr->data, 0, arg_type);
+			write_node (pd, expr->func.argv[arg], 0, arg_type);
 		}
 
 	if (ef->efunc != NULL) {
@@ -550,12 +555,12 @@ write_funcall (PolishData *pd, GnmExpr const *expr,
 
 		/* If XL requires more arguments than we do
 		 * pad the remainder with missing args */
-		for ( ; num_args < ef->efunc->min_args ; num_args++)
+		for ( ; arg < ef->efunc->min_args ; arg++)
 			push_guint8 (pd, FORMULA_PTG_MISSARG);
 
 		if (ef->efunc->min_args != ef->efunc->max_args) {
 			push_guint8  (pd, FORMULA_PTG_FUNC_VAR + op_class);
-			push_guint8  (pd, num_args | (prompt ? 0x80 : 0));
+			push_guint8  (pd, arg | (prompt ? 0x80 : 0));
 			push_guint16 (pd, ef->idx  | (cmdequiv ? 0x8000 : 0));
 		} else {
 			push_guint8  (pd, FORMULA_PTG_FUNC + op_class);
@@ -564,7 +569,7 @@ write_funcall (PolishData *pd, GnmExpr const *expr,
 	} else { /* Undocumented, assume result is XL_VAL */
 		push_guint8  (pd, FORMULA_PTG_FUNC_VAR +
 			xl_get_op_class (pd,  XL_VAL, target_type));
-		push_guint8  (pd, (num_args+1)	| (prompt   ? 0x80 : 0));
+		push_guint8  (pd, (arg + 1)	| (prompt   ? 0x80 : 0));
 		push_guint16 (pd, 0xff  	| (cmdequiv ? 0x8000 : 0));
 	}
 }
@@ -664,16 +669,14 @@ write_node (PolishData *pd, GnmExpr const *expr, int paren_level,
 		{ FORMULA_PTG_U_MINUS,	 7, 0, 0 }, /* Unary - */
 		{ FORMULA_PTG_U_PLUS,	 7, 0, 0 }, /* Unary + */
 		{ FORMULA_PTG_PERCENT,	 6, 0, 0 }, /* Percentage (NOT MODULO) */
-		{ 0, 0, 0, 0 },	/* Array    */
+		{ 0, 0, 0, 0 },	/* Array Corner   */
+		{ 0, 0, 0, 0 },	/* Array Element  */
 		{ 0, 0, 0, 0 }, /* Set      */
 		{ FORMULA_PTG_RANGE,	 9, 1, 0 },
 		{ FORMULA_PTG_INTERSECT, 8, 1, 0 }
 	};
-	int op;
-	g_return_if_fail (pd);
-	g_return_if_fail (expr);
+	const GnmExprOp op = GNM_EXPR_GET_OPER (expr);
 
-	op = expr->any.oper;
 	switch (op) {
 	case GNM_EXPR_OP_ANY_BINARY :
 		if (target_type != XL_ARRAY)
@@ -703,27 +706,22 @@ write_node (PolishData *pd, GnmExpr const *expr, int paren_level,
 		GnmValue const *v = expr->constant.value;
 		switch (v->type) {
 
-		case VALUE_INTEGER : {
+		case VALUE_FLOAT: {
 			guint8 data[10];
-			int i = value_get_as_int (v);
-			if (i >= 0 && i < 1<<16) {
+			gnm_float f = value_get_as_float (v);
+			int i;
+
+			if (f >= 0 && f <= 0xffff && f == (i = (int)f)) {
 				GSF_LE_SET_GUINT8  (data, FORMULA_PTG_INT);
 				GSF_LE_SET_GUINT16 (data + 1, i);
 				ms_biff_put_var_write (pd->ewb->bp, data, 3);
 			} else {
 				GSF_LE_SET_GUINT8 (data, FORMULA_PTG_NUM);
-				gsf_le_set_double (data + 1, value_get_as_float (v));
+				gsf_le_set_double (data + 1, f);
 				ms_biff_put_var_write (pd->ewb->bp, data, 9);
 			}
 			break;
 		}
-		case VALUE_FLOAT : {
-			guint8 data[10];
-			GSF_LE_SET_GUINT8 (data, FORMULA_PTG_NUM);
-			gsf_le_set_double (data+1, value_get_as_float (v));
-			ms_biff_put_var_write (pd->ewb->bp, data, 9);
-			break;
-		}
 		case VALUE_BOOLEAN : {
 			guint8 data[2];
 			GSF_LE_SET_GUINT8 (data, FORMULA_PTG_BOOL);
@@ -811,21 +809,29 @@ write_node (PolishData *pd, GnmExpr const *expr, int paren_level,
 			excel_formula_write_NAME_v7 (pd, expr, target_type);
 		break;
 
-	case GNM_EXPR_OP_ARRAY : {
-		GnmExprArray const *array = &expr->array;
-		guint8 data[5], ptg = FORMULA_PTG_EXPR;
+	case GNM_EXPR_OP_ARRAY_CORNER :
+	case GNM_EXPR_OP_ARRAY_ELEM : {
+		guint8 data[5];
+		int x, y, ptg;
 
+		if (GNM_EXPR_OP_ARRAY_ELEM == op) {
+			x = expr->array_elem.x;
+			y = expr->array_elem.y;
+		} else
+			x = y = 0;
+
+		ptg = FORMULA_PTG_EXPR;
 		if (pd->sheet != NULL) {
-			GnmExprArray const *corner = cell_is_array (sheet_cell_get (
-				pd->sheet, pd->col - array->x, pd->row - array->y));
+			GnmExprArrayCorner const *corner = cell_is_array_corner (
+				sheet_cell_get (pd->sheet, pd->col - x, pd->row - y));
 			if (NULL != corner &&
-			    gnm_expr_is_data_table (corner->corner.expr, NULL, NULL))
+			    gnm_expr_is_data_table (corner->expr, NULL, NULL))
 				ptg = FORMULA_PTG_TBL;
 		}
 
 		GSF_LE_SET_GUINT8 (data, ptg);
-		GSF_LE_SET_GUINT16 (data+1, pd->row - array->y);
-		GSF_LE_SET_GUINT16 (data+3, pd->col - array->x);
+		GSF_LE_SET_GUINT16 (data+1, pd->row - y);
+		GSF_LE_SET_GUINT16 (data+3, pd->col - x);
 		ms_biff_put_var_write (pd->ewb->bp, data, 5);
 
 		/* Be anal */
@@ -835,10 +841,10 @@ write_node (PolishData *pd, GnmExpr const *expr, int paren_level,
 
 	default : {
 		gchar *err = g_strdup_printf ("Unknown Operator %d",
-					      expr->any.oper);
+					      GNM_EXPR_GET_OPER (expr));
 		write_string (pd, err);
 		g_free (err);
-		g_warning ("Unhandled expr type %d", expr->any.oper);
+		g_warning ("Unhandled expr type %d", GNM_EXPR_GET_OPER (expr));
 		break;
 	}
 	}
@@ -873,15 +879,15 @@ write_arrays (PolishData *pd)
 			for (x = 0; x < array->v_array.x; x++) {
 				GnmValue const *v = array->v_array.vals[x][y];
 
-				if (v->type == VALUE_BOOLEAN) {
+				if (VALUE_IS_BOOLEAN (v)) {
 					push_guint8 (pd, 4);
 					push_guint32 (pd, v->v_bool.val ? 1 : 0);
 					push_guint32 (pd, 0);
-				} else if (v->type == VALUE_ERROR) {
+				} else if (VALUE_IS_ERROR (v)) {
 					push_guint8 (pd, 16);
 					push_guint32 (pd, excel_write_map_errcode (v));
 					push_guint32 (pd, 0);
-				} else if (VALUE_IS_NUMBER (v)) {
+				} else if (VALUE_IS_FLOAT (v)) {
 					push_guint8 (pd, 1);
 					gsf_le_set_double (data, value_get_as_float (v));
 					ms_biff_put_var_write (pd->ewb->bp, data, 8);
@@ -899,7 +905,40 @@ write_arrays (PolishData *pd)
 }
 
 guint32
-excel_write_formula (ExcelWriteState *ewb, GnmExpr const *expr,
+excel_write_array_formula (ExcelWriteState *ewb,
+			   GnmExprArrayCorner const *array,
+			   Sheet *sheet, int fn_col, int fn_row)
+{
+	PolishData pd;
+	unsigned start;
+	guint32 len;
+
+	g_return_val_if_fail (ewb, 0);
+	g_return_val_if_fail (array, 0);
+
+	pd.col     = fn_col;
+	pd.row     = fn_row;
+	pd.sheet   = sheet;
+	pd.ewb     = ewb;
+	pd.arrays  = NULL;
+	pd.context = CTXT_ARRAY;
+	pd.use_name_variant = FALSE;
+
+	start = ewb->bp->length;
+	write_node (&pd, array->expr, 0, XL_ROOT);
+	len = ewb->bp->length - start;
+
+	write_arrays (&pd);
+
+	return len;
+}
+
+/*
+ * This is called for all expressions, including array expressions.
+ * (But write_node will not write the inner expression for arrays.)
+ */
+guint32
+excel_write_formula (ExcelWriteState *ewb, GnmExprTop const *texpr,
 		     Sheet *sheet, int fn_col, int fn_row,
 		     ExcelFuncContext context)
 {
@@ -908,25 +947,35 @@ excel_write_formula (ExcelWriteState *ewb, GnmExpr const *expr,
 	guint32 len;
 
 	g_return_val_if_fail (ewb, 0);
-	g_return_val_if_fail (expr, 0);
+	g_return_val_if_fail (texpr, 0);
 
 	pd.col     = fn_col;
 	pd.row     = fn_row;
 	pd.sheet   = sheet;
 	pd.ewb     = ewb;
 	pd.arrays  = NULL;
-	if ((context == EXCEL_CALLED_FROM_CELL || context == EXCEL_CALLED_FROM_SHARED))
+	switch (context) {
+	case EXCEL_CALLED_FROM_CELL:
 		pd.context = CTXT_CELL;
-	else if (context == EXCEL_CALLED_FROM_NAME)
+		pd.use_name_variant = FALSE;
+		break;
+	case EXCEL_CALLED_FROM_SHARED:
+		pd.context = CTXT_CELL;
+		pd.use_name_variant = TRUE;
+		break;
+	case EXCEL_CALLED_FROM_NAME:
 		pd.context = CTXT_NAME;
-	else
-		pd.context = CTXT_ARRAY;
-	pd.use_name_variant =
-		context != EXCEL_CALLED_FROM_CELL &&
-		context != EXCEL_CALLED_FROM_ARRAY;
+		pd.use_name_variant = TRUE;
+		break;
+	case EXCEL_CALLED_FROM_CONDITION:
+	case EXCEL_CALLED_FROM_VALIDATION:
+	default:
+		pd.context = CTXT_ARRAY; /* What???  */
+		pd.use_name_variant = TRUE;
+	}
 
 	start = ewb->bp->length;
-	write_node (&pd, expr, 0, XL_ROOT);
+	write_node (&pd, texpr->expr, 0, XL_ROOT);
 	len = ewb->bp->length - start;
 
 	write_arrays (&pd);
