@@ -5,7 +5,7 @@
  * Authors:
  *   Michael Meeks <michael@ximian.com>
  *   Jukka-Pekka Iivonen <iivonen@iki.fi>
- *   Morten Welinder <terra@diku.dk>
+ *   Morten Welinder <terra@gnome.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -81,39 +81,21 @@ val_to_base (FunctionEvalInfo *ei,
 			      value_new_error_VALUE (ei->pos));
 
 	/* func.c ought to take care of this.  */
-	if (VALUE_TYPE (value) == VALUE_BOOLEAN)
+	if (VALUE_IS_BOOLEAN (value))
 		return value_new_error_VALUE (ei->pos);
-	if (aplaces && VALUE_TYPE (aplaces) == VALUE_BOOLEAN)
+	if (aplaces && VALUE_IS_BOOLEAN (aplaces))
 		return value_new_error_VALUE (ei->pos);
 
-	switch (VALUE_TYPE (value)) {
+	switch (value->type) {
 	default:
 		return value_new_error_NUM (ei->pos);
 
-	case VALUE_INTEGER: {
-		int val = value_get_as_int (value);
-		char buf[4 * sizeof (int)];
-		char *err;
-
-		if (val < min_value || val > max_value)
-			value_new_error_NUM (ei->pos);
-
-		sprintf (buf, "%d", val);
-		v = strtol (buf, &err, src_base);
-		if (*err != 0)
-			return value_new_error_NUM (ei->pos);
-
-		break;
-	}
-
 	case VALUE_STRING:
 		if (flags & V2B_STRINGS_GENERAL) {
 			vstring = format_match_number
 				(value_peek_string (value), NULL,
 				 workbook_date_conv (ei->pos->sheet->workbook));
-			if (!vstring ||
-			    VALUE_TYPE (vstring) == VALUE_BOOLEAN ||
-			    !VALUE_IS_NUMBER (vstring)) {
+			if (!vstring || !VALUE_IS_FLOAT (vstring)) {
 				if (vstring)
 					value_release (vstring);
 				return value_new_error_VALUE (ei->pos);
@@ -194,9 +176,10 @@ val_to_base (FunctionEvalInfo *ei,
 	}
 
 	if (aplaces) {
-		places = value_get_as_int (aplaces);
-		if (places < min || places > 10)
+		gnm_float fplaces = value_get_as_float (aplaces);
+		if (fplaces < min || fplaces > 10)
 			return value_new_error_NUM (ei->pos);
+		places = (int)fplaces;
 		if (v >= 0 && places > max)
 			max = places;
 	} else
@@ -243,12 +226,12 @@ static GnmValue *
 gnumeric_base (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	static const gnm_float max = 1 / GNM_EPSILON;
-	int base = value_get_as_int (argv[1]);
+	gnm_float base = value_get_as_float (argv[1]);
 
-	if (base < 2 || base > 36)
+	if (base < 2 || base >= 37)
 		return value_new_error_NUM (ei->pos);
 
-	return val_to_base (ei, argv[0], argv[2], 10, base,
+	return val_to_base (ei, argv[0], argv[2], 10, (int)base,
 			    -max, +max,
 			    V2B_STRINGS_GENERAL | V2B_STRINGS_0XH);
 }
@@ -466,9 +449,9 @@ static GnmFuncHelp const help_decimal[] = {
 static GnmValue *
 gnumeric_decimal (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	int base = value_get_as_int (argv[1]);
+	gnm_float base = value_get_as_float (argv[1]);
 
-	if (base < 2 || base > 36)
+	if (base < 2 || base >= 37)
 		return value_new_error_NUM (ei->pos);
 
 	return value_new_error_NUM (ei->pos);
@@ -690,14 +673,16 @@ static GnmFuncHelp const help_besseli[] = {
 static GnmValue *
 gnumeric_besseli (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	gnm_float x, order;
-
-	x = value_get_as_float (argv[0]);	/* value to evaluate I_n at. */
-	order = value_get_as_float (argv[1]);	/* the order */
+	gnm_float x = value_get_as_float (argv[0]);	/* value to evaluate I_n at. */
+	gnm_float order = value_get_as_float (argv[1]);	/* the order */
 
 	if (order < 0)
 		return value_new_error_NUM (ei->pos);
 
+	/* It seems that XL uses abs value of x */
+	if (x < 0.)
+		x = -x;
+
 	return value_new_float (bessel_i (x, order, 1.0));
 }
 
@@ -730,10 +715,8 @@ static GnmFuncHelp const help_besselk[] = {
 static GnmValue *
 gnumeric_besselk (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	gnm_float x, order;
-
-	x = value_get_as_float (argv[0]);	/* value to evaluate K_n at. */
-	order = value_get_as_float (argv[1]);	/* the order */
+	gnm_float x = value_get_as_float (argv[0]);	/* value to evaluate K_n at. */
+	gnm_float order = value_get_as_float (argv[1]);	/* the order */
 
 	if (order < 0)
 		return value_new_error_NUM (ei->pos);
@@ -769,15 +752,14 @@ static GnmFuncHelp const help_besselj[] = {
 static GnmValue *
 gnumeric_besselj (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	int x, y;
+	gnm_float x = value_get_as_float (argv[0]);
+	gnm_float y = value_get_as_float (argv[1]);
 
-	x = value_get_as_int (argv[0]);
-	y = value_get_as_int (argv[1]);
-
-	if (y < 0)
+	if (y < 0 || y > INT_MAX)
 		return value_new_error_NUM (ei->pos);
 
-	return value_new_float (jn (y, value_get_as_float (argv[0])));
+	/* FIXME: Why not gnm_jn?  */
+	return value_new_float (jn ((int)y, x));
 }
 
 /***************************************************************************/
@@ -809,17 +791,13 @@ static GnmFuncHelp const help_bessely[] = {
 static GnmValue *
 gnumeric_bessely (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	int y;
-	if (argv[0]->type != VALUE_INTEGER &&
-	    argv[1]->type != VALUE_INTEGER &&
-	    argv[0]->type != VALUE_FLOAT &&
-	    argv[1]->type != VALUE_FLOAT)
-		return value_new_error_VALUE (ei->pos);
+	gnm_float x = value_get_as_float (argv[0]);
+	gnm_float y = value_get_as_float (argv[1]);
 
-	if ((y = value_get_as_int (argv[1])) < 0)
+	if (y < 0 || y > INT_MAX)
 		return value_new_error_NUM (ei->pos);
 
-	return value_new_float (gnm_yn (y, value_get_as_float (argv[0])));
+	return value_new_float (gnm_yn ((int)y, x));
 }
 
 /***************************************************************************/
@@ -921,7 +899,7 @@ static GnmFuncHelp const help_convert[] = {
 	   "\t'y'  \tyocto \t\t1E-24\n"
 	   "\n"
 	   "* If @from_unit and @to_unit are different types, CONVERT returns "
-	   "#NUM! error.\n"
+	   "#N/A error.\n"
 	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1240,7 +1218,7 @@ gnumeric_convert (FunctionEvalInfo *ei, GnmValue const * const *argv)
 		    ei->pos))
 	        return v;
 
-	return value_new_error_NUM (ei->pos);
+	return value_new_error_NA (ei->pos);
 }
 
 /***************************************************************************/
@@ -1286,6 +1264,7 @@ gnumeric_erf (FunctionEvalInfo *ei, GnmValue const * const *argv)
 	ans = gnm_erf (lower);
 
 	if (argv[1]) {
+		/* FIXME: This is suboptimal.  */
 		upper = value_get_as_float (argv[1]);
 		ans = gnm_erf (upper) - ans;
 	}
@@ -1319,9 +1298,7 @@ static GnmFuncHelp const help_erfc[] = {
 static GnmValue *
 gnumeric_erfc (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	gnm_float x;
-
-	x = value_get_as_float (argv[0]);
+	gnm_float x = value_get_as_float (argv[0]);
 
 	return value_new_float (gnm_erfc (x));
 }
@@ -1352,36 +1329,10 @@ static GnmFuncHelp const help_delta[] = {
 static GnmValue *
 gnumeric_delta (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	GnmValue *err = NULL;
-	gboolean ans = FALSE;
-	GnmValue const *vx, *vy;
+	gnm_float x = value_get_as_float (argv[0]);
+	gnm_float y = argv[1] ? value_get_as_float (argv[1]) : 0;
 
-	vx = argv[0];
-	if (argv[1])
-		vy = argv[1];
-	else
-		vy = value_new_int (0);
-
-	/* Promote to the largest value */
-	switch ((vx->type > vy->type) ? vx->type : vy->type) {
-	case VALUE_BOOLEAN:
-		/* Only happens when both are bool */
-		ans = vx->v_bool.val == vy->v_bool.val;
-		break;
-	case VALUE_EMPTY:
-	case VALUE_INTEGER:
-		ans = value_get_as_int (vx) == value_get_as_int (vy);
-		break;
-	case VALUE_FLOAT:
-		ans = value_get_as_float (vx) == value_get_as_float (vy);
-		break;
-	default:
-		err = value_new_error (ei->pos, _("Impossible"));
-	}
-	if (!argv[1])
-		value_release ((GnmValue *)vy);
-
-	return (err != NULL) ? err : value_new_int (ans ? 1 : 0);
+	return value_new_int (x == y);
 }
 
 /***************************************************************************/
@@ -1409,36 +1360,10 @@ static GnmFuncHelp const help_gestep[] = {
 static GnmValue *
 gnumeric_gestep (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	GnmValue *err = NULL;
-	gboolean ans = FALSE;
-	GnmValue const *vx, *vy;
+	gnm_float x = value_get_as_float (argv[0]);
+	gnm_float y = argv[1] ? value_get_as_float (argv[1]) : 0;
 
-	vx = argv[0];
-	if (argv[1])
-		vy = argv[1];
-	else
-		vy = value_new_int (0);
-
-	/* Promote to the largest value */
-	switch ((vx->type > vy->type) ? vx->type : vy->type) {
-	case VALUE_BOOLEAN:
-		/* Only happens when both are bool */
-		ans = vx->v_bool.val >= vy->v_bool.val;
-		break;
-	case VALUE_EMPTY:
-	case VALUE_INTEGER:
-		ans = value_get_as_int (vx) >= value_get_as_int (vy);
-		break;
-	case VALUE_FLOAT:
-		ans = value_get_as_float (vx) >= value_get_as_float (vy);
-		break;
-	default:
-		err = value_new_error (ei->pos, _("Impossible"));
-	}
-
-	if (!argv[1])
-		value_release ((GnmValue *)vy);
-	return (err != NULL) ? err : value_new_int (ans ? 1 : 0);
+	return value_new_int (x >= y);
 }
 
 /***************************************************************************/
@@ -1487,9 +1412,9 @@ range_invsuminv (gnm_float const *xs, int n, gnm_float *res)
 }
 
 static GnmValue *
-gnumeric_invsuminv (FunctionEvalInfo *ei, GnmExprList const *nodes)
+gnumeric_invsuminv (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
-	return float_range_function (nodes, ei,
+	return float_range_function (argc, argv, ei,
 				     range_invsuminv,
 				     COLLECT_IGNORE_STRINGS |
 				     COLLECT_IGNORE_BOOLS |
