@@ -5,7 +5,7 @@
  * Authors:
  *   Jukka-Pekka Iivonen (iivonen@iki.fi)
  *   Jody Goldberg (jody@gnome.org)
- *   Morten Welinder (terra@diku.dk)
+ *   Morten Welinder (terra@gnome.org)
  *   Almer S. Tigelaar (almer@gnome.org)
  *   Harlan Grove
  *
@@ -48,6 +48,7 @@
 #include <number-match.h>
 #include <gnm-i18n.h>
 
+#include <goffice/app/go-doc.h>
 #include <goffice/app/go-plugin.h>
 #include <gnm-plugin.h>
 
@@ -247,7 +248,7 @@ gnumeric_cell (FunctionEvalInfo *ei, GnmValue const * const *argv)
 	 * the worksheet name, but they can't make any other changes to CELL?!
 	 */
 	} else if (!g_ascii_strcasecmp (info_type, "filename")) {
-		char const *name = workbook_get_uri (sheet->workbook);
+		char const *name = go_doc_get_uri (GO_DOC (sheet->workbook));
 
 		if (name == NULL)
 			return value_new_string ("");
@@ -281,14 +282,18 @@ gnumeric_cell (FunctionEvalInfo *ei, GnmValue const * const *argv)
 		GnmCell const *cell =
 			sheet_cell_get (sheet, ref->col, ref->row);
 
-		if (cell && cell->value && cell->value->type == VALUE_STRING) {
+		if (cell && cell->value && VALUE_IS_STRING (cell->value)) {
 			switch (gnm_style_get_align_h (mstyle)) {
-			case HALIGN_GENERAL: return value_new_string ("'");
-			case HALIGN_LEFT:    return value_new_string ("'");
-			case HALIGN_RIGHT:   return value_new_string ("\"");
-			case HALIGN_CENTER:  return value_new_string ("^");
-			case HALIGN_FILL:    return value_new_string ("\\");
-			default : 	     return value_new_string ("");
+			case HALIGN_GENERAL:
+			case HALIGN_LEFT:
+			case HALIGN_JUSTIFY:
+			case HALIGN_DISTRIBUTED:
+						return value_new_string ("'");
+			case HALIGN_RIGHT:	return value_new_string ("\"");
+			case HALIGN_CENTER_ACROSS_SELECTION:
+			case HALIGN_CENTER:	return value_new_string ("^");
+			case HALIGN_FILL:	return value_new_string ("\\");
+			default :		return value_new_string ("");
 			}
 		}
 		return value_new_string ("");
@@ -298,7 +303,7 @@ gnumeric_cell (FunctionEvalInfo *ei, GnmValue const * const *argv)
 		   !g_ascii_strcasecmp (info_type, "protect")) {
 		GnmStyle const *mstyle =
 			sheet_style_get (sheet, ref->col, ref->row);
-		return value_new_int (gnm_style_get_content_locked (mstyle) ? 1 : 0);
+		return value_new_int (gnm_style_get_contents_locked (mstyle) ? 1 : 0);
 
     /* different characteristics grouped for efficiency
      * TYPE needed for backward compatibility w/123 but otherwise useless
@@ -330,7 +335,7 @@ gnumeric_cell (FunctionEvalInfo *ei, GnmValue const * const *argv)
 		GnmCell const *cell =
 			sheet_cell_get (sheet, ref->col, ref->row);
 		if (cell && cell->value) {
-			if (cell->value->type == VALUE_STRING)
+			if (VALUE_IS_STRING (cell->value))
 				return value_new_string ("l");
 			else
 				return value_new_string ("v");
@@ -354,7 +359,6 @@ gnumeric_cell (FunctionEvalInfo *ei, GnmValue const * const *argv)
 	return value_new_error_VALUE (ei->pos);
 }
 
-#warning implement this
 #if 0
 /*
 *extension to CELL providing 123 @CELL/@CELLPOINTER functionality as
@@ -1199,9 +1203,10 @@ gnumeric_expression (FunctionEvalInfo *ei, GnmValue const * const *argv)
 
 		if (cell && cell_has_expr (cell)) {
 			GnmParsePos pos;
-			char *expr_string = gnm_expr_as_string (cell->base.expression,
-				parse_pos_init_cell (&pos, cell),
-				gnm_expr_conventions_default);
+			char *expr_string = gnm_expr_top_as_string
+				(cell->base.texpr,
+				 parse_pos_init_cell (&pos, cell),
+				 gnm_expr_conventions_default);
 			return value_new_string_nocopy (expr_string);
 		}
 	}
@@ -1392,7 +1397,7 @@ static GnmFuncHelp const help_iserror[] = {
 static GnmValue *
 gnumeric_iserror (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return value_new_bool (argv[0]->type == VALUE_ERROR);
+	return value_new_bool (VALUE_IS_ERROR (argv[0]));
 }
 
 /***************************************************************************/
@@ -1446,7 +1451,7 @@ static GnmFuncHelp const help_iserr[] = {
 static GnmValue *
 gnumeric_iserr (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return value_new_bool (argv[0]->type == VALUE_ERROR &&
+	return value_new_bool (VALUE_IS_ERROR (argv[0]) &&
 			       value_error_classify (argv[0]) != GNM_ERROR_NA);
 }
 
@@ -1563,7 +1568,7 @@ static GnmFuncHelp const help_isblank[] = {
 static GnmValue *
 gnumeric_isblank (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return value_new_bool (argv[0]->type == VALUE_EMPTY);
+	return value_new_bool (VALUE_IS_EMPTY (argv[0]));
 }
 
 /***************************************************************************/
@@ -1588,7 +1593,11 @@ static GnmFuncHelp const help_iseven[] = {
 static GnmValue *
 gnumeric_iseven (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return value_new_bool (!(value_get_as_int (argv[0]) & 1));
+	gnm_float x = value_get_as_float (argv[0]);
+	gnm_float r = gnm_fmod (gnm_abs (x), 2);
+
+	/* If x is too big, this will always be true.  */
+	return value_new_bool (r < 1);
 }
 
 /***************************************************************************/
@@ -1613,7 +1622,7 @@ static GnmFuncHelp const help_islogical[] = {
 static GnmValue *
 gnumeric_islogical (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return value_new_bool (argv[0]->type == VALUE_BOOLEAN);
+	return value_new_bool (VALUE_IS_BOOLEAN (argv[0]));
 }
 
 /***************************************************************************/
@@ -1638,7 +1647,7 @@ static GnmFuncHelp const help_isnontext[] = {
 static GnmValue *
 gnumeric_isnontext (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return value_new_bool (argv[0]->type != VALUE_STRING);
+	return value_new_bool (!VALUE_IS_STRING (argv[0]));
 }
 
 /***************************************************************************/
@@ -1663,8 +1672,7 @@ static GnmFuncHelp const help_isnumber[] = {
 static GnmValue *
 gnumeric_isnumber (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return value_new_bool (argv[0] != NULL &&
-		(argv[0]->type == VALUE_INTEGER || argv[0]->type == VALUE_FLOAT));
+	return value_new_bool (argv[0] && VALUE_IS_FLOAT (argv[0]));
 }
 
 /***************************************************************************/
@@ -1689,7 +1697,11 @@ static GnmFuncHelp const help_isodd[] = {
 static GnmValue *
 gnumeric_isodd (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return value_new_bool (value_get_as_int (argv[0]) & 1);
+	gnm_float x = value_get_as_float (argv[0]);
+	gnm_float r = gnm_fmod (gnm_abs (x), 2);
+
+	/* If x is too big, this will always be false.  */
+	return value_new_bool (r >= 1);
 }
 
 /***************************************************************************/
@@ -1712,19 +1724,13 @@ static GnmFuncHelp const help_isref[] = {
 };
 
 static GnmValue *
-gnumeric_isref (FunctionEvalInfo *ei, GnmExprList const *expr_node_list)
+gnumeric_isref (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
-	GnmExpr *t;
-
-	if (gnm_expr_list_length (expr_node_list) != 1)
+	if (argc != 1)
 		return value_new_error (ei->pos,
 					_("Invalid number of arguments"));
 
-	t = expr_node_list->data;
-	if (!t)
-		return NULL;
-
-	return value_new_bool (t->any.oper == GNM_EXPR_OP_CELLREF);
+	return value_new_bool (GNM_EXPR_GET_OPER (argv[0]) == GNM_EXPR_OP_CELLREF);
 }
 
 /***************************************************************************/
@@ -1749,7 +1755,7 @@ static GnmFuncHelp const help_istext[] = {
 static GnmValue *
 gnumeric_istext (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return value_new_bool (argv[0]->type == VALUE_STRING);
+	return value_new_bool (VALUE_IS_STRING (argv[0]));
 }
 
 /***************************************************************************/
@@ -1775,23 +1781,20 @@ static GnmFuncHelp const help_n[] = {
 static GnmValue *
 gnumeric_n (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	char const *str;
 	GnmValue *v;
 
-	if (argv[0]->type == VALUE_BOOLEAN)
-		return value_new_int (value_get_as_int(argv[0]));
-
 	if (VALUE_IS_NUMBER (argv[0]))
-		return value_dup (argv[0]);
+		return value_new_float (value_get_as_float (argv[0]));
 
-	if (argv[0]->type != VALUE_STRING)
+	if (!VALUE_IS_STRING (argv[0]))
 		return value_new_error_NUM (ei->pos);
 
-	str = value_peek_string (argv[0]);
-	v = format_match_number (str, NULL,
-		workbook_date_conv (ei->pos->sheet->workbook));
+	v = format_match_number (value_peek_string (argv[0]),
+				 NULL,
+				 workbook_date_conv (ei->pos->sheet->workbook));
 	if (v != NULL)
 		return v;
+
 	return value_new_float (0);
 }
 
@@ -1824,11 +1827,10 @@ static GnmValue *
 gnumeric_type (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	const GnmValue *v = argv[0];
-	switch (v ? VALUE_TYPE (v) : VALUE_EMPTY) {
+	switch (v ? v->type : VALUE_EMPTY) {
 	case VALUE_BOOLEAN:
 		return value_new_int (4);
 	case VALUE_EMPTY:
-	case VALUE_INTEGER:
 	case VALUE_FLOAT:
 		return value_new_int (1);
 	case VALUE_CELLRANGE:
