@@ -5,7 +5,7 @@
  * Authors:
  *   Miguel de Icaza (miguel@gnu.org)
  *   Jukka-Pekka Iivonen (iivonen@iki.fi)
- *   Morten Welinder (terra@diku.dk)
+ *   Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,25 +23,26 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include <func.h>
 
 #include <parse-util.h>
 #include <cell.h>
 #include <expr.h>
+#include <expr-impl.h>
 #include <value.h>
+#include <gnm-i18n.h>
 
-#include "plugin.h"
-#include "plugin-util.h"
-#include "module-plugin-defs.h"
+#include <goffice/app/go-plugin.h>
+#include <gnm-plugin.h>
 
-GNUMERIC_MODULE_PLUGIN_INFO_DECL;
+GNM_PLUGIN_MODULE_HEADER;
 
 /***************************************************************************/
 
-static char const *help_and = {
-	N_("@FUNCTION=AND\n"
+static GnmFuncHelp const help_and[] = {
+	{ GNM_FUNC_HELP_OLD,
+	F_("@FUNCTION=AND\n"
 	   "@SYNTAX=AND(b1, b2, ...)\n"
 
 	   "@DESCRIPTION="
@@ -49,7 +50,7 @@ static char const *help_and = {
 	   "if all of the expressions evaluate to TRUE, otherwise it returns "
 	   "FALSE.\n"
 	   "\n"
-	   "@b1 trough @bN are expressions that should evaluate to TRUE "
+	   "@b1 through @bN are expressions that should evaluate to TRUE "
 	   "or FALSE.  If an integer or floating point value is provided, "
 	   "zero is considered FALSE and anything else is TRUE.\n"
 	   "\n"
@@ -66,32 +67,34 @@ static char const *help_and = {
 	   "AND(A1>3,A2<2) equals TRUE.\n"
 	   "\n"
 	   "@SEEALSO=OR, NOT")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-callback_function_and (const GnmEvalPos *ep, GnmValue *value, void *closure)
+callback_function_and (GnmEvalPos const *ep, GnmValue const *value, void *closure)
 {
 	int *result = closure;
 
 	if (!VALUE_IS_STRING (value)) {
 		gboolean err;
-	*result = value_get_as_bool (value, &err) && *result;
-	if (err)
-		return value_new_error_VALUE (ep);
+		*result = value_get_as_bool (value, &err) && *result;
+		if (err)
+			return value_new_error_VALUE (ep);
 	}
 
 	return NULL;
 }
 
 static GnmValue *
-gnumeric_and (FunctionEvalInfo *ei, GnmExprList *nodes)
+gnumeric_and (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
 	int result = -1;
 
 	/* Yes, AND is actually strict.  */
-	GnmValue *v = function_iterate_argument_values (ei->pos,
-		callback_function_and, &result, nodes,
-		TRUE, CELL_ITER_IGNORE_BLANK);
+	GnmValue *v = function_iterate_argument_values
+		(ei->pos, callback_function_and, &result,
+		 argc, argv, TRUE, CELL_ITER_IGNORE_BLANK);
 	if (v != NULL)
 		return v;
 
@@ -104,8 +107,9 @@ gnumeric_and (FunctionEvalInfo *ei, GnmExprList *nodes)
 
 /***************************************************************************/
 
-static char const *help_not = {
-	N_("@FUNCTION=NOT\n"
+static GnmFuncHelp const help_not[] = {
+	{ GNM_FUNC_HELP_OLD,
+	F_("@FUNCTION=NOT\n"
 	   "@SYNTAX=NOT(number)\n"
 
 	   "@DESCRIPTION="
@@ -119,10 +123,12 @@ static char const *help_not = {
 	   "NOT(TRUE) equals FALSE.\n"
 	   "\n"
 	   "@SEEALSO=AND, OR")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_not (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_not (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gboolean err, val = value_get_as_bool (argv [0], &err);
 	if (err)
@@ -132,15 +138,16 @@ gnumeric_not (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_or = {
-	N_("@FUNCTION=OR\n"
+static GnmFuncHelp const help_or[] = {
+	{ GNM_FUNC_HELP_OLD,
+	F_("@FUNCTION=OR\n"
 	   "@SYNTAX=OR(b1, b2, ...)\n"
 
 	   "@DESCRIPTION="
 	   "OR implements the logical OR function: the result is TRUE if "
 	   "any of the values evaluated to TRUE.\n"
 	   "\n"
-	   "@b1 trough @bN are expressions that should evaluate to TRUE "
+	   "@b1 through @bN are expressions that should evaluate to TRUE "
 	   "or FALSE. If an integer or floating point value is provided, "
 	   "zero is considered FALSE and anything else is TRUE.\n"
 	   "\n"
@@ -155,32 +162,34 @@ static char const *help_or = {
 	   "OR(3>4,4<3) equals FALSE.\n"
 	   "\n"
 	   "@SEEALSO=AND, NOT")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-callback_function_or (const GnmEvalPos *ep, GnmValue *value, void *closure)
+callback_function_or (GnmEvalPos const *ep, GnmValue const *value, void *closure)
 {
 	int *result = closure;
 
 	if (!VALUE_IS_STRING (value)) {
 		gboolean err;
-	*result = value_get_as_bool (value, &err) || *result == 1;
-	if (err)
-		return value_new_error_VALUE (ep);
+		*result = value_get_as_bool (value, &err) || *result == 1;
+		if (err)
+			return value_new_error_VALUE (ep);
 	}
 
 	return NULL;
 }
 
 static GnmValue *
-gnumeric_or (FunctionEvalInfo *ei, GnmExprList *nodes)
+gnumeric_or (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
 	int result = -1;
 
 	/* Yes, OR is actually strict.  */
-	GnmValue *v = function_iterate_argument_values (ei->pos,
-		callback_function_or, &result, nodes,
-		TRUE, CELL_ITER_IGNORE_BLANK);
+	GnmValue *v = function_iterate_argument_values
+		(ei->pos, callback_function_or, &result,
+		 argc, argv, TRUE, CELL_ITER_IGNORE_BLANK);
 	if (v != NULL)
 		return v;
 
@@ -193,15 +202,16 @@ gnumeric_or (FunctionEvalInfo *ei, GnmExprList *nodes)
 
 /***************************************************************************/
 
-static char const *help_xor = {
-	N_("@FUNCTION=XOR\n"
+static GnmFuncHelp const help_xor[] = {
+	{ GNM_FUNC_HELP_OLD,
+	F_("@FUNCTION=XOR\n"
 	   "@SYNTAX=XOR(b1, b2, ...)\n"
 
 	   "@DESCRIPTION="
 	   "XOR implements the logical exclusive OR function: the result is "
 	   "TRUE if an odd number of the values evaluated to TRUE.\n"
 	   "\n"
-	   "@b1 trough @bN are expressions that should evaluate to TRUE "
+	   "@b1 through @bN are expressions that should evaluate to TRUE "
 	   "or FALSE. If an integer or floating point value is provided, "
 	   "zero is considered FALSE and anything else is TRUE.\n"
 	   "\n"
@@ -214,32 +224,34 @@ static char const *help_xor = {
 	   "XOR(3>4,4<3) equals FALSE.\n"
 	   "\n"
 	   "@SEEALSO=OR, AND, NOT")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-callback_function_xor (const GnmEvalPos *ep, GnmValue *value, void *closure)
+callback_function_xor (GnmEvalPos const *ep, GnmValue const *value, void *closure)
 {
 	int *result = closure;
 
 	if (!VALUE_IS_STRING (value)) {
 		gboolean err;
-	*result = value_get_as_bool (value, &err) ^ (*result == 1);
-	if (err)
-		return value_new_error_VALUE (ep);
+		*result = value_get_as_bool (value, &err) ^ (*result == 1);
+		if (err)
+			return value_new_error_VALUE (ep);
 	}
 
 	return NULL;
 }
 
 static GnmValue *
-gnumeric_xor (FunctionEvalInfo *ei, GnmExprList *nodes)
+gnumeric_xor (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
 	int result = -1;
 
 	/* Yes, XOR is actually strict.  */
-	GnmValue *v = function_iterate_argument_values (ei->pos,
-		callback_function_xor, &result, nodes,
-		TRUE, CELL_ITER_IGNORE_BLANK);
+	GnmValue *v = function_iterate_argument_values
+		(ei->pos, callback_function_xor, &result,
+		 argc, argv, TRUE, CELL_ITER_IGNORE_BLANK);
 	if (v != NULL)
 		return v;
 
@@ -252,8 +264,9 @@ gnumeric_xor (FunctionEvalInfo *ei, GnmExprList *nodes)
 
 /***************************************************************************/
 
-static char const *help_if = {
-	N_("@FUNCTION=IF\n"
+static GnmFuncHelp const help_if[] = {
+	{ GNM_FUNC_HELP_OLD,
+	F_("@FUNCTION=IF\n"
 	   "@SYNTAX=IF(condition[,if-true,if-false])\n"
 
 	   "@DESCRIPTION="
@@ -269,20 +282,20 @@ static char const *help_if = {
 	   "IF(FALSE,TRUE,FALSE) equals FALSE.\n"
 	   "\n"
 	   "@SEEALSO=")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_if (FunctionEvalInfo *ei, GnmValue **args)
+gnumeric_if (FunctionEvalInfo *ei, GnmValue const * const *args)
 {
 	gboolean err;
-	int argcount;
 	int res = value_get_as_bool (args[0], &err) ? 1 : 2;
 
 	if (args[res])
 		return value_dup (args[res]);
 
-	argcount = gnm_expr_get_func_argcount ((const GnmExpr *)ei->func_call);
-	if (argcount < res + 1)
+	if (ei->func_call->argc < res + 1)
 		/* arg-not-there: default to TRUE/FALSE.  */
 		return value_new_bool (res == 1);
 	else
@@ -292,8 +305,9 @@ gnumeric_if (FunctionEvalInfo *ei, GnmValue **args)
 
 /***************************************************************************/
 
-static char const *help_true = {
-	N_("@FUNCTION=TRUE\n"
+static GnmFuncHelp const help_true[] = {
+	{ GNM_FUNC_HELP_OLD,
+	F_("@FUNCTION=TRUE\n"
 	   "@SYNTAX=TRUE()\n"
 
 	   "@DESCRIPTION="
@@ -304,18 +318,21 @@ static char const *help_true = {
 	   "TRUE() equals TRUE.\n"
 	   "\n"
 	   "@SEEALSO=FALSE")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_true (FunctionEvalInfo *ei, GnmValue **args)
+gnumeric_true (FunctionEvalInfo *ei, GnmValue const * const *args)
 {
 	return value_new_bool (TRUE);
 }
 
 /***************************************************************************/
 
-static char const *help_false = {
-	N_("@FUNCTION=FALSE\n"
+static GnmFuncHelp const help_false[] = {
+	{ GNM_FUNC_HELP_OLD,
+	F_("@FUNCTION=FALSE\n"
 	   "@SYNTAX=FALSE()\n"
 
 	   "@DESCRIPTION="
@@ -326,10 +343,12 @@ static char const *help_false = {
 	   "FALSE() equals FALSE.\n"
 	   "\n"
 	   "@SEEALSO=TRUE")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_false (FunctionEvalInfo *ei, GnmValue **args)
+gnumeric_false (FunctionEvalInfo *ei, GnmValue const * const *args)
 {
 	return value_new_bool (FALSE);
 }
@@ -337,31 +356,31 @@ gnumeric_false (FunctionEvalInfo *ei, GnmValue **args)
 /***************************************************************************/
 
 const GnmFuncDescriptor logical_functions[] = {
-	{ "and", NULL, N_("number,number,"), &help_and, NULL,
+	{ "and", NULL, N_("number,number,"), help_and, NULL,
 	  gnumeric_and, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE + GNM_FUNC_AUTO_UNITLESS,
 	  GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
-	{ "or", NULL, N_("number,number,"), &help_or, NULL,
+	{ "or", NULL, N_("number,number,"), help_or, NULL,
 	  gnumeric_or, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE + GNM_FUNC_AUTO_UNITLESS,
 	  GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
-	{ "not", "b", N_("number"), &help_not, gnumeric_not,
+	{ "not", "b", N_("number"), help_not, gnumeric_not,
 	  NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE + GNM_FUNC_AUTO_UNITLESS,
 	  GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
-	{ "if", "b|EE", N_("condition,if true,if false"), &help_if,
+	{ "if", "b|EE", N_("condition,if true,if false"), help_if,
 	  gnumeric_if, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE + GNM_FUNC_AUTO_SECOND,
 	  GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
-	{ "true", "", "", &help_true, gnumeric_true,
+	{ "true", "", "", help_true, gnumeric_true,
 	  NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE + GNM_FUNC_AUTO_UNITLESS,
 	  GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
-	{ "false", "", "", &help_false, gnumeric_false,
+	{ "false", "", "", help_false, gnumeric_false,
 	  NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE + GNM_FUNC_AUTO_UNITLESS,
 	  GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
-	{ "xor", NULL, N_("number,number,"), &help_xor, NULL,
+	{ "xor", NULL, N_("number,number,"), help_xor, NULL,
 	  gnumeric_xor, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE + GNM_FUNC_AUTO_UNITLESS,
 	  GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_BASIC },
