@@ -6,7 +6,7 @@
  * Authors:
  *   Miguel de Icaza (miguel@gnu.org)
  *   Jukka-Pekka Iivonen (iivonen@iki.fi)
- *   Morten Welinder (terra@diku.dk)
+ *   Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,29 +24,27 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
 #include <gnumeric.h>
-
 #include <func.h>
 #include <mathfunc.h>
 #include <rangefunc.h>
 #include <value.h>
-
 #include <expr.h>
 #include <sheet.h>
 #include <cell.h>
 #include <collect.h>
+#include <gnm-i18n.h>
 
-#include "plugin.h"
-#include "plugin-util.h"
-#include "module-plugin-defs.h"
+#include <goffice/app/go-plugin.h>
+#include <gnm-plugin.h>
 
-GNUMERIC_MODULE_PLUGIN_INFO_DECL;
+GNM_PLUGIN_MODULE_HEADER;
 
 /***************************************************************************/
 
-static char const *help_rand = {
-	N_("@FUNCTION=RAND\n"
+static GnmFuncHelp const help_rand[] = {
+	{ GNM_FUNC_HELP_OLD,
+	F_("@FUNCTION=RAND\n"
 	   "@SYNTAX=RAND()\n"
 
 	   "@DESCRIPTION="
@@ -58,18 +56,21 @@ static char const *help_rand = {
 	   "than one.\n"
 	   "\n"
 	   "@SEEALSO=RANDBETWEEN")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_rand (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_rand (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	return value_new_float (random_01 ());
 }
 
 /***************************************************************************/
 
-static char const *help_randuniform = {
-	N_("@FUNCTION=RANDUNIFORM\n"
+static GnmFuncHelp const help_randuniform[] = {
+	{ GNM_FUNC_HELP_OLD,
+	F_("@FUNCTION=RANDUNIFORM\n"
 	   "@SYNTAX=RANDUNIFORM(a,b)\n"
 
 	   "@DESCRIPTION="
@@ -84,10 +85,12 @@ static char const *help_randuniform = {
 	   "equal to 1.4 but less than 4.2.\n"
 	   "\n"
 	   "@SEEALSO=RANDBETWEEN,RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randuniform (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randuniform (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float a = value_get_as_float (argv[0]);
 	gnm_float b = value_get_as_float (argv[1]);
@@ -100,8 +103,9 @@ gnumeric_randuniform (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randdiscrete = {
-	N_("@FUNCTION=RANDDISCRETE\n"
+static GnmFuncHelp const help_randdiscrete[] = {
+	{ GNM_FUNC_HELP_OLD,
+	F_("@FUNCTION=RANDDISCRETE\n"
 	   "@SYNTAX=RANDDISCRETE(val_range[,prob_range])\n"
 
 	   "@DESCRIPTION="
@@ -121,6 +125,8 @@ static char const *help_randdiscrete = {
 	   "RANDDISCRETE(A1:A6) returns one of the values in the range A1:A6.\n"
 	   "\n"
 	   "@SEEALSO=RANDBETWEEN,RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 typedef struct {
@@ -162,13 +168,13 @@ cb_randdiscrete (Sheet *sheet, int col, int row, GnmCell *cell, void *user_data)
 }
 
 static GnmValue *
-gnumeric_randdiscrete (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randdiscrete (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-        GnmValue          *value_range = argv[0];
-	GnmValue          *prob_range  = argv[1];
-	GnmValue          *ret;
-	int            cols, rows;
-	randdiscrete_t rd;
+        GnmValue const *value_range = argv[0];
+	GnmValue const *prob_range  = argv[1];
+	GnmValue       *ret;
+	int             cols, rows;
+	randdiscrete_t  rd;
 
 	rd.prob  = NULL;
 	rd.ind   = 0;
@@ -232,8 +238,9 @@ gnumeric_randdiscrete (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randexp = {
-        N_("@FUNCTION=RANDEXP\n"
+static GnmFuncHelp const help_randexp[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDEXP\n"
            "@SYNTAX=RANDEXP(b)\n"
 
            "@DESCRIPTION="
@@ -243,10 +250,12 @@ static char const *help_randexp = {
            "RANDEXP(0.5).\n"
            "\n"
            "@SEEALSO=RAND,RANDBETWEEN")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randexp (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randexp (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float x = value_get_as_float (argv[0]);
 
@@ -255,8 +264,9 @@ gnumeric_randexp (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randpoisson = {
-        N_("@FUNCTION=RANDPOISSON\n"
+static GnmFuncHelp const help_randpoisson[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDPOISSON\n"
            "@SYNTAX=RANDPOISSON(lambda)\n"
 
            "@DESCRIPTION="
@@ -268,10 +278,12 @@ static char const *help_randpoisson = {
            "RANDPOISSON(3).\n"
            "\n"
            "@SEEALSO=RAND,RANDBETWEEN")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randpoisson (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randpoisson (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float x = value_get_as_float (argv[0]);
 
@@ -283,8 +295,9 @@ gnumeric_randpoisson (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randbinom = {
-        N_("@FUNCTION=RANDBINOM\n"
+static GnmFuncHelp const help_randbinom[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDBINOM\n"
            "@SYNTAX=RANDBINOM(p,trials)\n"
 
            "@DESCRIPTION="
@@ -297,31 +310,35 @@ static char const *help_randbinom = {
            "RANDBINOM(0.5,2).\n"
            "\n"
            "@SEEALSO=RAND,RANDBETWEEN")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randbinom (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randbinom (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float p      = value_get_as_float (argv[0]);
-	int        trials = value_get_as_int (argv[1]);
+	gnm_float trials = value_get_as_float (argv[1]);
 
 	if (p < 0 || p > 1 || trials < 0)
 		return value_new_error_NUM (ei->pos);
 
-        return value_new_float (random_binomial (p, trials));
+        return value_new_float (random_binomial (p, gnm_floor (trials)));
 }
 
 /***************************************************************************/
 
-static char const *help_randbetween = {
-	N_("@FUNCTION=RANDBETWEEN\n"
+static GnmFuncHelp const help_randbetween[] = {
+	{ GNM_FUNC_HELP_OLD,
+	F_("@FUNCTION=RANDBETWEEN\n"
 	   "@SYNTAX=RANDBETWEEN(bottom,top)\n"
 
 	   "@DESCRIPTION="
 	   "RANDBETWEEN function returns a random integer number "
 	   "between and including @bottom and @top.\n"
            "\n"
-	   "* If @bottom or @top is non-integer, they are truncated.\n"
+	   "* If @bottom is non-integer, it is rounded up.\n"
+	   "* If @top is non-integer, it is rounded down.\n"
 	   "* If @bottom > @top, RANDBETWEEN returns #NUM! error.\n"
 	   "* This function is Excel compatible.\n"
 	   "\n"
@@ -329,27 +346,31 @@ static char const *help_randbetween = {
 	   "RANDBETWEEN(3,7).\n"
 	   "\n"
 	   "@SEEALSO=RAND,RANDUNIFORM")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randbetween (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randbetween (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-        int bottom, top;
-	gnm_float r;
+	gnm_float bottom = value_get_as_float (argv[0]);
+	gnm_float top = value_get_as_float (argv[1]);
 
-	bottom = value_get_as_int (argv[0]);
-	top    = value_get_as_int (argv[1]);
 	if (bottom > top)
 		return value_new_error_NUM (ei->pos);
 
-	r = bottom + floorgnum ((top + 1.0 - bottom) * random_01 ());
-	return value_new_int ((int)r);
+	/* Bottom is ceiled, top floored.  Neither is truncated.  */
+	bottom = gnm_ceil (bottom);
+	top = gnm_floor (top);
+
+	return value_new_float (bottom + gnm_floor ((top + 1.0 - bottom) * random_01 ()));
 }
 
 /***************************************************************************/
 
-static char const *help_randnegbinom = {
-        N_("@FUNCTION=RANDNEGBINOM\n"
+static GnmFuncHelp const help_randnegbinom[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDNEGBINOM\n"
            "@SYNTAX=RANDNEGBINOM(p,failures)\n"
 
            "@DESCRIPTION="
@@ -363,24 +384,27 @@ static char const *help_randnegbinom = {
            "RANDNEGBINOM(0.5,2).\n"
            "\n"
            "@SEEALSO=RAND,RANDBETWEEN")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randnegbinom (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randnegbinom (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float p = value_get_as_float (argv[0]);
-	int failures = value_get_as_int (argv[1]);
+	gnm_float failures = value_get_as_float (argv[1]);
 
-	if (p < 0 || p > 1 || failures < 0)
+	if (p < 0 || p > 1 || failures < 1)
 		return value_new_error_NUM (ei->pos);
 
-        return value_new_float (random_negbinom (p, failures));
+        return value_new_float (random_negbinom (p, gnm_floor (failures)));
 }
 
 /***************************************************************************/
 
-static char const *help_randbernoulli = {
-        N_("@FUNCTION=RANDBERNOULLI\n"
+static GnmFuncHelp const help_randbernoulli[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDBERNOULLI\n"
            "@SYNTAX=RANDBERNOULLI(p)\n"
 
            "@DESCRIPTION="
@@ -392,10 +416,12 @@ static char const *help_randbernoulli = {
            "RANDBERNOULLI(0.5).\n"
            "\n"
            "@SEEALSO=RAND,RANDBETWEEN")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randbernoulli (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randbernoulli (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float p = value_get_as_float (argv[0]);
 
@@ -407,8 +433,9 @@ gnumeric_randbernoulli (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randnorm = {
-        N_("@FUNCTION=RANDNORM\n"
+static GnmFuncHelp const help_randnorm[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDNORM\n"
            "@SYNTAX=RANDNORM(mean,stdev)\n"
 
            "@DESCRIPTION="
@@ -420,10 +447,12 @@ static char const *help_randnorm = {
            "RANDNORM(0,1).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randnorm (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randnorm (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float mean  = value_get_as_float (argv[0]);
 	gnm_float stdev = value_get_as_float (argv[1]);
@@ -436,8 +465,9 @@ gnumeric_randnorm (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randcauchy = {
-        N_("@FUNCTION=RANDCAUCHY\n"
+static GnmFuncHelp const help_randcauchy[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDCAUCHY\n"
            "@SYNTAX=RANDCAUCHY(a)\n"
 
            "@DESCRIPTION="
@@ -451,10 +481,12 @@ static char const *help_randcauchy = {
            "RANDCAUCHY(1).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randcauchy (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randcauchy (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float a = value_get_as_float (argv[0]);
 
@@ -466,8 +498,9 @@ gnumeric_randcauchy (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randlognorm = {
-        N_("@FUNCTION=RANDLOGNORM\n"
+static GnmFuncHelp const help_randlognorm[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDLOGNORM\n"
            "@SYNTAX=RANDLOGNORM(zeta,sigma)\n"
 
            "@DESCRIPTION="
@@ -477,10 +510,12 @@ static char const *help_randlognorm = {
            "RANDLOGNORM(1,2).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randlognorm (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randlognorm (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float zeta  = value_get_as_float (argv[0]);
 	gnm_float sigma = value_get_as_float (argv[1]);
@@ -490,8 +525,9 @@ gnumeric_randlognorm (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randweibull = {
-        N_("@FUNCTION=RANDWEIBULL\n"
+static GnmFuncHelp const help_randweibull[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDWEIBULL\n"
            "@SYNTAX=RANDWEIBULL(a,b)\n"
 
            "@DESCRIPTION="
@@ -501,10 +537,12 @@ static char const *help_randweibull = {
            "RANDWEIBULL(1,2).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randweibull (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randweibull (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float a = value_get_as_float (argv[0]);
 	gnm_float b = value_get_as_float (argv[1]);
@@ -514,8 +552,9 @@ gnumeric_randweibull (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randlaplace = {
-        N_("@FUNCTION=RANDLAPLACE\n"
+static GnmFuncHelp const help_randlaplace[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDLAPLACE\n"
            "@SYNTAX=RANDLAPLACE(a)\n"
 
            "@DESCRIPTION="
@@ -527,10 +566,12 @@ static char const *help_randlaplace = {
            "RANDLAPLACE(1).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randlaplace (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randlaplace (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float a = value_get_as_float (argv[0]);
 
@@ -539,8 +580,9 @@ gnumeric_randlaplace (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randrayleigh = {
-        N_("@FUNCTION=RANDRAYLEIGH\n"
+static GnmFuncHelp const help_randrayleigh[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDRAYLEIGH\n"
            "@SYNTAX=RANDRAYLEIGH(sigma)\n"
 
            "@DESCRIPTION="
@@ -550,10 +592,12 @@ static char const *help_randrayleigh = {
            "RANDRAYLEIGH(1).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randrayleigh (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randrayleigh (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float sigma = value_get_as_float (argv[0]);
 
@@ -562,8 +606,9 @@ gnumeric_randrayleigh (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randrayleightail = {
-        N_("@FUNCTION=RANDRAYLEIGHTAIL\n"
+static GnmFuncHelp const help_randrayleightail[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDRAYLEIGHTAIL\n"
            "@SYNTAX=RANDRAYLEIGHTAIL(a,sigma)\n"
 
            "@DESCRIPTION="
@@ -577,10 +622,12 @@ static char const *help_randrayleightail = {
            "RANDRAYLEIGHTAIL(0.3,1).\n"
            "\n"
            "@SEEALSO=RAND,RANDRAYLEIGH")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randrayleightail (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randrayleightail (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float a     = value_get_as_float (argv[0]);
 	gnm_float sigma = value_get_as_float (argv[1]);
@@ -590,8 +637,9 @@ gnumeric_randrayleightail (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randgamma = {
-        N_("@FUNCTION=RANDGAMMA\n"
+static GnmFuncHelp const help_randgamma[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDGAMMA\n"
            "@SYNTAX=RANDGAMMA(a,b)\n"
 
            "@DESCRIPTION="
@@ -603,10 +651,12 @@ static char const *help_randgamma = {
            "RANDGAMMA(1,2).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randgamma (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randgamma (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float a = value_get_as_float (argv[0]);
 	gnm_float b = value_get_as_float (argv[1]);
@@ -619,8 +669,9 @@ gnumeric_randgamma (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randpareto = {
-        N_("@FUNCTION=RANDPARETO\n"
+static GnmFuncHelp const help_randpareto[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDPARETO\n"
            "@SYNTAX=RANDPARETO(a,b)\n"
 
            "@DESCRIPTION="
@@ -630,10 +681,12 @@ static char const *help_randpareto = {
            "RANDPARETO(1,2).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randpareto (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randpareto (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float a = value_get_as_float (argv[0]);
 	gnm_float b = value_get_as_float (argv[1]);
@@ -643,8 +696,9 @@ gnumeric_randpareto (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randfdist = {
-        N_("@FUNCTION=RANDFDIST\n"
+static GnmFuncHelp const help_randfdist[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDFDIST\n"
            "@SYNTAX=RANDFDIST(nu1,nu2)\n"
 
            "@DESCRIPTION="
@@ -654,10 +708,12 @@ static char const *help_randfdist = {
            "RANDFDIST(1,2).\n"
            "\n"
            "@SEEALSO=RAND,RANDGAMMA")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randfdist (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randfdist (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float nu1 = value_get_as_float (argv[0]);
 	gnm_float nu2 = value_get_as_float (argv[1]);
@@ -667,8 +723,9 @@ gnumeric_randfdist (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randbeta = {
-        N_("@FUNCTION=RANDBETA\n"
+static GnmFuncHelp const help_randbeta[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDBETA\n"
            "@SYNTAX=RANDBETA(a,b)\n"
 
            "@DESCRIPTION="
@@ -678,10 +735,12 @@ static char const *help_randbeta = {
            "RANDBETA(1,2).\n"
            "\n"
            "@SEEALSO=RAND,RANDGAMMA")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randbeta (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randbeta (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float a = value_get_as_float (argv[0]);
 	gnm_float b = value_get_as_float (argv[1]);
@@ -691,8 +750,9 @@ gnumeric_randbeta (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randlogistic = {
-        N_("@FUNCTION=RANDLOGISTIC\n"
+static GnmFuncHelp const help_randlogistic[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDLOGISTIC\n"
            "@SYNTAX=RANDLOGISTIC(a)\n"
 
            "@DESCRIPTION="
@@ -705,10 +765,12 @@ static char const *help_randlogistic = {
            "RANDLOGISTIC(1).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randlogistic (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randlogistic (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float a = value_get_as_float (argv[0]);
 
@@ -717,8 +779,9 @@ gnumeric_randlogistic (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randgeom = {
-        N_("@FUNCTION=RANDGEOM\n"
+static GnmFuncHelp const help_randgeom[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDGEOM\n"
            "@SYNTAX=RANDGEOM(p)\n"
 
            "@DESCRIPTION="
@@ -733,23 +796,26 @@ static char const *help_randgeom = {
            "RANDGEOM(0.4).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randgeom (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randgeom (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float p = value_get_as_float (argv[0]);
 
 	if (p < 0 || p > 1)
 		return value_new_error_NUM (ei->pos);
 
-        return value_new_int (random_geometric (p));
+        return value_new_float (random_geometric (p));
 }
 
 /***************************************************************************/
 
-static char const *help_randhyperg = {
-        N_("@FUNCTION=RANDHYPERG\n"
+static GnmFuncHelp const help_randhyperg[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDHYPERG\n"
            "@SYNTAX=RANDHYPERG(n1,n2,t)\n"
 
            "@DESCRIPTION="
@@ -764,22 +830,27 @@ static char const *help_randhyperg = {
            "RANDHYPERG(21,1,9).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randhyperg (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randhyperg (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	unsigned int n1 = value_get_as_int (argv[0]);
-	unsigned int n2 = value_get_as_int (argv[1]);
-	unsigned int t = value_get_as_int (argv[2]);
+	gnm_float n1 = value_get_as_float (argv[0]);
+	gnm_float n2 = value_get_as_float (argv[1]);
+	gnm_float t = value_get_as_float (argv[2]);
 
-        return value_new_int (random_hypergeometric (n1, n2, t));
+        return value_new_float (random_hypergeometric (gnm_floor (n1),
+						       gnm_floor (n2),
+						       gnm_floor (t)));
 }
 
 /***************************************************************************/
 
-static char const *help_randlog = {
-        N_("@FUNCTION=RANDLOG\n"
+static GnmFuncHelp const help_randlog[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDLOG\n"
            "@SYNTAX=RANDLOG(p)\n"
 
            "@DESCRIPTION="
@@ -791,23 +862,26 @@ static char const *help_randlog = {
            "RANDLOG(0.72).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randlog (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randlog (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float p = value_get_as_float (argv[0]);
 
 	if (p < 0 || p > 1)
 		return value_new_error_NUM (ei->pos);
 
-        return value_new_int (random_logarithmic (p));
+        return value_new_float (random_logarithmic (p));
 }
 
 /***************************************************************************/
 
-static char const *help_randchisq = {
-        N_("@FUNCTION=RANDCHISQ\n"
+static GnmFuncHelp const help_randchisq[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDCHISQ\n"
            "@SYNTAX=RANDCHISQ(nu)\n"
 
            "@DESCRIPTION="
@@ -817,10 +891,12 @@ static char const *help_randchisq = {
            "RANDCHISQ(0.5).\n"
            "\n"
            "@SEEALSO=RAND,RANDGAMMA")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randchisq (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randchisq (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float nu = value_get_as_float (argv[0]);
 
@@ -829,8 +905,9 @@ gnumeric_randchisq (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randtdist = {
-        N_("@FUNCTION=RANDTDIST\n"
+static GnmFuncHelp const help_randtdist[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDTDIST\n"
            "@SYNTAX=RANDTDIST(nu)\n"
 
            "@DESCRIPTION="
@@ -840,10 +917,12 @@ static char const *help_randtdist = {
            "RANDTDIST(0.5).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randtdist (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randtdist (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float nu = value_get_as_float (argv[0]);
 
@@ -852,8 +931,9 @@ gnumeric_randtdist (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randgumbel = {
-        N_("@FUNCTION=RANDGUMBEL\n"
+static GnmFuncHelp const help_randgumbel[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDGUMBEL\n"
            "@SYNTAX=RANDGUMBEL(a,b[,type])\n"
 
            "@DESCRIPTION="
@@ -868,28 +948,30 @@ static char const *help_randgumbel = {
            "RANDGUMBEL(0.5,1,2).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randgumbel (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randgumbel (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float a = value_get_as_float (argv[0]);
 	gnm_float b = value_get_as_float (argv[1]);
-	int type     = (argv[2] == NULL) ? 1 : value_get_as_int (argv[2]);
-
-	if (type != 1 && type != 2)
-		return value_new_error_NUM (ei->pos);
+	gnm_float type = argv[2] ? value_get_as_float (argv[2]) : 1;
 
 	if (type == 1)
 		return value_new_float (random_gumbel1 (a, b));
-	else
+	else if (type == 2)
 		return value_new_float (random_gumbel2 (a, b));
+	else
+		return value_new_error_NUM (ei->pos);
 }
 
 /***************************************************************************/
 
-static char const *help_randlevy = {
-        N_("@FUNCTION=RANDLEVY\n"
+static GnmFuncHelp const help_randlevy[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDLEVY\n"
            "@SYNTAX=RANDLEVY(c,alpha[,beta])\n"
 
            "@DESCRIPTION="
@@ -906,10 +988,12 @@ static char const *help_randlevy = {
            "RANDLEVY(0.5,0.1,1).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randlevy (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randlevy (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float c     = value_get_as_float (argv[0]);
 	gnm_float alpha = value_get_as_float (argv[1]);
@@ -923,8 +1007,9 @@ gnumeric_randlevy (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randexppow = {
-        N_("@FUNCTION=RANDEXPPOW\n"
+static GnmFuncHelp const help_randexppow[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDEXPPOW\n"
            "@SYNTAX=RANDEXPPOW(a,b)\n"
 
            "@DESCRIPTION="
@@ -941,10 +1026,12 @@ static char const *help_randexppow = {
            "RANDEXPPOW(0.5,0.1).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randexppow (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randexppow (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float a = value_get_as_float (argv[0]);
 	gnm_float b = value_get_as_float (argv[1]);
@@ -954,8 +1041,9 @@ gnumeric_randexppow (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_randlandau = {
-        N_("@FUNCTION=RANDLANDAU\n"
+static GnmFuncHelp const help_randlandau[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDLANDAU\n"
            "@SYNTAX=RANDLANDAU()\n"
 
            "@DESCRIPTION="
@@ -972,18 +1060,21 @@ static char const *help_randlandau = {
            "RANDLANDAU().\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randlandau (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randlandau (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	return value_new_float (random_landau ());
 }
 
 /***************************************************************************/
 
-static char const *help_randnormtail = {
-        N_("@FUNCTION=RANDNORMTAIL\n"
+static GnmFuncHelp const help_randnormtail[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=RANDNORMTAIL\n"
            "@SYNTAX=RANDNORMTAIL(a,sigma)\n"
 
            "@DESCRIPTION="
@@ -1005,10 +1096,12 @@ static char const *help_randnormtail = {
            "RANDNORMTAIL(0.5,0.1).\n"
            "\n"
            "@SEEALSO=RAND")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
-gnumeric_randnormtail (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_randnormtail (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float a     = value_get_as_float (argv[0]);
 	gnm_float sigma = value_get_as_float (argv[1]);
@@ -1018,8 +1111,9 @@ gnumeric_randnormtail (FunctionEvalInfo *ei, GnmValue **argv)
 
 /***************************************************************************/
 
-static char const *help_simtable = {
-        N_("@FUNCTION=SIMTABLE\n"
+static GnmFuncHelp const help_simtable[] = {
+	{ GNM_FUNC_HELP_OLD,
+        F_("@FUNCTION=SIMTABLE\n"
            "@SYNTAX=SIMTABLE(d1, d2, ..., dN)\n"
 
            "@DESCRIPTION="
@@ -1053,6 +1147,8 @@ static char const *help_simtable = {
            "#3.\n"
            "\n"
            "@SEEALSO=")
+	},
+	{ GNM_FUNC_HELP_END }
 };
 
 typedef struct {
@@ -1061,7 +1157,7 @@ typedef struct {
 } simtable_t;
 
 static GnmValue *
-callback_function_simtable (const GnmEvalPos *ep, GnmValue *value, void *closure)
+callback_function_simtable (GnmEvalPos const *ep, GnmValue const *value, void *closure)
 {
 	simtable_t *p = closure;
 
@@ -1073,7 +1169,7 @@ callback_function_simtable (const GnmEvalPos *ep, GnmValue *value, void *closure
 }
 
 static GnmValue *
-gnumeric_simtable (FunctionEvalInfo *ei, GnmExprList *nodes)
+gnumeric_simtable (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
 	simtable_t p;
 
@@ -1081,8 +1177,8 @@ gnumeric_simtable (FunctionEvalInfo *ei, GnmExprList *nodes)
 	p.value = NULL;
 
 	function_iterate_argument_values
-		(ei->pos, callback_function_simtable, &p, nodes,
-		 FALSE, CELL_ITER_IGNORE_BLANK);
+		(ei->pos, callback_function_simtable, &p,
+		 argc, argv, FALSE, CELL_ITER_IGNORE_BLANK);
 
 	/* See if there was any value worth using. */
 	if (p.value == NULL)
@@ -1094,100 +1190,100 @@ gnumeric_simtable (FunctionEvalInfo *ei, GnmExprList *nodes)
 /***************************************************************************/
 
 const GnmFuncDescriptor random_functions[] = {
-	{ "rand",    "", "",           &help_rand,
+	{ "rand",    "", "",           help_rand,
 	  gnumeric_rand, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
-        { "randbernoulli", "f", N_("p"),   &help_randbernoulli,
+        { "randbernoulli", "f", N_("p"),   help_randbernoulli,
 	  gnumeric_randbernoulli, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randbeta", "ff", N_("a,b"),   &help_randbeta,
+        { "randbeta", "ff", N_("a,b"),   help_randbeta,
 	  gnumeric_randbeta, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randbetween", "ff", N_("bottom,top"), &help_randbetween,
+        { "randbetween", "ff", N_("bottom,top"), help_randbetween,
 	  gnumeric_randbetween, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
-        { "randbinom", "ff", N_("p,trials"), &help_randbinom,
+        { "randbinom", "ff", N_("p,trials"), help_randbinom,
 	  gnumeric_randbinom, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randcauchy", "f", N_("a"),   &help_randcauchy,
+        { "randcauchy", "f", N_("a"),   help_randcauchy,
 	  gnumeric_randcauchy, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randchisq", "f", N_("nu"),   &help_randchisq,
+        { "randchisq", "f", N_("nu"),   help_randchisq,
 	  gnumeric_randchisq, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
  	{ "randdiscrete", "r|r", N_("value_range,prob_range"),
-	  &help_randdiscrete, gnumeric_randdiscrete, NULL, NULL, NULL, NULL,
+	  help_randdiscrete, gnumeric_randdiscrete, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-	{ "randexp", "f", N_("b"),         &help_randexp,
+	{ "randexp", "f", N_("b"),         help_randexp,
 	  gnumeric_randexp, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randexppow", "ff", N_("a,b"),         &help_randexppow,
+        { "randexppow", "ff", N_("a,b"),         help_randexppow,
 	  gnumeric_randexppow, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randfdist", "ff", N_("nu1,nu2"),      &help_randfdist,
+        { "randfdist", "ff", N_("nu1,nu2"),      help_randfdist,
 	  gnumeric_randfdist, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randgamma", "ff", N_("a,b"),    &help_randgamma,
+        { "randgamma", "ff", N_("a,b"),    help_randgamma,
 	  gnumeric_randgamma, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randnormtail", "ff", N_("a,sigma"),    &help_randnormtail,
+        { "randnormtail", "ff", N_("a,sigma"),    help_randnormtail,
 	  gnumeric_randnormtail, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randgeom", "f", N_("p"),    &help_randgeom,
+        { "randgeom", "f", N_("p"),    help_randgeom,
 	  gnumeric_randgeom, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randgumbel", "ff|f", N_("a,b,type"),    &help_randgumbel,
+        { "randgumbel", "ff|f", N_("a,b,type"),    help_randgumbel,
 	  gnumeric_randgumbel, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randhyperg", "fff", N_("n1,n2,t"),    &help_randhyperg,
+        { "randhyperg", "fff", N_("n1,n2,t"),    help_randhyperg,
 	  gnumeric_randhyperg, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randlandau", "", "", &help_randlandau,
+        { "randlandau", "", "", help_randlandau,
 	  gnumeric_randlandau, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randlaplace", "f", N_("a"), &help_randlaplace,
+        { "randlaplace", "f", N_("a"), help_randlaplace,
 	  gnumeric_randlaplace, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randlevy", "ff|f", N_("c,alpha,beta"), &help_randlevy,
+        { "randlevy", "ff|f", N_("c,alpha,beta"), help_randlevy,
 	  gnumeric_randlevy, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randlog", "f", N_("p"), &help_randlog,
+        { "randlog", "f", N_("p"), help_randlog,
 	  gnumeric_randlog, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randlogistic", "f", N_("a"), &help_randlogistic,
+        { "randlogistic", "f", N_("a"), help_randlogistic,
 	  gnumeric_randlogistic, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randlognorm", "ff", N_("zeta,sigma"), &help_randlognorm,
+        { "randlognorm", "ff", N_("zeta,sigma"), help_randlognorm,
 	  gnumeric_randlognorm, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randnegbinom", "ff", N_("p,failures"), &help_randnegbinom,
+        { "randnegbinom", "ff", N_("p,failures"), help_randnegbinom,
 	  gnumeric_randnegbinom, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randnorm", "ff", N_("mean,stdev"), &help_randnorm,
+        { "randnorm", "ff", N_("mean,stdev"), help_randnorm,
 	  gnumeric_randnorm, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randpareto", "ff", N_("a,b"), &help_randpareto,
+        { "randpareto", "ff", N_("a,b"), help_randpareto,
 	  gnumeric_randpareto, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randpoisson", "f", N_("lambda"), &help_randpoisson,
+        { "randpoisson", "f", N_("lambda"), help_randpoisson,
 	  gnumeric_randpoisson, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randrayleigh", "f", N_("sigma"), &help_randrayleigh,
+        { "randrayleigh", "f", N_("sigma"), help_randrayleigh,
 	  gnumeric_randrayleigh, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randrayleightail", "ff", N_("a,sigma"), &help_randrayleightail,
+        { "randrayleightail", "ff", N_("a,sigma"), help_randrayleightail,
 	  gnumeric_randrayleightail, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randtdist", "f", N_("nu"), &help_randtdist,
+        { "randtdist", "f", N_("nu"), help_randtdist,
 	  gnumeric_randtdist, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randuniform", "ff", N_("a,b"), &help_randuniform,
+        { "randuniform", "ff", N_("a,b"), help_randuniform,
 	  gnumeric_randuniform, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-        { "randweibull", "ff", N_("a,b"), &help_randweibull,
+        { "randweibull", "ff", N_("a,b"), help_randweibull,
 	  gnumeric_randweibull, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
-	{ "simtable", 0, N_("d1,d2,...,dN"), &help_simtable,
+	{ "simtable", NULL, N_("d1,d2,...,dN"), help_simtable,
 	  NULL,	gnumeric_simtable, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
 
