@@ -46,6 +46,7 @@
 #include <ranges.h>
 #include <goffice/app/io-context.h>
 #include <goffice/app/error-info.h>
+#include <goffice/utils/go-glib-extras.h>
 
 #include <gsf/gsf-input.h>
 #include <libxml/HTMLparser.h>
@@ -73,7 +74,7 @@ html_get_sheet (char const *name, Workbook *wb)
 			workbook_sheet_attach (wb, sheet);
 		}
 	} else
-		sheet = workbook_sheet_add (wb, -1, FALSE);
+		sheet = workbook_sheet_add (wb, -1);
 	return sheet;
 }
 
@@ -83,10 +84,12 @@ html_append_text (GString *buf, const xmlChar *text)
 	const xmlChar *p;
 	
 	while (*text) {
-		while (g_ascii_isspace (*text))
-			text++;
+		while (g_unichar_isspace (g_utf8_get_char (text)))
+			text = g_utf8_next_char (text);
 		if (*text) {
-			for (p = text; *p && !g_ascii_isspace (*p); p++)
+			for (p = text; 
+			     *p && !g_unichar_isspace (g_utf8_get_char (p)); 
+			     p =  g_utf8_next_char (p))
 				;
 			if (buf->len > 0)
 				g_string_append_c (buf, ' ');
@@ -300,7 +303,7 @@ html_read_table (htmlNodePtr cur, htmlDocPtr doc, WorkbookView *wb_view,
 	g_return_if_fail (cur != NULL);
 	g_return_if_fail (wb_view != NULL);
 
-	wb = wb_view_workbook (wb_view);
+	wb = wb_view_get_workbook (wb_view);
 	for (ptr = cur->children; ptr != NULL ; ptr = ptr->next) {
 		if (ptr->type != XML_ELEMENT_NODE)
 			continue;
@@ -474,6 +477,9 @@ html_file_open (GOFileOpener const *fo, IOContext *io_context,
 
 	g_return_if_fail (input != NULL);
 
+	if (gsf_input_seek (input, 0, G_SEEK_SET))
+		return;
+
 	size = gsf_input_size (input);
 	if (size >= 4) {
 		size -= 4;
@@ -541,3 +547,49 @@ html_file_open (GOFileOpener const *fo, IOContext *io_context,
 		gnumeric_io_error_info_set (io_context,
 			error_info_new_str (_("Unable to parse the html.")));
 }
+
+/* Quick and dirty html probe. */
+gboolean 
+html_file_probe (GOFileOpener const *fo, GsfInput *input, FileProbeLevel pl)
+{
+	gsf_off_t size = 200;
+	guint8 const* buf = gsf_input_read (input, size, NULL);
+	gchar *ulstr = NULL;
+	gboolean res = FALSE;
+	int try;
+
+	/* Avoid seeking in large streams - try to read, fall back if 
+	 * stream is too short.  (Actually, currently _size does not
+	 * involve any syscalls -- MW).  */
+	if (!buf) {
+		size = gsf_input_size (input);
+		buf = gsf_input_read (input, size, NULL);
+		if (!buf)
+			return res;
+	}
+
+	/*
+	 * It is conceivable that encoding guessing could fail
+	 * if our truncated buffer had partial characters.  We
+	 * really need go_guess_encoding_truncated, but for now
+	 * let's just try cutting a byte away at a time.
+	 */
+	for (try = 0; try < MIN (size, 6); try++) {
+		char *ustr;
+		if (go_guess_encoding (buf, size - try, NULL, &ustr)) {
+			ulstr = g_utf8_strdown (ustr, -1);
+			g_free (ustr);
+			break;
+		}
+	}
+	if (!ulstr)
+		return res;
+
+	res = (strstr (ulstr, "<table") != NULL ||
+	       strstr (ulstr, "<html") != NULL ||
+	       strstr (ulstr, "<!doctype html") != NULL);
+
+	g_free (ulstr);
+
+	return res;
+}
