@@ -54,8 +54,8 @@
 #include <style-border.h>
 #include <sheet-style.h>
 #include <parse-util.h>
-#include <cellspan.h>
 #include <rendered-value.h>
+#include <cellspan.h>
 
 #include <locale.h>
 #include <gsf/gsf-output.h>
@@ -207,6 +207,53 @@ latex_math_fputs_utf (char const *p, GsfOutput *output)
 	}
 }
 
+/**
+ * latex_convert_latin_to_utf
+ *
+ * @text: string to convert
+ *
+ * return value needs to be freed with g_free
+ *
+ * call g_convert_with_fallback and also handle utf minus.
+ *
+ */
+static char *
+latex_convert_latin_to_utf (char const *text)
+{
+	char * encoded_text = NULL;
+
+	gsize bytes_read;
+	gsize bytes_written;
+
+	if (g_utf8_strchr (text,-1, 0x2212) == NULL) {
+		encoded_text = g_convert_with_fallback
+			(text, strlen (text),
+			 "ISO-8859-1", "UTF-8", (gchar *)"?", 
+			 &bytes_read, &bytes_written, NULL);
+	} else {
+		gunichar* ucs_string = NULL;
+		gunichar* this_unichar;
+		char *new_text;
+		glong items_read;
+		glong items_written;
+
+		ucs_string = g_utf8_to_ucs4_fast (text, -1, &items_written);
+		for (this_unichar = ucs_string; *this_unichar != '\0'; this_unichar++) {
+			if (*this_unichar == 0x2212)
+				*this_unichar = 0x002d;
+		}
+		new_text = g_ucs4_to_utf8 (ucs_string, -1, &items_read, &items_written, NULL);
+		g_free (ucs_string);
+		encoded_text = g_convert_with_fallback
+			(new_text, strlen (new_text),
+			 "ISO-8859-1", "UTF-8", (gchar *)"?", 
+			 &bytes_read, &bytes_written, NULL);
+		g_free (new_text);
+	}
+
+	return encoded_text;
+}
+
 /**
  * latex_fputs_latin :
  *
@@ -221,19 +268,8 @@ latex_fputs_latin (char const *text, GsfOutput *output)
 {
 	char * encoded_text = NULL;
 	char * p;
-	gsize bytes_read;
-	gsize bytes_written;
-	GError * error = NULL;
 
-	encoded_text = g_convert_with_fallback
-		(text, strlen (text),
-		 "ISO-8859-1", "UTF-8", (gchar *)"?", 
-		 &bytes_read, &bytes_written, &error);
-	if (error != NULL)
-	{
-		g_warning ("UTF-8 to Latin1 conversion failed for:\n%s", text);
-		g_error_free (error);
-	}
+	encoded_text = latex_convert_latin_to_utf (text);
 
 	for (p = encoded_text; *p; p++) {
 		switch (*p) {
@@ -278,19 +314,8 @@ latex_math_fputs_latin (char const *text, GsfOutput *output)
 {
 	char * encoded_text = NULL;
 	char * p;
-	gsize bytes_read;
-	gsize bytes_written;
-	GError * error = NULL;
 
-	encoded_text = g_convert_with_fallback
-		(text, strlen (text),
-		 "ISO-8859-1", "UTF-8", (gchar *)"?", 
-		 &bytes_read, &bytes_written, &error);
-	if (error != NULL)
-	{
-		g_warning ("UTF-8 to Latin1 conversion failed for:\n%s", text);
-		g_error_free (error);
-	}
+	encoded_text = latex_convert_latin_to_utf (text);
 
 	for (p = encoded_text; *p; p++) {
 		switch (*p) {
@@ -524,7 +549,7 @@ latex2e_write_table_header(GsfOutput *output, int num_cols)
 
 	gsf_output_printf (output, "%%\t\\multicolumn{%d}{c}{The First Header} \\\\ \\hline \n", num_cols);
 	gsf_output_printf (output, "%%\t\\multicolumn{1}{c}{colTag}\t%%Column 1\n");
-	for (col = 1 ; col < num_cols; col++)
+	for (col = 2 ; col < num_cols; col++)
 		gsf_output_printf (output, "%%\t&\\multicolumn{1}{c}{colTag}\t%%Column %d\n",col);
 	gsf_output_printf (output, "%%\t&\\multicolumn{1}{c}{colTag}\t\\\\ \\hline %%Last column\n");
 	gsf_output_printf (output, "%%\t\\endfirsthead\n\n");
@@ -533,7 +558,7 @@ latex2e_write_table_header(GsfOutput *output, int num_cols)
 	gsf_output_printf (output, "%%\t\\hline\n");
 	gsf_output_printf (output, "%%\t\\multicolumn{%d}{l}{\\ldots\\small\\slshape continued} \\\\ \\hline\n", num_cols);
 	gsf_output_printf (output, "%%\t\\multicolumn{1}{c}{colTag}\t%%Column 1\n");
-	for (col = 1 ; col < num_cols; col++)
+	for (col = 2 ; col < num_cols; col++)
 				gsf_output_printf (output, "%%\t&\\multicolumn{1}{c}{colTag}\t%%Column %d\n",col);
 	gsf_output_printf (output, "%%\t&\\multicolumn{1}{c}{colTag}\t\\\\ \\hline %%Last column\n");
 	gsf_output_printf (output, "%%\t\\endhead\n\n");
@@ -565,14 +590,14 @@ latex2e_write_table_header(GsfOutput *output, int num_cols)
 static StyleBorderType
 latex2e_find_vline (int col, int row, Sheet *sheet, GnmStyleElement which_border)
 {
-	GnmBorder	   *border;
-	GnmStyle             *mstyle;
+	GnmBorder const	*border;
+	GnmStyle const	*style;
 
 	if (col < 0 || row < 0)
 		return STYLE_BORDER_NONE;
 
-	mstyle = sheet_style_get (sheet, col, row);
-	border = gnm_style_get_border (mstyle, which_border);
+	style = sheet_style_get (sheet, col, row);
+	border = gnm_style_get_border (style, which_border);
 
 	if (!(style_border_is_blank (border) ||
 	      border->line_type == STYLE_BORDER_NONE))
@@ -581,13 +606,13 @@ latex2e_find_vline (int col, int row, Sheet *sheet, GnmStyleElement which_border
 	if (which_border == MSTYLE_BORDER_LEFT) {
 		if (col < 1)
 			return STYLE_BORDER_NONE;
-		mstyle = sheet_style_get (sheet, col - 1, row);
-		border = gnm_style_get_border (mstyle, MSTYLE_BORDER_RIGHT);
+		style = sheet_style_get (sheet, col - 1, row);
+		border = gnm_style_get_border (style, MSTYLE_BORDER_RIGHT);
 		return ((style_border_is_blank (border)) ? STYLE_BORDER_NONE :
 			border->line_type);
 	} else {
-		mstyle = sheet_style_get (sheet, col + 1, row);
-		border = gnm_style_get_border (mstyle, MSTYLE_BORDER_LEFT);
+		style = sheet_style_get (sheet, col + 1, row);
+		border = gnm_style_get_border (style, MSTYLE_BORDER_LEFT);
 		return ((style_border_is_blank (border)) ? STYLE_BORDER_NONE :
 			border->line_type);
 	}
@@ -686,16 +711,16 @@ latex2e_write_multicolumn_cell (GsfOutput *output, GnmCell *cell, int start_col,
 	char * rendered_string;
 	gushort r,g,b;
 	gboolean wrap = FALSE;
-	FormatFamily cell_format_family;
+	GOFormatFamily cell_format_family;
 	int merge_width = 0;
 	StyleBorderType left_border = STYLE_BORDER_NONE;
 	StyleBorderType right_border = STYLE_BORDER_NONE;
 
 	/* Print the cell according to its style. */
-	GnmStyle *mstyle = cell_get_mstyle (cell);
-	gboolean hidden = gnm_style_get_content_hidden (mstyle);
+	GnmStyle const *style = cell_get_style (cell);
+	gboolean hidden = gnm_style_get_contents_hidden (style);
 
-	g_return_if_fail (mstyle != NULL);
+	g_return_if_fail (style != NULL);
 
 	if (num_merged_cols > 1 || num_merged_rows > 1) {
 		ColRowInfo const * ci;
@@ -776,10 +801,11 @@ latex2e_write_multicolumn_cell (GsfOutput *output, GnmCell *cell, int start_col,
 
 	/* Send the alignment of the cell through a routine to deal with
 	 * HALIGN_GENERAL and then deal with the three cases. */
-	switch ( style_default_halign(mstyle, cell) ) {
+	switch ( style_default_halign(style, cell) ) {
 	case HALIGN_RIGHT:
 		gsf_output_printf (output, "\\gnumericPB{\\raggedleft}");
 		break;
+	case HALIGN_DISTRIBUTED:
 	case HALIGN_CENTER:
 	case HALIGN_CENTER_ACROSS_SELECTION:
 		gsf_output_printf (output, "\\gnumericPB{\\centering}");
@@ -794,15 +820,16 @@ latex2e_write_multicolumn_cell (GsfOutput *output, GnmCell *cell, int start_col,
 	}
 
         /* Check whether we should do word wrapping */
-	wrap = gnm_style_get_wrap_text (mstyle);
+	wrap = gnm_style_get_wrap_text (style);
 
 	/* if we don't wrap put it into an mbox, adjusted to width 0 to avoid moving */
 	/* it to the second line of the parbox */
 	if (!wrap)
-		switch ( style_default_halign(mstyle, cell) ) {
+		switch ( style_default_halign(style, cell) ) {
 		case HALIGN_RIGHT:
 			gsf_output_printf (output, "\\gnumbox[r]{");
 			break;
+		case HALIGN_DISTRIBUTED:
 		case HALIGN_CENTER:
 		case HALIGN_CENTER_ACROSS_SELECTION:
 			gsf_output_printf (output, "\\gnumbox{");
@@ -843,22 +870,24 @@ latex2e_write_multicolumn_cell (GsfOutput *output, GnmCell *cell, int start_col,
 		if (hidden)
 		  gsf_output_printf (output, "\\phantom{");
 
-		if (font_is_monospaced (mstyle))
+		if (font_is_monospaced (style))
 			gsf_output_printf (output, "\\texttt{");
-		else if (font_is_sansserif (mstyle))
+		else if (font_is_sansserif (style))
 			gsf_output_printf (output, "\\textsf{");
-		if (gnm_style_get_font_bold (mstyle))
+		if (gnm_style_get_font_bold (style))
 			gsf_output_printf (output, "\\textbf{");
-		if (gnm_style_get_font_italic (mstyle))
+		if (gnm_style_get_font_italic (style))
 			gsf_output_printf (output, "\\textit{");
 
 
 		cell_format_family = cell_get_format (cell)->family;
-		if (cell_format_family == FMT_NUMBER || cell_format_family == FMT_CURRENCY ||
-		    cell_format_family == FMT_PERCENT || cell_format_family == FMT_FRACTION ||
-		    cell_format_family == FMT_SCIENCE){
+		if (cell_format_family == GO_FORMAT_NUMBER ||
+		    cell_format_family == GO_FORMAT_CURRENCY ||
+		    cell_format_family == GO_FORMAT_PERCENTAGE ||
+		    cell_format_family == GO_FORMAT_FRACTION ||
+		    cell_format_family == GO_FORMAT_SCIENTIFIC){
 			gsf_output_printf (output, "$");
-		        if (gnm_style_get_font_italic(mstyle))
+		        if (gnm_style_get_font_italic(style))
 			    gsf_output_printf (output, "\\gnumericmathit{");
 
 			/* Print the cell contents. */
@@ -866,7 +895,7 @@ latex2e_write_multicolumn_cell (GsfOutput *output, GnmCell *cell, int start_col,
 			latex_math_fputs (rendered_string, output);
 			g_free (rendered_string);
 
-		        if (gnm_style_get_font_italic(mstyle))
+		        if (gnm_style_get_font_italic(style))
 			    gsf_output_printf (output, "}");
 			gsf_output_printf (output, "$");
 		} else {
@@ -877,13 +906,13 @@ latex2e_write_multicolumn_cell (GsfOutput *output, GnmCell *cell, int start_col,
 		}
 
 		/* Close the styles for the cell. */
-		if (gnm_style_get_font_italic (mstyle))
+		if (gnm_style_get_font_italic (style))
 			gsf_output_printf (output, "}");
-		if (gnm_style_get_font_bold (mstyle))
+		if (gnm_style_get_font_bold (style))
 			gsf_output_printf (output, "}");
-		if (font_is_monospaced (mstyle))
+		if (font_is_monospaced (style))
 			gsf_output_printf (output, "}");
-		else if (font_is_sansserif (mstyle))
+		else if (font_is_sansserif (style))
 			gsf_output_printf (output, "}");
 		if (hidden)
 		  gsf_output_printf (output, "}");
@@ -926,13 +955,13 @@ static gboolean
 latex2e_find_hhlines (StyleBorderType *clines, int length, int col, int row,
 		      Sheet *sheet, GnmStyleElement type)
 {
-	GnmStyle *mstyle;
-	GnmBorder	   *border;
- 	GnmRange const *merge_range;
+	GnmStyle const	*style;
+	GnmBorder const	*border;
+ 	GnmRange const	*merge_range;
 	GnmCellPos pos;
 
-	mstyle = sheet_style_get (sheet, col, row);
-	border = gnm_style_get_border (mstyle, type);
+	style = sheet_style_get (sheet, col, row);
+	border = gnm_style_get_border (style, type);
 	if (style_border_is_blank (border))
 		return FALSE;
 	clines[0] = border->line_type;
