@@ -284,13 +284,11 @@ lotus_negate (const GnmExpr *e)
 {
 	const GnmExpr *res;
 
-	if (e->any.oper == GNM_EXPR_OP_UNARY_NEG) {
-		res = e->unary.value;
-		gnm_expr_ref (res);
-		gnm_expr_unref (e);
+	if (GNM_EXPR_GET_OPER (e) == GNM_EXPR_OP_UNARY_NEG) {
+		res = gnm_expr_copy (e->unary.value);
+		gnm_expr_free (e);
 	} else {
 		res = gnm_expr_new_unary (GNM_EXPR_OP_UNARY_NEG, e);
-		gnm_expr_unref (e);
 	}
 
 	return res;
@@ -479,7 +477,7 @@ handle_named_func (GnmExprList **stack, const GnmParsePos *orig,
 }
 
 
-static GnmExpr const *
+static GnmExprTop const *
 lotus_parse_formula_old (LotusState *state, GnmParsePos *orig,
 			 guint8 const *data, guint32 len)
 {
@@ -526,7 +524,7 @@ lotus_parse_formula_old (LotusState *state, GnmParsePos *orig,
 
 		case LOTUS_FORMULA_STRING:
 			parse_list_push_value (&stack,
-				lotus_new_string (data + i + 1));
+				lotus_new_string (data + i + 1, state->lmbcs_group));
 			i += 2 + strlen (data + i + 1);
 			break;
 
@@ -565,7 +563,7 @@ lotus_parse_formula_old (LotusState *state, GnmParsePos *orig,
 	if (gnm_expr_list_length (stack) != 1)
 		g_warning ("%s: args remain on stack",
 			   cell_coord_name (orig->eval.col, orig->eval.row));
-	return parse_list_pop (&stack, orig);
+	return gnm_expr_top_new (parse_list_pop (&stack, orig));
 }
 
 static void
@@ -585,7 +583,7 @@ get_new_cellref (GnmCellRef *dst, int relbits, const guint8 *data,
 		dst->col -= orig->eval.col;
 }
 
-static GnmExpr const *
+static GnmExprTop const *
 lotus_parse_formula_new (LotusState *state, GnmParsePos *orig,
 			 guint8 const *data, guint32 len)
 {
@@ -653,7 +651,7 @@ lotus_parse_formula_new (LotusState *state, GnmParsePos *orig,
 
 		case LOTUS_FORMULA_STRING:
 			parse_list_push_value (&stack,
-				lotus_new_string (data + i + 1));
+				lotus_new_string (data + i + 1, state->lmbcs_group));
 			i += 2 + strlen (data + i + 1);
 			break;
 
@@ -711,7 +709,9 @@ lotus_parse_formula_new (LotusState *state, GnmParsePos *orig,
 		case LOTUS_FORMULA_SPLFUNC: {
 			int args = data[i + 1];
 			int fnamelen = GSF_LE_GET_GUINT16 (data + i + 2);
-			char *name = lotus_get_lmbcs (data + (i + 4), len - (i + 4));
+			char *name = lotus_get_lmbcs (data + (i + 4),
+						      len - (i + 4),
+						      state->lmbcs_group);
 			size_t namelen;
 			char *p;
 			const LFuncInfo *f;
@@ -747,21 +747,21 @@ lotus_parse_formula_new (LotusState *state, GnmParsePos *orig,
 	if (gnm_expr_list_length (stack) != 1)
 		g_warning ("%s: args remain on stack",
 			   cell_coord_name (orig->eval.col, orig->eval.row));
-	return parse_list_pop (&stack, orig);
+	return gnm_expr_top_new (parse_list_pop (&stack, orig));
 }
 
 
-GnmExpr const *
+GnmExprTop const *
 lotus_parse_formula (LotusState *state, GnmParsePos *pos,
 		     guint8 const *data, guint32 len)
 {
-	const GnmExpr *result = (state->version >= LOTUS_VERSION_123V4)
+	const GnmExprTop *result = (state->version >= LOTUS_VERSION_123V4)
 		? lotus_parse_formula_new (state, pos, data, len)
 		: lotus_parse_formula_old (state, pos, data, len);
 
 #if FORMULA_DEBUG > 0
 	{
-		char *txt = gnm_expr_as_string (result, pos, gnm_expr_conventions_default);
+		char *txt = gnm_expr_top_as_string (result, pos, gnm_expr_conventions_default);
 		g_print ("Lotus: %s!%s: %s\n",
 			 pos->sheet->name_unquoted,
 			 cell_coord_name (pos->eval.col, pos->eval.row),
