@@ -30,21 +30,15 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
 #include <gnumeric.h>
-#include <sys/types.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <string.h>
+#include "mps.h"
 #include "cell.h"
 #include "sheet.h"
 #include "value.h"
-#include "file.h"
-#include "io-context.h"
+#include <goffice/app/io-context.h>
 #include "workbook-view.h"
 #include "workbook.h"
-#include "plugin-util.h"
-#include "module-plugin-defs.h"
+#include <gnm-plugin.h>
 #include "ranges.h"
 #include "style.h"
 #include "value.h"
@@ -53,9 +47,16 @@
 #include "parse-util.h"
 #include "func.h"
 #include "expr.h"
-#include "mps.h"
+#include <goffice/app/file.h>
+#include <goffice/utils/go-format.h>
+#include <goffice/app/error-info.h>
+#include <glib/gi18n.h>
+#include <sys/types.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <string.h>
 
-GNUMERIC_MODULE_PLUGIN_INFO_DECL;
+GNM_PLUGIN_MODULE_HEADER;
 
 
 /*************************************************************************
@@ -65,20 +66,17 @@ GNUMERIC_MODULE_PLUGIN_INFO_DECL;
 
 /* Writes a string into a cell. */
 static inline void
-mps_set_cell (Sheet *sh, int col, int row, const gchar *str)
+mps_set_cell (Sheet *sheet, int col, int row, const gchar *str)
 {
-        GnmCell *cell = sheet_cell_fetch (sh, col, row);
-
-        sheet_cell_set_value (cell, value_new_string (str));
+        cell_set_value (sheet_cell_fetch (sheet, col, row),
+			value_new_string (str));
 }
 
 /* Writes a float into a cell. */
 static inline void
-mps_set_cell_float (Sheet *sh, int col, int row, const gnm_float f)
+mps_set_cell_float (Sheet *sheet, int col, int row, const gnm_float f)
 {
-        GnmCell *cell = sheet_cell_fetch (sh, col, row);
-
-        sheet_cell_set_value (cell, value_new_float (f));
+        cell_set_value (sheet_cell_fetch (sheet, col, row), value_new_float (f));
 }
 
 static void
@@ -88,11 +86,11 @@ mps_set_style (Sheet *sh, int c1, int r1, int c2, int r2,
         GnmStyle *mstyle;
 	GnmRange  range;
 
-	mstyle = mstyle_new ();
+	mstyle = gnm_style_new ();
 	range_init (&range, c1, r1, c2, r2);
-	mstyle_set_font_italic (mstyle, italic);
-	mstyle_set_font_bold   (mstyle, bold);
-	mstyle_set_font_uline  (mstyle, ulined);
+	gnm_style_set_font_italic (mstyle, italic);
+	gnm_style_set_font_bold   (mstyle, bold);
+	gnm_style_set_font_uline  (mstyle, ulined);
 	sheet_style_apply_range (sh, &range, mstyle);
 }
 
@@ -191,14 +189,14 @@ mps_write_sheet_labels (MpsInputContext *ctxt, Sheet *sh)
 	if (n_rows_per_fn == 1) {
 		for (i = 0; i < ctxt->n_cols; i++)
 			mps_set_cell (sh, VARIABLE_COL + i, row,
-				      ctxt->col_name_tbl [i]);
+				      ctxt->col_name_tbl[i]);
 	} else {
 		GString *buf;
 		for (i = 0; i < MAX_COL; i++) {
 			buf = g_string_new (NULL);
 			g_string_append_printf (buf, "C[%d]", i + 1);
 			mps_set_cell (sh, VARIABLE_COL + i, row, buf->str);
-			g_string_free (buf, FALSE);
+			g_string_free (buf, TRUE);
 		}
 
 		for (i = 0; i < n_rows_per_fn; i++) {
@@ -206,7 +204,7 @@ mps_write_sheet_labels (MpsInputContext *ctxt, Sheet *sh)
 			g_string_append_printf (buf, "R[%d]", i + 1);
 			mps_set_cell (sh, VARIABLE_COL - 1, row + i + 1,
 				      buf->str);
-			g_string_free (buf, FALSE);
+			g_string_free (buf, TRUE);
 		}
 		mps_set_style (sh, VARIABLE_COL - 1, row,
 			       VARIABLE_COL - 1, row + n_rows_per_fn,
@@ -248,14 +246,14 @@ mps_write_sheet_labels (MpsInputContext *ctxt, Sheet *sh)
 	if (n_rows_per_fn == 1) {
 		for (i = 0; i < ctxt->n_cols; i++)
 			mps_set_cell (sh, CONSTRAINT_COL + i, row,
-				      ctxt->col_name_tbl [i]);
+				      ctxt->col_name_tbl[i]);
 	} else {
 		GString *buf;
 		for (i = 0; i < MAX_COL; i++) {
 			buf = g_string_new (NULL);
 			g_string_append_printf (buf, "C[%d]", i + 1);
 			mps_set_cell (sh, CONSTRAINT_COL + i, row, buf->str);
-			g_string_free (buf, FALSE);
+			g_string_free (buf, TRUE);
 		}
 	}
 	mps_set_style (sh, CONSTRAINT_COL - 1, row,
@@ -283,10 +281,9 @@ mps_write_coefficients (MpsInputContext *ctxt, Sheet *sh,
 	GSList  *current;
 	int     i, n, r, ecol, inc2;
 	int     n_rows_per_fn;
-	GString *var_range [2];
 	GnmRange   range, v_range;
 	GnmCell    *cell;
-	GString *buf;
+	const GnmExprTop *texpr;
 
 	/*
 	 * Add objective function stuff into the sheet.
@@ -315,8 +312,6 @@ mps_write_coefficients (MpsInputContext *ctxt, Sheet *sh,
 	/* Print constraints. */
 	inc2               = 2 * n_rows_per_fn;
 	param->constraints = NULL;
-	var_range [0]      = g_string_new (NULL);
-	var_range [1]      = g_string_new (NULL);
 
 	/* Initialize var_range to contain the range name of the
 	 * objective function variables. */
@@ -325,25 +320,22 @@ mps_write_coefficients (MpsInputContext *ctxt, Sheet *sh,
 		    VARIABLE_ROW + n_rows_per_fn - 1,
 		    (ctxt->n_cols % MAX_COL),
 		    VARIABLE_ROW + n_rows_per_fn - 1);
-	g_string_append_printf (var_range [i], "%s", range_name (&v_range));
 
 	i = 0;
 	for (current = ctxt->rows; current != NULL; current = current->next) {
 	          SolverConstraint   *c;
-		  MpsRow             *row;
+		  MpsRow             *row = current->data;
 		  int                col, r;
-		  GnmExpr            *expr;
-		  GnmExprList        *args;
+		  const GnmExprTop   *texpr;
 		  GnmCellRef          ref1, ref2;
 
-		  static const gchar *type_str[] = {
+		  static const gchar *const type_str[] = {
 			  "=", "<=", ">="
 		  };
 		  static const SolverConstraintType type_map[] = {
 			  SolverEQ, SolverLE, SolverGE
 		  };
 
-		  row = (MpsRow *) current->data;
 		  if (row->type == ObjectiveRow)
 		          continue;
 		  col = CONSTRAINT_COL;
@@ -358,7 +350,7 @@ mps_write_coefficients (MpsInputContext *ctxt, Sheet *sh,
 			   * memory, and, in addition, to speed up the loading.
 			   */
 #ifndef MPS_WRITE_ZERO_COEFFICIENTS
-			  if (ctxt->matrix [row->index][n] != 0)
+			  if (ctxt->matrix[row->index][n] != 0)
 #endif
 				  mps_set_cell_float
 					  (sh, col + n % MAX_COL,
@@ -375,35 +367,33 @@ mps_write_coefficients (MpsInputContext *ctxt, Sheet *sh,
 
 		  /* Add LHS field using SUMPRODUCT function. */
 		  range_init (&range, col, r, ctxt->n_cols, r);
-		  args = (GnmExprList *) g_list_append
-			  (NULL, (gpointer) gnm_expr_new_constant
-				 (value_new_cellrange_r (NULL, &v_range)));
-
-		  args = (GnmExprList *) g_list_append
-			  ((GList *) args, (gpointer) gnm_expr_new_constant
-				 (value_new_cellrange_r (NULL, &range)));
-
 		  cell = sheet_cell_fetch (sh, ecol + 1, r);
-		  expr = (GnmExpr *) gnm_expr_new_funcall
-			  (gnm_func_lookup ("SUMPRODUCT", NULL), args);
-		  cell_set_expr (cell, expr);
+		  texpr = gnm_expr_top_new
+			  (gnm_expr_new_funcall2
+			   (gnm_func_lookup ("SUMPRODUCT", NULL),
+			    gnm_expr_new_constant
+			    (value_new_cellrange_r (NULL, &v_range)),
+			    gnm_expr_new_constant
+			    (value_new_cellrange_r (NULL, &range))));
+		  cell_set_expr (cell, texpr);
+		  gnm_expr_top_unref (texpr);
 		  cell_queue_recalc (cell);
 
 		  /* Add Slack calculation */
-		  cellref_init (&ref1, sh, ecol + 1, r, FALSE);
-		  cellref_init (&ref2, sh, ecol + 3, r, FALSE);
-		  expr = (GnmExpr *) gnm_expr_new_binary
-			  (gnm_expr_new_cellref (&ref1),
-			   GNM_EXPR_OP_SUB,
-			   gnm_expr_new_cellref (&ref2));
-		  args = (GnmExprList *) g_list_append (NULL, (gpointer) expr);
+		  gnm_cellref_init (&ref1, sh, ecol + 1, r, FALSE);
+		  gnm_cellref_init (&ref2, sh, ecol + 3, r, FALSE);
 		  cell = sheet_cell_fetch (sh, ecol + 4, r);
-		  expr = (GnmExpr *) gnm_expr_new_funcall
-			  (gnm_func_lookup ("ABS", NULL), args);
-		  cell_set_expr (cell, expr);
+		  texpr = gnm_expr_top_new
+			  (gnm_expr_new_funcall1
+			   (gnm_func_lookup ("ABS", NULL),
+			    gnm_expr_new_binary
+			    (gnm_expr_new_cellref (&ref1),
+			     GNM_EXPR_OP_SUB,
+			     gnm_expr_new_cellref (&ref2))));
+		  cell_set_expr (cell, texpr);
+		  gnm_expr_top_unref (texpr);
 		  cell_queue_recalc (cell);
 
-
 		  /* Add Solver constraint */
 		  c          = g_new (SolverConstraint, 1);
 		  c->lhs.col = ecol + 1;
@@ -426,7 +416,7 @@ mps_write_coefficients (MpsInputContext *ctxt, Sheet *sh,
 	current = ctxt->rhs;
 	r   = CONSTRAINT_ROW  +  inc2;
 	while (current != NULL) {
-	          MpsRhs *rhs = (MpsRhs *) current->data;
+	          MpsRhs *rhs = current->data;
 
 		  mps_set_cell_float (sh, ecol + 3,
 				      r + rhs->row->index * n_rows_per_fn,
@@ -435,29 +425,26 @@ mps_write_coefficients (MpsInputContext *ctxt, Sheet *sh,
 	}
 
 	/* Write the objective fn. */
-	buf = g_string_new (NULL);
 	range_init (&range, VARIABLE_COL,
 		    VARIABLE_ROW + 1 + n_rows_per_fn,
 		    ctxt->n_cols,
 		    VARIABLE_ROW + 1 + n_rows_per_fn);
-	g_string_append_printf (buf, "=SUMPRODUCT(%s,%s)",
-			   var_range[0]->str,
-			   range_name (&range));
-
+	texpr = gnm_expr_top_new
+		(gnm_expr_new_funcall2
+		 (gnm_func_lookup ("SUMPRODUCT", NULL),
+		  gnm_expr_new_constant
+		  (value_new_cellrange_r (NULL, &v_range)),
+		  gnm_expr_new_constant
+		  (value_new_cellrange_r (NULL, &range))));
 	cell = sheet_cell_fetch (sh, OBJECTIVE_VALUE_COL, MAIN_INFO_ROW);
-	sheet_cell_set_text (cell, buf->str, NULL);
-	g_string_free (buf, FALSE);
+	cell_set_expr (cell, texpr);
+	gnm_expr_top_unref (texpr);
+	cell_queue_recalc (cell);
 
 	/* Store the input cell range for the Solver dialog. */
-	g_string_free (var_range [0], FALSE);
-	var_range [0] = g_string_new (NULL);
 	range_init (&range, VARIABLE_COL, VARIABLE_ROW,
 		    MAX_COL, VARIABLE_ROW + n_rows_per_fn - 1);
-	g_string_append_printf (var_range [0], "%s", range_name (&range));
-
-	param->input_entry_str = g_strdup (var_range [0]->str);
-	g_string_free (var_range [0], FALSE);
-	g_string_free (var_range [1], FALSE);
+	param->input_entry_str = g_strdup (range_as_string (&range));
 }
 
 /* Creates the spreadsheet model. */
@@ -488,7 +475,7 @@ mps_create_sheet (MpsInputContext *ctxt, WorkbookView *wbv)
 			mps_set_cell (sh, VARIABLE_COL - 1,
 				      VARIABLE_ROW + 1 + i + n_rows_per_fn,
 				      buf->str);
-			g_string_free (buf, FALSE);
+			g_string_free (buf, TRUE);
 		}
 	}
 
@@ -502,7 +489,7 @@ mps_create_sheet (MpsInputContext *ctxt, WorkbookView *wbv)
 
 
 	/* Autofit column A */
-	i = sheet_col_size_fit_pixels (sh, 0);
+	i = sheet_col_size_fit_pixels (sh, 0, 0, SHEET_MAX_ROWS - 1, FALSE);
 	if (i == 0)
 	          return;
 	sheet_col_set_size_pixels (sh, 0, i, TRUE);
@@ -528,7 +515,7 @@ mps_input_context_new (IOContext *io_context, Workbook *wb, GsfInput *input)
 	ctxt->input	     = (GsfInputTextline *)gsf_input_textline_new (input);
 	ctxt->line_no        = 1;
 	ctxt->line           = NULL;
-	ctxt->sheet          = workbook_sheet_add (wb, NULL, FALSE);
+	ctxt->sheet          = workbook_sheet_add (wb, -1);
 
 	ctxt->name           = NULL;
 	ctxt->rows           = NULL;
@@ -638,12 +625,12 @@ mps_input_context_destroy (MpsInputContext *ctxt)
  */
 
 void
-mps_file_open (GnmFileOpener const *fo, IOContext *io_context,
+mps_file_open (GOFileOpener const *fo, IOContext *io_context,
                WorkbookView *wbv, GsfInput *input)
 {
         MpsInputContext *ctxt;
 
-	ctxt = mps_input_context_new (io_context, wb_view_workbook (wbv),
+	ctxt = mps_input_context_new (io_context, wb_view_get_workbook (wbv),
 				      input);
 	if (ctxt != NULL) {
 	        mps_parse_file (ctxt);
