@@ -1,3 +1,4 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * sc.c - file import of SC/xspread files
  * Copyright 1999 Jeff Garzik <jgarzik@mandrakesoft.com>
@@ -16,6 +17,7 @@
 #include <goffice/app/io-context.h>
 #include <goffice/app/error-info.h>
 #include <goffice/utils/go-glib-extras.h>
+#include "sheet-style.h"
 #include "workbook-view.h"
 #include "workbook.h"
 #include "parse-util.h"
@@ -140,6 +142,16 @@ sc_parse_coord (char const **strdata, int *col, int *row)
 }
 
 
+static void
+set_h_align (GnmCell const *cell, GnmHAlign ha)
+{
+	GnmRange r;
+	GnmStyle *style = gnm_style_new ();
+	gnm_style_set_align_h (style, ha);
+	r.start = r.end = cell->pos;
+	sheet_style_apply_range	(cell->base.sheet, &r, style);
+}
+
 static gboolean
 sc_parse_label (ScParseState *state, char const *cmd, char const *str, int col, int row)
 {
@@ -147,7 +159,6 @@ sc_parse_label (ScParseState *state, char const *cmd, char const *str, int col,
 	char *s = NULL, *tmpout;
 	char const *tmpstr;
 	gboolean result = FALSE;
-	sc_string_cmd_t cmdtype;
 
 	g_return_val_if_fail (state, FALSE);
 	g_return_val_if_fail (state->sheet, FALSE);
@@ -183,24 +194,13 @@ sc_parse_label (ScParseState *state, char const *cmd, char const *str, int col,
 	cell_set_text (cell, s);
 
 	if (strcmp (cmd, "leftstring") == 0)
-		cmdtype = LEFTSTRING;
+		set_h_align (cell, HALIGN_LEFT);
 	else if (strcmp (cmd, "rightstring") == 0)
-		cmdtype = RIGHTSTRING;
+		set_h_align (cell, HALIGN_RIGHT);
+#if 0
 	else
 		cmdtype = LABEL;
-
-	if (cmdtype == LEFTSTRING || cmdtype == RIGHTSTRING) {
-		GnmStyle *mstyle;
-
-		mstyle = cell_get_mstyle (cell);
-		if (!mstyle)
-			goto err_out;
-
-		if (cmdtype == LEFTSTRING)
-			gnm_style_set_align_h (mstyle, HALIGN_LEFT);
-		else
-			gnm_style_set_align_h (mstyle, HALIGN_RIGHT);
-	}
+#endif
 
 	result = TRUE;
 	/* fall through */
@@ -327,7 +327,7 @@ sc_rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *pp,
 static gboolean
 sc_parse_let (ScParseState *state, char const *cmd, char const *str, int col, int row)
 {
-	GnmExpr const *tree;
+	GnmExprTop const *texpr;
 	GnmCell *cell;
 	GnmParsePos pos;
 	GnmValue const *v;
@@ -348,28 +348,27 @@ sc_parse_let (ScParseState *state, char const *cmd, char const *str, int col, in
 	for (p1 = (char *) str, p2 = str2; *p1; p1++)
 		if (*p1 != '@')
 			*p2++ = *p1;
-	tree = gnm_expr_parse_str (str2,
+	texpr = gnm_expr_parse_str (str2,
 				   parse_pos_init_cell (&pos, cell),
 				   GNM_EXPR_PARSE_DEFAULT,
 				   state->exprconv,
 				   NULL);
 	g_free (str2);
-	if (!tree) {
+	if (!texpr) {
 		g_warning ("cannot parse cmd='%s', str='%s', col=%d, row=%d.",
 			   cmd, str, col, row);
-		goto out;
+		return TRUE;
 	}
 
-	v = gnm_expr_get_constant (tree);
+	v = gnm_expr_top_get_constant (texpr);
 	if (v && VALUE_IS_NUMBER (v)) {
 		cell_set_value (cell, value_dup (v));
 	} else {
-		cell_set_expr (cell, tree);
+		cell_set_expr (cell, texpr);
 		cell_queue_recalc (cell);
 	}
 
-out:
-	if (tree) gnm_expr_unref (tree); 
+	if (texpr) gnm_expr_top_unref (texpr); 
 	return TRUE;
 }
 
@@ -511,7 +510,6 @@ sc_conventions (void)
 	convs->ref_parser = sc_rangeref_parse;
 	convs->range_sep_colon = TRUE;
 	convs->sheet_sep_exclamation	= TRUE;
-	convs->dots_in_names		= TRUE;
 	convs->unknown_function_handler = gnm_func_placeholder_factory;
 	convs->function_rewriter_hash =
 		g_hash_table_new (go_ascii_strcase_hash, 
@@ -534,7 +532,7 @@ sc_file_open (GOFileOpener const *fo, IOContext *io_context,
 	ErrorInfo *error;
 	ScParseState state;
 
-	wb    = wb_view_workbook (wb_view);
+	wb    = wb_view_get_workbook (wb_view);
 	name  = workbook_sheet_get_free_name (wb, "SC", FALSE, TRUE);
 	state.sheet = sheet_new (wb, name);
 	g_free (name);
