@@ -27,13 +27,15 @@
 #include "commands.h"
 
 #include <gnumeric-gconf.h>
+#include <goffice/app/go-doc.h>
+#include <goffice/utils/go-glib-extras.h>
+#include <goffice/utils/go-file.h>
 #include <gsf/gsf-impl-utils.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkiconfactory.h>
 #include <gtk/gtkselection.h>
 #include <gtk/gtkicontheme.h>
 #include <glib/gi18n.h>
-#include <goffice/utils/go-glib-extras.h>
 
 #define GNM_APP(o)		(G_TYPE_CHECK_INSTANCE_CAST((o), GNM_APP_TYPE, GnmApp))
 #define GNM_APP_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST((k),	 GNM_APP_TYPE, GnmAppClass))
@@ -141,7 +143,7 @@ gnm_app_workbook_list_add (Workbook *wb)
 
 	app->workbook_list = g_list_prepend (app->workbook_list, wb);
 	g_signal_connect (G_OBJECT (wb),
-		"filename_changed",
+		"notify::uri",
 		G_CALLBACK (_gnm_app_flag_windows_changed), NULL);
 	_gnm_app_flag_windows_changed ();
 	g_signal_emit (G_OBJECT (app), signals [WORKBOOK_ADDED], 0, wb);
@@ -365,17 +367,56 @@ gnm_app_clipboard_area_get (void)
 	return NULL;
 }
 
-struct wb_name_closure {
-	Workbook *wb;
-	char const *name;
-};
-static gboolean
-cb_workbook_name (Workbook * wb, gpointer closure)
+Workbook *
+gnm_app_workbook_get_by_name (char const *name,
+			      char const *ref_uri)
 {
-	struct wb_name_closure *dat = closure;
-	const char *wb_uri = workbook_get_uri (wb);
+	Workbook *wb;
+	char *filename = NULL;
 
-	if (wb_uri && strcmp (wb_uri, dat->name) == 0) {
+	/* Try as URI.  */
+	wb = gnm_app_workbook_get_by_uri (name);
+	if (wb)
+		goto out;
+
+	filename = g_filename_from_utf8 (name, -1, NULL, NULL, NULL);
+
+	/* Try as absolute filename.  */
+	if (filename && g_path_is_absolute (filename)) {
+		char *uri = go_filename_to_uri (filename);
+		wb = gnm_app_workbook_get_by_uri (uri);
+		g_free (uri);
+		if (wb)
+			goto out;
+	}
+
+	if (filename && ref_uri) {
+		char *rel_uri = go_url_encode (filename, 1);
+		char *uri = go_url_resolve_relative (ref_uri, rel_uri);
+		g_free (rel_uri);
+		wb = gnm_app_workbook_get_by_uri (uri);
+		g_free (uri);
+		if (wb)
+			goto out;
+	}
+
+ out:
+	g_free (filename);
+	return wb;
+}
+
+struct wb_uri_closure {
+	Workbook *wb;
+	char const *uri;
+};
+
+static gboolean
+cb_workbook_uri (Workbook * wb, gpointer closure)
+{
+	struct wb_uri_closure *dat = closure;
+	char const *wb_uri = go_doc_get_uri (GO_DOC (wb));
+
+	if (wb_uri && strcmp (wb_uri, dat->uri) == 0) {
 		dat->wb = wb;
 		return FALSE;
 	}
@@ -383,14 +424,14 @@ cb_workbook_name (Workbook * wb, gpointer closure)
 }
 
 Workbook *
-gnm_app_workbook_get_by_name (char const * const name)
+gnm_app_workbook_get_by_uri (char const *uri)
 {
-	struct wb_name_closure close;
-	close.wb = NULL;
-	close.name = name;
-	gnm_app_workbook_foreach (&cb_workbook_name, &close);
+	struct wb_uri_closure closure;
+	closure.wb = NULL;
+	closure.uri = uri;
+	gnm_app_workbook_foreach (&cb_workbook_uri, &closure);
 
-	return close.wb;
+	return closure.wb;
 }
 
 gboolean
@@ -416,21 +457,25 @@ struct wb_index_closure
 };
 
 static gboolean
-cb_workbook_index (G_GNUC_UNUSED Workbook * wb, gpointer closure)
+cb_workbook_index (Workbook *wb, gpointer closure)
 {
 	struct wb_index_closure *dat = closure;
-	return (--(dat->index) != 0);
+	if (--(dat->index) == 0) {
+		dat->wb = wb;
+		return FALSE;
+	}
+	return TRUE;
 }
 
 Workbook *
 gnm_app_workbook_get_by_index (int i)
 {
-	struct wb_index_closure close;
-	close.wb = NULL;
-	close.index = i;
-	gnm_app_workbook_foreach (&cb_workbook_index, &close);
+	struct wb_index_closure closure;
+	closure.wb = NULL;
+	closure.index = i;
+	gnm_app_workbook_foreach (&cb_workbook_index, &closure);
 
-	return close.wb;
+	return closure.wb;
 }
 
 double
