@@ -56,7 +56,7 @@ sv_select_cur_row (SheetView *sv)
 	if (sel != NULL) {
 		GnmRange r = *sel;
 		sv_selection_reset (sv);
-		sv_selection_add_range (sv,
+		sv_selection_add_full (sv,
 			sv->edit_pos.col, sv->edit_pos.row,
 			0, r.start.row, SHEET_MAX_COLS-1, r.end.row);
 		sheet_update (sv->sheet);
@@ -76,7 +76,7 @@ sv_select_cur_col (SheetView *sv)
 	if (sel != NULL) {
 		GnmRange r = *sel;
 		sv_selection_reset (sv);
-		sv_selection_add_range (sv,
+		sv_selection_add_full (sv,
 			sv->edit_pos.col, sv->edit_pos.row,
 			r.start.col, 0, r.end.col, SHEET_MAX_ROWS-1);
 		sheet_update (sv->sheet);
@@ -87,32 +87,22 @@ sv_select_cur_col (SheetView *sv)
  * sv_select_cur_array :
  * @sv: The sheet
  *
- * if the current cell is part of an array select
- * the entire array.
- */
+ * If the editpos is part of an array clear the selection and select the array.
+ **/
 void
 sv_select_cur_array (SheetView *sv)
 {
-	GnmExprArray const *array;
-	int col, row;
+	GnmRange a;
+	int const c = sv->edit_pos.col;
+	int const r = sv->edit_pos.row;
 
-	col = sv->edit_pos.col;
-	row = sv->edit_pos.row;
-	array = cell_is_array (sheet_cell_get (sv->sheet, col, row));
-
-	if (array == NULL)
+	if (!cell_array_bound (sheet_cell_get (sv->sheet, c, r), &a))
 		return;
 
+	/* leave the edit pos where it is, select the entire array. */
 	sv_selection_reset (sv);
-	/*
-	 * leave the edit cell where it is,
-	 * select the entire array.
-	 */
-	sv_selection_add_range (sv, col, row,
-				   col - array->x, row - array->y,
-				   col - array->x + array->cols - 1,
-				   row - array->y + array->rows - 1);
-
+	sv_selection_add_full (sv, c, r,
+		a.start.col, a.start.row, a.end.col, a.end.row);
 	sheet_update (sv->sheet);
 }
 
@@ -222,13 +212,9 @@ sv_select_cur_depends (SheetView *sv)
 
 		/* now select the ranges */
 		while (ptr) {
-			GnmRange *r = ptr->data;
-			sv_selection_add_range (sv,
-						r->start.col, r->start.row,
-						r->start.col, r->start.row,
-						r->end.col, r->end.row);
+			sv_selection_add_range (sv, ptr->data);
 			g_free (ptr->data);
-			ptr = g_list_remove (ptr, r);
+			ptr = g_list_remove (ptr, ptr->data);
 		}
 	}
 	sheet_update (sv->sheet);
@@ -254,7 +240,7 @@ sv_select_cur_inputs (SheetView *sv)
 		sv->edit_pos.col, sv->edit_pos.row);
 	if (cell == NULL || !cell_has_expr (cell))
 		return;
-	ranges = gnm_expr_get_ranges (cell->base.expression);
+	ranges = gnm_expr_top_get_ranges (cell->base.texpr);
 	if (ranges == NULL)
 		return;
 
@@ -273,13 +259,13 @@ sv_select_cur_inputs (SheetView *sv)
 		if (r->a.sheet != NULL && r->a.sheet != sv->sheet)
 			continue;
 
-		sv_selection_add_range (sv,
-					cellref_get_abs_col (&r->a, &ep),
-					cellref_get_abs_row (&r->a, &ep),
-					cellref_get_abs_col (&r->a, &ep),
-					cellref_get_abs_row (&r->a, &ep),
-					cellref_get_abs_col (&r->b, &ep),
-					cellref_get_abs_row (&r->b, &ep));
+		sv_selection_add_full (sv,
+			gnm_cellref_get_col (&r->a, &ep),
+			gnm_cellref_get_row (&r->a, &ep),
+			gnm_cellref_get_col (&r->a, &ep),
+			gnm_cellref_get_row (&r->a, &ep),
+			gnm_cellref_get_col (&r->b, &ep),
+			gnm_cellref_get_row (&r->b, &ep));
 		value_release (v);
 	}
 	g_slist_free (ranges);
