@@ -11,10 +11,6 @@ void command_repeat (WorkbookControl *wbc);
 void command_setup_combos	(WorkbookControl *wbc);
 void command_list_release	(GSList *cmds);
 
-GString *cmd_range_list_to_string_utility (Sheet *sheet, GSList const *ranges);
-char *cmd_range_to_str_utility  (Sheet *sheet, GnmRange const *range);
-char *cmd_cell_pos_name_utility (Sheet *sheet, GnmCellPos const *pos);
-
 gboolean cmd_set_text		(WorkbookControl *wbc, Sheet *sheet,
 				 GnmCellPos const *pos, const char *new_text,
 				 PangoAttrList *markup);
@@ -78,8 +74,7 @@ gboolean cmd_merge_cells	(WorkbookControl *wbc, Sheet *sheet,
 gboolean cmd_unmerge_cells	(WorkbookControl *wbc, Sheet *sheet,
 				 GSList const *selection);
 
-gboolean cmd_search_replace     (WorkbookControl *wbc, Sheet *sheet,
-				 GnmSearchReplace *sr);
+gboolean cmd_search_replace     (WorkbookControl *wbc, GnmSearchReplace *sr);
 
 gboolean cmd_colrow_std_size    (WorkbookControl *wbc, Sheet *sheet,
 				 gboolean is_cols, double new_default);
@@ -94,17 +89,12 @@ gboolean cmd_objects_move	(WorkbookControl *wbc,
 gboolean cmd_object_format	(WorkbookControl *wbc, SheetObject *so,
 				 gpointer orig_style);
 
-gboolean cmd_reorganize_sheets	(WorkbookControl *wbc, GSList *new_order,
-				 GSList *changed_names, GSList *new_names, 
-				 GSList *deleted_sheets,
-				 GSList *color_changed, GSList *new_colors_back,
-				 GSList *new_colors_fore,
-				 GSList *protection_changed, GSList *new_locks,
-				 GSList *visibility_changed, 
-				 GSList *new_visibility);
+gboolean cmd_reorganize_sheets  (WorkbookControl *wbc,
+				 WorkbookSheetState *old_state,
+				 Sheet *undo_sheet);
 
 gboolean cmd_rename_sheet	(WorkbookControl *wbc, Sheet *sheet,
-				 char const *old_name, char const *new_name);
+				 char const *new_name);
 
 gboolean cmd_set_comment	(WorkbookControl *wbc, Sheet *sheet,
 				 GnmCellPos const *pos, const char *new_text);
@@ -116,10 +106,12 @@ gboolean cmd_analysis_tool	(WorkbookControl *wbc, Sheet *sheet,
 gboolean cmd_merge_data		(WorkbookControl *wbc, Sheet *sheet,
 				 GnmValue *merge_zone, GSList *merge_fields, GSList *merge_data);
 
-gboolean cmd_change_summary	(WorkbookControl *wbc, GSList *sin_changes);
+gboolean cmd_change_meta_data	(WorkbookControl *wbc, GSList *changes, GSList *removed);
 gboolean cmd_print_setup	(WorkbookControl *wbc, Sheet *sheet, PrintInformation const *pi);
 gboolean cmd_define_name	(WorkbookControl *wbc, char const *name,
-				 GnmParsePos const *pp, GnmExpr const *expr);
+				 GnmParsePos const *pp,
+				 GnmExprTop const *texpr);
+gboolean cmd_remove_name        (WorkbookControl *wbc, GnmNamedExpr *nexpr);
 
 typedef enum  {
 	cmd_object_pull_to_front,
@@ -151,11 +143,10 @@ gboolean cmd_solver (WorkbookControl *wbc,
 gboolean cmd_goal_seek (WorkbookControl *wbc,
 			GnmCell *cell, GnmValue *ov, GnmValue *nv);
 
-gboolean cmd_clone_sheet (WorkbookControl *wbc, Sheet *sheet);
-
 gboolean cmd_tabulate (WorkbookControl *wbc, gpointer data);
 
 gboolean cmd_so_graph_config (WorkbookControl *wbc, SheetObject *sog,  
                               GObject *n_graph, GObject *o_graph);
+gboolean cmd_toggle_rtl (WorkbookControl *wbc, Sheet *sheet);
 
 #endif /* GNUMERIC_COMMANDS_H */
