@@ -27,7 +27,6 @@
 #include "dialogs.h"
 #include "help.h"
 
-#include <math.h>
 #include <sheet.h>
 #include <sheet-view.h>
 #include <sheet-merge.h>
@@ -82,6 +81,9 @@
 #include <gtk/gtkicontheme.h>
 #include <gtk/gtkbox.h>
 
+#include <math.h>
+#include <string.h>
+
 static struct {
 	char const *Cname;
 	GnmUnderline ut;
@@ -338,7 +340,7 @@ setup_pattern_button (GdkScreen *screen,
 						      TRUE);
 		}
 	} else
-		g_warning ("CellFormat : Unexpected missing glade widget");
+		g_warning ("CellFormat: Unexpected missing glade widget");
 }
 
 static void
@@ -438,7 +440,7 @@ init_button_image (GladeXML *gui, char const *name)
 
 static void
 cb_number_format_changed (G_GNUC_UNUSED GtkWidget *widget,
-			  char *fmt,
+			  const char *fmt,
 			  FormatState *state)
 {
 	gboolean changed = FALSE;
@@ -447,9 +449,8 @@ cb_number_format_changed (G_GNUC_UNUSED GtkWidget *widget,
 	if (!state->enable_edit)
 		return;
 
-	if (fmt)
-	{
-	  	gnm_style_set_format_text (state->result, g_strdup (fmt));
+	if (fmt) {
+	  	gnm_style_set_format_text (state->result, fmt);
 		changed =  TRUE;
 	}
 
@@ -517,7 +518,7 @@ cb_align_h_toggle (GtkToggleButton *button, FormatState *state)
 					  supports_indent);
 		gtk_widget_set_sensitive (GTK_WIDGET (state->align.indent_label),
 					  supports_indent);
-		/* TODO : Should we 0 the indent ? */
+		/* TODO: Should we 0 the indent ? */
 		fmt_dialog_changed (state);
 	}
 }
@@ -581,25 +582,26 @@ fmt_dialog_init_align_page (FormatState *state)
 		char const *const	name;
 		GnmHAlign	align;
 	} const h_buttons[] = {
-	    { "halign_left",	HALIGN_LEFT },
-	    { "halign_center",	HALIGN_CENTER },
-	    { "halign_right",	HALIGN_RIGHT },
-	    { "halign_general",	HALIGN_GENERAL },
-	    { "halign_justify",	HALIGN_JUSTIFY },
-	    { "halign_fill",	HALIGN_FILL },
-	    { "halign_center_across_selection",	HALIGN_CENTER_ACROSS_SELECTION },
-	    { NULL }
+		{ "halign_left",	HALIGN_LEFT },
+		{ "halign_center",	HALIGN_CENTER },
+		{ "halign_right",	HALIGN_RIGHT },
+		{ "halign_general",	HALIGN_GENERAL },
+		{ "halign_justify",	HALIGN_JUSTIFY },
+		{ "halign_fill",	HALIGN_FILL },
+		{ "halign_center_across_selection",	HALIGN_CENTER_ACROSS_SELECTION },
+		{ "halign_distributed",	                HALIGN_DISTRIBUTED },
+		{ NULL }
 	};
 	static struct {
 		char const *const	name;
 		GnmVAlign	align;
 	} const v_buttons[] = {
-	    { "valign_top", VALIGN_TOP },
-	    { "valign_center", VALIGN_CENTER },
-	    { "valign_bottom", VALIGN_BOTTOM },
-	    { "valign_justify", VALIGN_JUSTIFY },
-	    { "valign_distributed", VALIGN_DISTRIBUTED },
-	    { NULL }
+		{ "valign_top", VALIGN_TOP },
+		{ "valign_center", VALIGN_CENTER },
+		{ "valign_bottom", VALIGN_BOTTOM },
+		{ "valign_justify", VALIGN_JUSTIFY },
+		{ "valign_distributed", VALIGN_DISTRIBUTED },
+		{ NULL }
 	};
 
 	GtkWidget *w;
@@ -804,7 +806,7 @@ fmt_dialog_init_font_page (FormatState *state)
 	g_return_if_fail (uline != NULL);
 	g_return_if_fail (strike != NULL);
 
-	/* TODO : How to insert the font box in the right place initially */
+	/* TODO: How to insert the font box in the right place initially */
 	gtk_widget_show (tmp);
 	gtk_box_pack_start (GTK_BOX (container), tmp, TRUE, TRUE, 0);
 	gtk_box_reorder_child (GTK_BOX (container), tmp, 0);
@@ -1066,7 +1068,7 @@ border_get_mstyle (FormatState const *state, StyleBorderLocation const loc)
 	}
 	return style_border_fetch
 		(state->border.edge[loc].pattern_index, color,
-		 style_border_get_orientation (loc + MSTYLE_BORDER_TOP));
+		 style_border_get_orientation (loc));
 }
 
 /* See if either the color or pattern for any segment has changed and
@@ -1151,13 +1153,13 @@ border_event (GtkWidget *widget, GdkEventButton *event, FormatState *state)
 	else if (y >= B-5.)	which = STYLE_BORDER_BOTTOM;
 	else if (x >= R-5.)	which = STYLE_BORDER_RIGHT;
 	else switch (state->selection_mask) {
-	case 1 :
+	case 1:
 		if ((x < V) == (y < H))
 			which = STYLE_BORDER_REV_DIAG;
 		else
 			which = STYLE_BORDER_DIAG;
 		break;
-	case 2 :
+	case 2:
 		if (H-5. < y  && y < H+5.)
 			which = STYLE_BORDER_HORIZ;
 		else {
@@ -1170,7 +1172,7 @@ border_event (GtkWidget *widget, GdkEventButton *event, FormatState *state)
 				which = STYLE_BORDER_DIAG;
 		}
 		break;
-	case 4 :
+	case 4:
 		if (V-5. < x  && x < V+5.)
 			which = STYLE_BORDER_VERT;
 		else {
@@ -1183,7 +1185,7 @@ border_event (GtkWidget *widget, GdkEventButton *event, FormatState *state)
 				which = STYLE_BORDER_DIAG;
 		}
 		break;
-	case 8 :
+	case 8:
 		if (V-5. < x  && x < V+5.)
 			which = STYLE_BORDER_VERT;
 		else if (H-5. < y  && y < H+5.)
@@ -1200,7 +1202,7 @@ border_event (GtkWidget *widget, GdkEventButton *event, FormatState *state)
 		}
 		break;
 
-	default :
+	default:
 		g_assert_not_reached ();
 	}
 
@@ -1463,7 +1465,7 @@ static void
 cb_protection_locked_toggle (GtkToggleButton *button, FormatState *state)
 {
 	if (state->enable_edit) {
-		gnm_style_set_content_locked (state->result,
+		gnm_style_set_contents_locked (state->result,
 			gtk_toggle_button_get_active (button));
 		fmt_dialog_changed (state);
 	}
@@ -1473,7 +1475,7 @@ static void
 cb_protection_hidden_toggle (GtkToggleButton *button, FormatState *state)
 {
 	if (state->enable_edit) {
-		gnm_style_set_content_hidden (state->result,
+		gnm_style_set_contents_hidden (state->result,
 			gtk_toggle_button_get_active (button));
 		fmt_dialog_changed (state);
 	}
@@ -1496,8 +1498,8 @@ fmt_dialog_init_protection_page (FormatState *state)
 	GtkWidget *w;
 	gboolean flag = FALSE;
 
-	flag = (state->conflicts & (1 << MSTYLE_CONTENT_LOCKED))
-		? FALSE : gnm_style_get_content_locked (state->style);
+	flag = (state->conflicts & (1 << MSTYLE_CONTENTS_LOCKED))
+		? FALSE : gnm_style_get_contents_locked (state->style);
 	w = glade_xml_get_widget (state->gui, "protection_locked");
 	state->protection.locked = GTK_CHECK_BUTTON (w);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), flag);
@@ -1505,8 +1507,8 @@ fmt_dialog_init_protection_page (FormatState *state)
 		"toggled",
 		G_CALLBACK (cb_protection_locked_toggle), state);
 
-	flag = (state->conflicts & (1 << MSTYLE_CONTENT_HIDDEN))
-		? FALSE : gnm_style_get_content_hidden (state->style);
+	flag = (state->conflicts & (1 << MSTYLE_CONTENTS_HIDDEN))
+		? FALSE : gnm_style_get_contents_hidden (state->style);
 	w = glade_xml_get_widget (state->gui, "protection_hidden");
 	state->protection.hidden = GTK_CHECK_BUTTON (w);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), flag);
@@ -1526,7 +1528,7 @@ fmt_dialog_init_protection_page (FormatState *state)
 
 /*****************************************************************************/
 
-static GnmExpr const *
+static GnmExprTop const *
 validation_entry_to_expr (Sheet *sheet, GnmExprEntry *gee)
 {
 	GnmParsePos pp;
@@ -1551,21 +1553,23 @@ validation_rebuild_validation (FormatState *state)
 		ValidationOp    op    = gtk_combo_box_get_active (state->validation.op);
 		char *title = gtk_editable_get_chars (GTK_EDITABLE (state->validation.error.title), 0, -1);
 		char *msg   = gnumeric_textview_get_text (state->validation.error.msg);
-		GnmExpr const *expr0 = validation_entry_to_expr (state->sheet,
-								 state->validation.expr0.entry);
-		GnmExpr const *expr1 = NULL;
+		GnmExprTop const *texpr0 =
+			validation_entry_to_expr (state->sheet,
+						  state->validation.expr0.entry);
+		GnmExprTop const *texpr1 = NULL;
 
-		if (expr0 != NULL) {
+		if (texpr0 != NULL) {
 			if (type == VALIDATION_TYPE_CUSTOM || type == VALIDATION_TYPE_IN_LIST) {
 				state->validation.valid = 1;
 				op = VALIDATION_OP_NONE;
 			} else if (op == VALIDATION_OP_BETWEEN || op == VALIDATION_OP_NOT_BETWEEN) {
-				expr1 = validation_entry_to_expr (state->sheet, state->validation.expr1.entry);
-				if (expr1 != NULL)
+				texpr1 = validation_entry_to_expr (state->sheet,
+								   state->validation.expr1.entry);
+				if (texpr1 != NULL)
 					state->validation.valid = 2;
 				else {
 					state->validation.valid = -2;
-					gnm_expr_unref (expr0);
+					gnm_expr_top_unref (texpr0);
 				}
 			} else
 				state->validation.valid = 1;
@@ -1573,11 +1577,16 @@ validation_rebuild_validation (FormatState *state)
 			state->validation.valid = -1;
 
 		if (state->validation.valid > 0) {
-			gnm_style_set_validation (state->result,
-					       validation_new (style, type, op, title, msg,
-							       expr0, expr1,
-							       gtk_toggle_button_get_active (state->validation.allow_blank),
-							       gtk_toggle_button_get_active (state->validation.use_dropdown)));
+			gboolean allow_blank = gtk_toggle_button_get_active (state->validation.allow_blank);
+			gboolean use_dropdown = gtk_toggle_button_get_active (state->validation.use_dropdown);
+			gnm_style_set_validation
+				(state->result,
+				 validation_new
+				 (style, type, op, title, msg,
+				  texpr0,
+				  texpr1,
+				  allow_blank,
+				  use_dropdown));
 		}
 
 		g_free (msg);
@@ -1604,10 +1613,10 @@ cb_validation_error_action_changed (G_GNUC_UNUSED GtkMenuShell *ignored,
 		char const *s = NULL;
 
 		switch (index) {
-		case 1 : s = GTK_STOCK_DIALOG_ERROR;	break;
-		case 2 : s = GTK_STOCK_DIALOG_WARNING;	break;
-		case 3 : s = GTK_STOCK_DIALOG_INFO;	break;
-		default :
+		case 1: s = GTK_STOCK_DIALOG_ERROR;	break;
+		case 2: s = GTK_STOCK_DIALOG_WARNING;	break;
+		case 3: s = GTK_STOCK_DIALOG_INFO;	break;
+		default:
 			 g_warning ("Unknown validation style");
 			 return;
 		}
@@ -1633,37 +1642,43 @@ cb_validation_sensitivity (G_GNUC_UNUSED GtkMenuShell *ignored,
 		state->validation.constraint_type);
 
 	switch (type) {
-	case VALIDATION_TYPE_IN_LIST :		msg0 = _("Source"); break;
-	case VALIDATION_TYPE_CUSTOM :		msg0 = _("Criteria"); break;
+	case VALIDATION_TYPE_IN_LIST:		msg0 = _("Source"); break;
+	case VALIDATION_TYPE_CUSTOM:		msg0 = _("Criteria"); break;
 
-	case VALIDATION_TYPE_AS_INT :
-	case VALIDATION_TYPE_AS_NUMBER :
-	case VALIDATION_TYPE_AS_DATE :
-	case VALIDATION_TYPE_AS_TIME :
-	case VALIDATION_TYPE_TEXT_LENGTH : {
+	case VALIDATION_TYPE_AS_INT:
+	case VALIDATION_TYPE_AS_NUMBER:
+	case VALIDATION_TYPE_AS_DATE:
+	case VALIDATION_TYPE_AS_TIME:
+	case VALIDATION_TYPE_TEXT_LENGTH: {
 		ValidationOp const op = gtk_combo_box_get_active (
 			state->validation.op);
 		has_operators = TRUE;
 		switch (op) {
-		case VALIDATION_OP_BETWEEN : case VALIDATION_OP_NOT_BETWEEN :
-			msg0 = _("Min :");
-			msg1 = _("Max :");
+		case VALIDATION_OP_NONE:
 			break;
-		case VALIDATION_OP_EQUAL : case VALIDATION_OP_NOT_EQUAL :
-			msg0 = _("Value :");
+		case VALIDATION_OP_BETWEEN:
+		case VALIDATION_OP_NOT_BETWEEN:
+			msg0 = _("Min:");
+			msg1 = _("Max:");
 			break;
-		case VALIDATION_OP_GT : case VALIDATION_OP_GTE :
-			msg0 =_("Min :");
+		case VALIDATION_OP_EQUAL:
+		case VALIDATION_OP_NOT_EQUAL:
+			msg0 = _("Value:");
 			break;
-		case VALIDATION_OP_LT : case VALIDATION_OP_LTE :
-			msg0 = _("Max :");
+		case VALIDATION_OP_GT:
+		case VALIDATION_OP_GTE:
+			msg0 =_("Min:");
 			break;
-		default :
-			g_warning ("Unknown operator index");
+		case VALIDATION_OP_LT:
+		case VALIDATION_OP_LTE:
+			msg0 = _("Max:");
+			break;
+		default:
+			g_warning ("Unknown operator index %d", (int)op);
 		}
 		break;
 	}
-	default :
+	default:
 		break;
 	}
 
@@ -1863,9 +1878,9 @@ fmt_dialog_init_validation_page (FormatState *state)
 		parse_pos_init (&pp, state->sheet->workbook, state->sheet,
 			state->sv->edit_pos.col, state->sv->edit_pos.row);
 		gnm_expr_entry_load_from_expr (state->validation.expr0.entry,
-			v->expr[0], &pp);
+			v->texpr[0], &pp);
 		gnm_expr_entry_load_from_expr (state->validation.expr1.entry,
-			v->expr[1], &pp);
+			v->texpr[1], &pp);
 	}
 
 	cb_validation_sensitivity (NULL, state);
@@ -1979,19 +1994,16 @@ cb_fmt_dialog_dialog_buttons (GtkWidget *btn, FormatState *state)
 			state->protection.sheet_protected_changed = FALSE;
 		}
 
-		gnm_style_ref (state->result);
-
 		for (i = STYLE_BORDER_TOP; i < STYLE_BORDER_EDGE_MAX; i++)
 			borders[i] = border_get_mstyle (state, i);
 
 		cmd_selection_format (WORKBOOK_CONTROL (state->wbcg),
 			state->result, borders, NULL);
-
-		gnm_style_unref (state->result);
-		sheet_update (state->sheet);
-
+		/* state->result got absorbed.  */
 		/* Get a fresh style to accumulate results in */
 		state->result = gnm_style_new ();
+		sheet_update (state->sheet);
+
 
 		gtk_widget_set_sensitive (state->apply_button, FALSE);
 	}
@@ -2243,7 +2255,7 @@ fmt_dialog_impl (FormatState *state, FormatDialogPosition_t pageno)
 
 	/* Setup pattern buttons & select the current pattern (or the 1st
 	 * if none is selected)
-	 * NOTE : This must be done AFTER the colour has been setup to
+	 * NOTE: This must be done AFTER the colour has been setup to
 	 * avoid having it erased by initialization.
 	 */
 	state->back.pattern.draw_preview = &draw_pattern_selected;
@@ -2383,9 +2395,8 @@ dialog_cell_format (WorkbookControlGUI *wbcg, FormatDialogPosition_t pageno)
 	state->dialog_changed	= NULL;
 	state->dialog_changed_user_data = NULL;
 
-	(void) selection_foreach_range (state->sv, TRUE,
-					fmt_dialog_selection_type,
-					state);
+	(void) sv_selection_foreach (state->sv,
+		fmt_dialog_selection_type, state);
 	state->selection_mask	= 1 << state->selection_mask;
 
 	fmt_dialog_impl (state, pageno);
