@@ -396,23 +396,9 @@ translate_range (GnmValue *range, SortFlowState *state)
 	state->header = gtk_toggle_button_get_active (
 		GTK_TOGGLE_BUTTON (state->cell_sort_header_check));
 
-	if (state->sel == NULL) {
-		state->sel = range;
-		load_model_data(state);
-		return;
-	}
-	
-	if (state->sel->v_range.cell.a.sheet != range->v_range.cell.a.sheet ||
-	    state->sel->v_range.cell.a.col != range->v_range.cell.a.col ||
-	    state->sel->v_range.cell.a.row != range->v_range.cell.a.row ||
-	    state->sel->v_range.cell.b.col != range->v_range.cell.b.col ||
-	    state->sel->v_range.cell.b.row != range->v_range.cell.b.row) {
-		value_release (state->sel);
-		state->sel = range;
-	} else {
-		value_release (state->sel);
-		state->sel = range;
-	}
+	if (state->sel != NULL)
+	  	value_release (state->sel);
+	state->sel = range;
 	load_model_data(state);
 }
 
@@ -764,23 +750,25 @@ build_sort_field_base_menu (SortFlowState *state)
 	gint index;
 
 	GtkWidget *menu = gtk_menu_new ();
-	GList* items;
+	GList* items = NULL;
 
-	if (state->is_cols) {
-		start = state->sel->v_range.cell.a.col;
-		end  = state->sel->v_range.cell.b.col;
-		index = state->sel->v_range.cell.a.row;
-	} else {
-		start = state->sel->v_range.cell.a.row;
-		end  = state->sel->v_range.cell.b.row;
-		index = state->sel->v_range.cell.a.col;
-	}
-
-	build_sort_field_menu (start, end, index, menu, state, 
+	if (state->sel != NULL) {
+	  if (state->is_cols) {
+	    start = state->sel->v_range.cell.a.col;
+	    end  = state->sel->v_range.cell.b.col;
+	    index = state->sel->v_range.cell.a.row;
+	  } else {
+	    start = state->sel->v_range.cell.a.row;
+	    end  = state->sel->v_range.cell.b.row;
+	    index = state->sel->v_range.cell.a.col;
+	  }
+	  
+	  build_sort_field_menu (start, end, index, menu, state, 
 			       state->sort_items);
-
-	items = gtk_container_get_children (GTK_CONTAINER (menu));
-
+	  
+	  items = gtk_container_get_children (GTK_CONTAINER (menu));
+	}
+	
 	if (items == NULL) {
 		GtkWidget *item;
 		item = (GtkWidget *) gtk_menu_item_new_with_label(state->is_cols ? 
@@ -994,7 +982,7 @@ dialog_init (SortFlowState *state)
 	gnm_expr_entry_set_update_policy (state->range_entry, GTK_UPDATE_DISCONTINUOUS);
 	gtk_widget_show (GTK_WIDGET (state->range_entry));
 	g_signal_connect_swapped (G_OBJECT (state->range_entry),
-		"update",
+		"changed",
 		G_CALLBACK (cb_update_sensitivity), state);
 
 	table = GTK_TABLE (glade_xml_get_widget (state->gui, "cell_sort_spec_table"));
@@ -1092,6 +1080,8 @@ dialog_init (SortFlowState *state)
 
 	state->cell_sort_header_check = glade_xml_get_widget (state->gui,
 							      "cell_sort_header_check");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (state->cell_sort_header_check),
+				      gnm_app_prefs->sort_default_has_header);
 	g_signal_connect_swapped (G_OBJECT (state->cell_sort_header_check),
 		"toggled",
 		G_CALLBACK (cb_sort_header_check), state);
@@ -1183,7 +1173,7 @@ dialog_cell_sort (WorkbookControlGUI *wbcg)
 
 	state = g_new (SortFlowState, 1);
 	state->wbcg  = wbcg;
-	state->wb    = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb    = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 	state->sv    = wb_control_cur_sheet_view (WORKBOOK_CONTROL (wbcg));
 	state->sheet = sv_sheet (state->sv);
 	state->warning_dialog = NULL;
