@@ -32,6 +32,7 @@
 #include <func.h>
 #include <gui-util.h>
 #include <ranges.h>
+#include <value.h>
 #include <sheet-view.h>
 #include <selection.h>
 #include <widgets/gnumeric-expr-entry.h>
@@ -185,7 +186,7 @@ construct_consolidate (ConsolidateState *state, data_analysis_output_t  *dao)
 				    SOURCE_COLUMN, &source,
 				    -1);
 		if (strlen(source) != 0) {
-			range_value = global_range_parse (state->base.sheet, source);
+			range_value = value_new_cellrange_str (state->base.sheet, source);
 			
 			if (range_value == NULL) {
 				state->construct_error = g_strdup_printf (
@@ -302,7 +303,7 @@ cb_consolidate_ok_clicked (GtkWidget *button, ConsolidateState *state)
 	 * a suitable error message
 	 */
 	if (cs == NULL) {
-		gnumeric_notice_nonmodal (GTK_WINDOW (state->base.dialog),
+		go_gtk_notice_nonmodal_dialog (GTK_WINDOW (state->base.dialog),
 					  &state->base.warning_dialog,
 					  GTK_MESSAGE_ERROR,
 					  state->construct_error);
@@ -320,7 +321,7 @@ cb_consolidate_ok_clicked (GtkWidget *button, ConsolidateState *state)
 		    (button == state->base.ok_button))
 			gtk_widget_destroy (state->base.dialog);
 	} else {
-		gnumeric_notice_nonmodal (GTK_WINDOW (state->base.dialog),
+		go_gtk_notice_nonmodal_dialog (GTK_WINDOW (state->base.dialog),
 					  &state->base.warning_dialog,
 					  GTK_MESSAGE_ERROR,
 					  _("The output range overlaps "
@@ -518,16 +519,11 @@ dialog_consolidate_tool_init (ConsolidateState *state)
 	 */
 	if ((r = selection_first_range (state->base.sv, NULL, NULL)) != NULL 
 	    && !range_is_singleton (r))
-		selection_foreach_range (state->base.sv, TRUE, &add_source_area, state);
+		sv_selection_foreach (state->base.sv, &add_source_area, state);
 
 	adjust_source_areas (state);
-
-	gtk_widget_grab_focus   (GTK_WIDGET (state->function));
-
-	dialog_set_button_sensitivity(NULL, state);
-
+	dialog_set_button_sensitivity (NULL, state);
 	state->base.state_destroy = (state_destroy_t)cb_dialog_destroy;
-	
 }
 
 void
