@@ -123,7 +123,7 @@ dialog_row_height_set_value (gnm_float value, RowHeightState *state)
 static void
 dialog_row_height_load_value (RowHeightState *state)
 {
-	GList *l;
+	GSList *l;
 	gnm_float value = 0.0;
 	state->orig_is_default = TRUE;
 	state->orig_some_default = FALSE;
@@ -259,7 +259,7 @@ dialog_row_height (WorkbookControlGUI *wbcg, gboolean use_default)
 	state->sv = wb_control_cur_sheet_view (WORKBOOK_CONTROL (wbcg));
 	state->sheet = sv_sheet (state->sv);
 	state->adjusting = FALSE;
-	state->gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	state->gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"row-height.glade", NULL, NULL);
 	g_return_if_fail (state->gui != NULL);
 
