@@ -302,10 +302,11 @@ populate_sheet_list (SheetManager *state)
 	GtkTreeIter iter;
 	GtkWidget *scrolled = glade_xml_get_widget (state->gui, "scrolled");
 	WorkbookControl *wbc = WORKBOOK_CONTROL (state->wbcg);
-	Workbook *wb = wb_control_workbook (wbc);
+	Workbook *wb = wb_control_get_workbook (wbc);
 	Sheet *cur_sheet = wb_control_cur_sheet (wbc);
 	int i, n = workbook_sheet_count (wb);
 	GtkCellRenderer *renderer;
+	GtkTreePath *sel_path = NULL;
 
 	state->model = gtk_list_store_new (NUM_COLMNS,
 					   G_TYPE_BOOLEAN,
@@ -319,7 +320,7 @@ populate_sheet_list (SheetManager *state)
 					   G_TYPE_BOOLEAN,
 					   GDK_TYPE_COLOR,
 					   GDK_TYPE_COLOR,
-                       G_TYPE_BOOLEAN,
+					   G_TYPE_BOOLEAN,
 					   GDK_TYPE_PIXBUF);
 	state->sheet_list = GTK_TREE_VIEW (gtk_tree_view_new_with_model
 					   (GTK_TREE_MODEL (state->model)));
@@ -358,7 +359,8 @@ populate_sheet_list (SheetManager *state)
 							    : state->image_ltr),
 				    -1);
 		if (sheet == cur_sheet)
-			gtk_tree_selection_select_iter (selection, &iter);
+			sel_path = gtk_tree_model_get_path (GTK_TREE_MODEL (state->model),
+							    &iter);
 		state->old_order = g_slist_prepend (state->old_order, sheet);
 	}
 
@@ -432,6 +434,11 @@ populate_sheet_list (SheetManager *state)
 		"changed",
 		G_CALLBACK (cb_selection_changed), state);
 
+	if (sel_path) {
+		gtk_tree_selection_select_path (selection, sel_path);
+		gtk_tree_path_free (sel_path);
+	}
+
 	gtk_container_add (GTK_CONTAINER (scrolled), GTK_WIDGET (state->sheet_list));
 }
 
@@ -519,8 +526,8 @@ cb_add_clicked (G_GNUC_UNUSED GtkWidget *ignore, SheetManager *state)
 			    SHEET_POINTER, NULL,
 			    IS_EDITABLE_COLUMN,	TRUE,
 			    IS_DELETED,	FALSE,
-			    BACKGROUND_COLOUR, 0,
-			    FOREGROUND_COLOUR, 0,
+			    BACKGROUND_COLOUR, NULL,
+			    FOREGROUND_COLOUR, NULL,
 			    SHEET_DIRECTION, FALSE,
 			    SHEET_DIRECTION_IMAGE, state->image_ltr,
 			    -1);
@@ -671,7 +678,7 @@ static void
 cb_ok_clicked (G_GNUC_UNUSED GtkWidget *ignore, SheetManager *state)
 {
 	WorkbookControl *wbc = WORKBOOK_CONTROL (state->wbcg);
-	Workbook *wb = wb_control_workbook (wbc);
+	Workbook *wb = wb_control_get_workbook (wbc);
 	char *error;
 	gboolean changed;
 	WorkbookSheetState *old_state;
@@ -726,7 +733,7 @@ cb_ok_clicked (G_GNUC_UNUSED GtkWidget *ignore, SheetManager *state)
 				back ? style_color_new_gdk (back) : NULL;
 
 			if (!this_sheet)
-				this_sheet = workbook_sheet_add (wb, i, FALSE);
+				this_sheet = workbook_sheet_add (wb, i);
 
 			g_object_set (this_sheet,
 				      "protected", is_locked,
@@ -754,14 +761,14 @@ cb_ok_clicked (G_GNUC_UNUSED GtkWidget *ignore, SheetManager *state)
 		n++;
 	}
 
-	cmd_reorganize_sheets2 (wbc, old_state);
+	cmd_reorganize_sheets (wbc, old_state, NULL);
 	gtk_widget_destroy (GTK_WIDGET (state->dialog));
 }
 
 static void
 cb_sheet_order_destroy (SheetManager *state)
 {
-	Workbook *wb = wb_control_workbook (WORKBOOK_CONTROL (state->wbcg));
+	Workbook *wb = wb_control_get_workbook (WORKBOOK_CONTROL (state->wbcg));
 
 	/* Stop to listen to changes in the sheet order. */
 	if (state->sheet_order_changed_listener)
@@ -805,7 +812,7 @@ dialog_sheet_order_update_sheet_order (SheetManager *state)
 	gboolean is_rtl;
 	GdkColor *back, *fore;
 	GtkTreeIter iter;
-	Workbook *wb = wb_control_workbook (WORKBOOK_CONTROL (state->wbcg));
+	Workbook *wb = wb_control_get_workbook (WORKBOOK_CONTROL (state->wbcg));
 	gint i, j, n_sheets, n_children;
 	GtkTreeModel *model = GTK_TREE_MODEL (state->model);
 	Sheet *sheet_wb, *sheet_model;
@@ -994,7 +1001,7 @@ dialog_sheet_order (WorkbookControlGUI *wbcg)
                                              GTK_ICON_SIZE_LARGE_TOOLBAR,
                                              "Gnumeric-Sheet-Manager");
 	/* Listen for changes in the sheet order. */
-	wb = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	wb = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 	state->sheet_order_changed_listener = g_signal_connect (G_OBJECT (wb),
 		"sheet_order_changed", G_CALLBACK (cb_sheet_order_changed),
 		state);
