@@ -106,7 +106,7 @@ typedef struct {
 } algorithm_def_t;
 
 static const algorithm_def_t algorithm_defs [] = {
-	{ N_("Simplex (LP Solve 5.0)"), LPSolve, SolverLPModel },
+	{ N_("Simplex (LP Solve)"), LPSolve, SolverLPModel },
 	{ N_("Revised Simplex (GLPK 4.5)"), GLPKSimplex, SolverLPModel },
 	{ N_("< Not available >"), QPDummy, SolverQPModel },
 	{ NULL, 0, 0 }
@@ -177,7 +177,7 @@ is_hom_row_or_col_ref (GnmExprEntry *entry_1, GnmExprEntry *entry_2,
 	return res;
 }
 
-static void
+static gboolean
 dialog_set_sec_button_sensitivity (G_GNUC_UNUSED GtkWidget *dummy,
 				   SolverState *state)
 {
@@ -197,6 +197,9 @@ dialog_set_sec_button_sensitivity (G_GNUC_UNUSED GtkWidget *dummy,
 	gtk_widget_set_sensitive (state->add_button, ready);
 	gtk_widget_set_sensitive (state->change_button, select_ready && ready);
 	gtk_widget_set_sensitive (state->delete_button, select_ready);
+
+	/* Return TRUE iff the current constraint is valid.  */
+	return ready;
 }
 
 static void
@@ -257,17 +260,18 @@ release_constraint (constraint_t * data)
 static void
 cb_dialog_delete_clicked (G_GNUC_UNUSED GtkWidget *button, SolverState *state)
 {
-	GtkTreeIter iter;
-	GtkTreeModel *store;
 	if (state->constr != NULL) {
+		GtkTreeIter iter;
+		GtkTreeModel *store;
+
 		release_constraint (state->constr);
 		state->constr = NULL;
-	} else
-		return;
-	gtk_tree_selection_get_selected (
-		gtk_tree_view_get_selection (state->constraint_list),
-		&store, &iter);
-	gtk_list_store_remove ((GtkListStore*)store, &iter);
+
+		if (gtk_tree_selection_get_selected (
+			    gtk_tree_view_get_selection (state->constraint_list),
+			    &store, &iter))
+			gtk_list_store_remove ((GtkListStore*)store, &iter);
+	}
 }
 
 static  constraint_t*
@@ -318,26 +322,30 @@ constraint_fill_row (SolverState *state, GtkListStore *store, GtkTreeIter *iter)
 static void
 cb_dialog_add_clicked (SolverState *state)
 {
-	GtkTreeIter   iter;
-	GtkListStore *store = GTK_LIST_STORE (gtk_tree_view_get_model (state->constraint_list));
-	gtk_list_store_append (store, &iter);
-	constraint_fill_row (state, store, &iter);
+	if (dialog_set_sec_button_sensitivity (NULL, state)) {
+		GtkTreeIter   iter;
+		GtkListStore *store = GTK_LIST_STORE (gtk_tree_view_get_model (state->constraint_list));
+
+		gtk_list_store_append (store, &iter);
+		constraint_fill_row (state, store, &iter);
+	}
 }
 
 static void
 cb_dialog_change_clicked (GtkWidget *button, SolverState *state)
 {
-	GtkTreeIter iter;
-	GtkTreeModel *store;
 	if (state->constr != NULL) {
+		GtkTreeIter iter;
+		GtkTreeModel *store;
+
 		release_constraint (state->constr);
 		state->constr = NULL;
-	} else
-		return;
-	gtk_tree_selection_get_selected (
-		gtk_tree_view_get_selection (state->constraint_list),
-		&store, &iter);
-	state->constr = constraint_fill_row (state, (GtkListStore *)store, &iter);
+
+		if (gtk_tree_selection_get_selected (
+			    gtk_tree_view_get_selection (state->constraint_list),
+			    &store, &iter))
+			state->constr = constraint_fill_row (state, (GtkListStore *)store, &iter);
+	}
 }
 
 static void
@@ -527,20 +535,31 @@ check_int_constraints (GnmValue *input_range, SolverState *state)
 {
 	GtkTreeModel *store;
 	GtkTreeIter iter;
-	const constraint_t *a_constraint;
-	const gchar *text;
 
 	store = gtk_tree_view_get_model (state->constraint_list);
 	if (gtk_tree_model_get_iter_first (store, &iter))
 		do {
+			const constraint_t *a_constraint;
+			gchar *text;
+
 			gtk_tree_model_get (store, &iter, 0, &text, 1, &a_constraint, -1);
-			if (a_constraint == NULL)
+			if (a_constraint == NULL) {
+				g_free (text);
 				break;
+			}
+
 			if ((a_constraint->type != SolverINT) &&
-				(a_constraint->type != SolverBOOL))
+			    (a_constraint->type != SolverBOOL)) {
+				g_free (text);
 				continue;
-			if (!global_range_contained (state->sheet, a_constraint->lhs_value, input_range))
+			}
+
+			if (!global_range_contained (state->sheet,
+						     a_constraint->lhs_value,
+						     input_range))
 				return text;
+
+			g_free (text);
 		} while (gtk_tree_model_iter_next (store, &iter));
 	return NULL;
 }
@@ -665,19 +684,6 @@ save_original_values (GSList *input_cells)
 	return ov;
 }
 
-/**
- * cb_destroy:
- * @data:
- * @user_data:
- *
- *
- **/
-static void
-cb_destroy (gpointer data, G_GNUC_UNUSED gpointer user_data)
-{
-	g_free (data);
-}
-
 
 /* Returns FALSE if the reports deleted the current sheet
  * and forced the dialog to die */
@@ -764,7 +770,7 @@ solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
 			((GtkWindow *) state->dialog,
 			 &(state->warning_dialog),
 			 GTK_MESSAGE_ERROR,
-			 solver_max_time_err);
+			 SOLVER_MAX_TIME_ERR);
 		err = solver_reports (WORKBOOK_CONTROL(state->wbcg),
 				      state->sheet, res,
 				      FALSE, FALSE, FALSE,
@@ -790,18 +796,18 @@ solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
 }
 
 static void
-solver_add_scenario (SolverState *state, SolverResults *res, gchar *name)
+solver_add_scenario (SolverState *state, SolverResults *res, const gchar *name)
 {
 	SolverParameters *param = res->param;
-	GnmValue            *input_range;
+	GnmValue         *input_range;
 	const gchar      *comment = _("Optimal solution created by solver.\n");
 	scenario_t       *scenario;
 
 	input_range = gnm_expr_entry_parse_as_value (state->change_cell_entry,
 						     state->sheet);
 
-	scenario_add_new (name, input_range, g_strdup (param->input_entry_str),
-			  g_strdup (comment), state->sheet, &scenario);
+	scenario_add_new (name, input_range, param->input_entry_str,
+			  comment, state->sheet, &scenario);
 	scenario_add (state->sheet, scenario);
 	if (input_range != NULL)
 		value_release (input_range);
@@ -956,11 +962,10 @@ cb_dialog_solve_clicked (G_GNUC_UNUSED GtkWidget *button,
 	conv.sheet     = state->sheet;
 	conv.c_listing = state->constraint_list;
 	convert_constraint_format (&conv);
-	if (param->constraints != NULL) {
-		g_slist_foreach	(param->constraints, cb_destroy, NULL);
-		g_slist_free (param->constraints);
-		param->constraints = NULL;
-	}
+	g_slist_foreach	(param->constraints,
+			 (GFunc)solver_constraint_destroy,
+			 NULL);
+	g_slist_free (param->constraints);
 	param->constraints = conv.c_list;
 	if (param->constraints == NULL) {
 		go_gtk_notice_nonmodal_dialog
@@ -1072,6 +1077,7 @@ dialog_init (SolverState *state)
 				   GNUMERIC_HELP_LINK_SOLVER);
 
 	state->add_button  = glade_xml_get_widget (state->gui, "addbutton");
+	gtk_button_set_alignment (GTK_BUTTON (state->add_button), 0.5, .5);
 	g_signal_connect_swapped (G_OBJECT (state->add_button), "clicked",
 		G_CALLBACK (cb_dialog_add_clicked), state);
 
@@ -1082,6 +1088,7 @@ dialog_init (SolverState *state)
 
 	state->delete_button = glade_xml_get_widget (state->gui,
 						     "deletebutton");
+	gtk_button_set_alignment (GTK_BUTTON (state->delete_button), 0.5, .5);
 	g_signal_connect (G_OBJECT (state->delete_button), "clicked",
 			  G_CALLBACK (cb_dialog_delete_clicked), state);
 
@@ -1334,7 +1341,7 @@ dialog_solver (WorkbookControlGUI *wbcg, Sheet *sheet)
 
 	state                 = g_new (SolverState, 1);
 	state->wbcg           = wbcg;
-	state->wb             = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb             = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 	state->sheet          = sheet;
 	state->ov             = NULL;
 	state->ov_stack       = NULL;
