@@ -4,9 +4,9 @@
 #include "dialog-stf-preview.h"
 #include "gui-util.h"
 #include <stf-parse.h>
-#include "widgets/widget-charmap-selector.h"
-#include "widgets/widget-locale-selector.h"
-#include "widgets/widget-format-selector.h"
+#include <goffice/gtk/go-charmap-sel.h>
+#include <goffice/gtk/go-locale-sel.h>
+#include <goffice/gtk/go-format-sel.h>
 
 #include <gtk/gtkradiobutton.h>
 #include <gtk/gtkspinbutton.h>
@@ -28,18 +28,19 @@ typedef struct {
 	GtkCheckButton  *line_break_mac;
      
 	/* Page members that are created at run-time */
-	CharmapSelector *charmap_selector;
+	GOCharmapSel	*charmap_selector;
 	RenderData_t    *renderdata;
 } MainInfo_t;
 
 /* for the csv_page */
 typedef struct {
 	GtkCheckButton  *csv_tab, *csv_colon, *csv_comma;
-	GtkCheckButton  *csv_space, *csv_semicolon, *csv_pipe;
-	GtkCheckButton  *csv_slash, *csv_hyphen, *csv_bang;
+	GtkCheckButton  *csv_space, *csv_semicolon;
+	GtkCheckButton  *csv_hyphen;
 	GtkCheckButton  *csv_custom;
 	GtkEntry        *csv_customseparator;
 	GtkCheckButton  *csv_duplicates;
+	GtkCheckButton  *csv_trim_seps;
 	GtkCheckButton  *csv_2x_indicator;
 	GtkWidget       *csv_textindicator;
 	GtkEntry        *csv_textfield;
@@ -63,11 +64,11 @@ typedef struct {
 typedef struct {
 	GtkWidget         *format_data_container;
         GtkWidget	  *format_trim;
-	NumberFormatSelector *format_selector;
+	GOFormatSel	  *format_selector;
 	GtkWidget         *column_selection_label;
      
 	/* Page members that are created at run-time */
-	LocaleSelector    *locale_selector;
+	GOLocaleSel	  *locale_selector;
 	RenderData_t      *renderdata;
 	GPtrArray         *formats; /* Contains GnmFormat* */
 	int                index;
@@ -122,7 +123,6 @@ typedef struct {
 	FixedInfo_t           fixed;
 	FormatInfo_t          format;
 
-	gboolean              canceled;   /* Indicates whether the user pressed cancel button */
 	StfParseOptions_t    *parseoptions;
 } StfDialogData;
 
