@@ -68,7 +68,7 @@ typedef struct {
 		gboolean	recalc_auto;
 		gboolean	iteration_enabled;
 		int		max_iterations;
-		double		iteration_tolerance;
+		gnm_float	iteration_tolerance;
 	} old;
 } AttrState;
 
@@ -95,7 +95,7 @@ cb_page_select (G_GNUC_UNUSED GtkNotebook *notebook,
 /*****************************************************************************/
 
 static void
-get_entry_values (AttrState *state, int *max_iterations, double *iteration_tolerance)
+get_entry_values (AttrState *state, int *max_iterations, gnm_float *iteration_tolerance)
 {
 	if (!entry_to_int (state->view.max_iterations, max_iterations, TRUE))
 		*max_iterations = state->old.max_iterations;
@@ -108,7 +108,7 @@ cb_widget_changed (G_GNUC_UNUSED GtkWidget *widget, AttrState *state)
 {
 	gboolean changed;
 	int max_iterations;
-	double iteration_tolerance;
+	gnm_float iteration_tolerance;
 
 	get_entry_values (state, &max_iterations, &iteration_tolerance);
 	changed =
@@ -157,7 +157,6 @@ cb_attr_dialog_dialog_apply (G_GNUC_UNUSED GtkWidget *button,
 	workbook_iteration_max_number	(state->wb, state->old.max_iterations);
 	workbook_iteration_tolerance	(state->wb, state->old.iteration_tolerance);
 
-	wb_view_prefs_update (state->wbv);
 	cb_widget_changed (NULL, state);
 }
 
@@ -331,7 +330,7 @@ dialog_workbook_attr (WorkbookControlGUI *wbcg)
 	if (gnumeric_dialog_raise_if_exists (wbcg, WORKBOOK_ATTRIBUTE_KEY))
 		return;
 
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"workbook-attr.glade", NULL, NULL);
         if (gui == NULL)
                 return;
@@ -341,7 +340,7 @@ dialog_workbook_attr (WorkbookControlGUI *wbcg)
 	state->gui = gui;
 	state->wbcg = wbcg;
 	state->wbv  = wb_control_view (WORKBOOK_CONTROL (wbcg));
-	state->wb   = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb   = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 
 	attr_dialog_impl (state);
 }
