@@ -5,94 +5,83 @@
 #include "numbers.h"
 #include "parse-util.h"
 
-struct _GnmExprConstant {
-	GnmExprOp oper;
-	int       ref_count;
 
+
+struct _GnmExprConstant {
+	guint8 oper;
 	GnmValue const *value;
 };
 
 struct _GnmExprFunction {
-	GnmExprOp oper;
-	int       ref_count;
-
-	GnmFunc     *func;
-	GnmExprList *arg_list;
+	guint8 oper;
+	int argc;
+	GnmFunc *func;
+	GnmExprConstPtr *argv;
 };
 
 struct _GnmExprUnary {
-	GnmExprOp oper;
-	int       ref_count;
-
+	guint8 oper;
 	GnmExpr const *value;
 };
 
 struct _GnmExprBinary {
-	GnmExprOp oper;
-	int       ref_count;
-
+	guint8 oper;
 	GnmExpr const *value_a;
 	GnmExpr const *value_b;
 };
 
 /* We could break this out into multiple types to be more space efficient */
 struct _GnmExprName {
-	GnmExprOp oper;
-	int       ref_count;
-
-	Sheet	     *optional_scope;
-	Workbook     *optional_wb_scope;
+	guint8 oper;
+	Sheet *optional_scope;
+	Workbook *optional_wb_scope;
 	GnmNamedExpr *name;
 };
 
 struct _GnmExprCellRef {
-	GnmExprOp oper;
-	int       ref_count;
-
+	guint8 oper;
 	GnmCellRef ref;
 };
 
-struct _GnmExprArray {
-	GnmExprOp oper;
-	int       ref_count;
-
-	int x, y;
-	int cols, rows;
-	struct {
-		/* Upper left corner */
-		GnmValue *value;	/* Last array result */
-		GnmExpr const *expr;	/* Real Expression */
-	} corner;
+struct _GnmExprArrayCorner {
+	guint8 oper;
+	guint32 cols, rows;
+	GnmValue *value;	/* Last array result */
+	GnmExpr const *expr;	/* Real Expression */
+};
+struct _GnmExprArrayElem {
+	guint8 oper;
+	guint32 x, y;
 };
 
 struct _GnmExprSet {
-	GnmExprOp oper;
-	int       ref_count;
-
-	GnmExprList *set;
+	guint8 oper;
+	int argc;
+	GnmExprConstPtr *argv;
 };
 
 union _GnmExpr {
-	struct {
-		GnmExprOp oper;
-		int       ref_count;
-	} any;
-
+	guint8                  oper;
 	GnmExprConstant		constant;
 	GnmExprFunction		func;
 	GnmExprUnary		unary;
 	GnmExprBinary		binary;
 	GnmExprName		name;
 	GnmExprCellRef		cellref;
-	GnmExprArray		array;
+	GnmExprArrayCorner	array_corner;
+	GnmExprArrayElem	array_elem;
 	GnmExprSet		set;
 };
 
+#define GNM_EXPR_GET_OPER(e) ((GnmExprOp)((e)->oper))
+
 #define gnm_expr_constant_init(expr, val)	\
 do {						\
-	(expr)->ref_count = 1;			\
-	(expr)->oper = GNM_EXPR_OP_CONSTANT;		\
+	(expr)->oper = GNM_EXPR_OP_CONSTANT;	\
 	(expr)->value = val;			\
 } while (0)
 
+
+#define GNM_EXPR_TOP_MAGIC 0x01123622
+
 #endif /* GNUMERIC_EXPR_IMPL_H */
