@@ -28,6 +28,7 @@
 #include "sheet-style.h"
 
 #include <gdk/gdkkeysyms.h>
+#include <goffice/utils/go-glib-extras.h>
 
 static void
 cb_nexpr_remove (GnmNamedExpr *nexpr)
@@ -39,6 +40,46 @@ cb_nexpr_remove (GnmNamedExpr *nexpr)
 	expr_name_unref (nexpr);
 }
 
+static void
+cb_collect_name_deps (gpointer key, G_GNUC_UNUSED gpointer value,
+		      gpointer user_data)
+{
+	GSList **list = user_data;
+	*list = g_slist_prepend (*list, key);
+}
+
+static GSList *
+expr_name_unlink_deps (GnmNamedExpr *nexpr)
+{
+	GSList *ptr, *deps = NULL;
+
+	if (nexpr->dependents == NULL)
+		return NULL;
+
+	g_hash_table_foreach (nexpr->dependents, cb_collect_name_deps, &deps);
+
+	/* pull them out */
+	for (ptr = deps ; ptr != NULL ; ptr = ptr->next) {
+		GnmDependent *dep = ptr->data;
+		if (dependent_is_linked (dep))
+			dependent_unlink (dep);
+	}
+	return deps;
+}
+
+static void
+expr_name_relink_deps (GnmNamedExpr *nexpr)
+{
+	GSList *deps = NULL;
+
+	if (nexpr->dependents == NULL)
+		return;
+
+	g_hash_table_foreach (nexpr->dependents, cb_collect_name_deps, &deps);
+	dependents_link (deps);
+	g_slist_free (deps);
+}
+
 static GnmNamedExprCollection *
 gnm_named_expr_collection_new (void)
 {
@@ -54,9 +95,9 @@ gnm_named_expr_collection_new (void)
 
 /**
  * gnm_named_expr_collection_free :
- * @names : a POINTER to the collection of names
+ * @names : The collection of names
  *
- * Frees names in the local scope.
+ * Frees names defined in the local scope.
  * NOTE : THIS DOES NOT INVALIDATE NAMES THAT REFER
  *        TO THIS SCOPE.
  *        eg
@@ -64,13 +105,57 @@ gnm_named_expr_collection_new (void)
  *           to sheet1.  That will remain!
  **/
 void
-gnm_named_expr_collection_free (GnmNamedExprCollection **names)
+gnm_named_expr_collection_free (GnmNamedExprCollection *names)
 {
-	if (*names != NULL) {
-		g_hash_table_destroy ((*names)->names);
-		g_hash_table_destroy ((*names)->placeholders);
-		g_free (*names);
-		*names = NULL;
+	if (names != NULL) {
+		g_hash_table_destroy (names->names);
+		g_hash_table_destroy (names->placeholders);
+		g_free (names);
+	}
+}
+
+static void
+cb_unlink_all_names (G_GNUC_UNUSED gpointer key,
+		     gpointer value,
+		     G_GNUC_UNUSED gpointer user_data)
+{
+	GnmNamedExpr *nexpr = value;
+	GSList *deps = expr_name_unlink_deps (nexpr);
+	g_slist_free (deps);
+}
+
+void
+gnm_named_expr_collection_unlink (GnmNamedExprCollection *names)
+{
+	if (!names)
+		return;
+
+	if (names->names) {
+		g_hash_table_foreach (names->names,
+				      cb_unlink_all_names,
+				      NULL);
+	}
+}
+
+static void
+cb_relink_all_names (G_GNUC_UNUSED gpointer key,
+		     gpointer value,
+		     G_GNUC_UNUSED gpointer user_data)
+{
+	GnmNamedExpr *nexpr = value;
+	expr_name_relink_deps (nexpr);
+}
+
+void
+gnm_named_expr_collection_relink (GnmNamedExprCollection *names)
+{
+	if (!names)
+		return;
+
+	if (names->names) {
+		g_hash_table_foreach (names->names,
+				      cb_relink_all_names,
+				      NULL);
 	}
 }
 
@@ -78,10 +163,13 @@ static GnmNamedExpr *
 gnm_named_expr_collection_lookup (GnmNamedExprCollection const *scope,
 				  char const *name)
 {
-	GnmNamedExpr *nexpr = g_hash_table_lookup (scope->names, name);
-	if (nexpr == NULL)
-		nexpr = g_hash_table_lookup (scope->placeholders, name);
-	return nexpr;
+	if (scope != NULL) {
+		GnmNamedExpr *nexpr = g_hash_table_lookup (scope->names, name);
+		if (nexpr == NULL)
+			nexpr = g_hash_table_lookup (scope->placeholders, name);
+		return nexpr;
+	} else
+		return NULL;
 }
 
 static void
@@ -110,7 +198,7 @@ cb_check_name (G_GNUC_UNUSED gpointer key, GnmNamedExpr *nexpr,
 	if (!nexpr->active || nexpr->is_hidden)
 		return;
 
-	v = gnm_expr_get_range (nexpr->expr);
+	v = gnm_expr_top_get_range (nexpr->texpr);
 	if (v != NULL) {
 		if (v->type == VALUE_CELLRANGE) {
 			GnmRangeRef const *ref = &v->v_range.cell;
@@ -129,6 +217,7 @@ cb_check_name (G_GNUC_UNUSED gpointer key, GnmNamedExpr *nexpr,
 		value_release (v);
 	}
 }
+
 static GnmNamedExpr *
 gnm_named_expr_collection_check (GnmNamedExprCollection *scope,
 				 Sheet const *sheet, GnmRange const *r)
@@ -149,37 +238,10 @@ gnm_named_expr_collection_check (GnmNamedExprCollection *scope,
 
 /******************************************************************************/
 
-static void
-cb_collect_name_deps (gpointer key, G_GNUC_UNUSED gpointer value,
-		      gpointer user_data)
-{
-	GSList **list = user_data;
-	*list = g_slist_prepend (*list, key);
-}
-
-static GSList *
-expr_name_unlink_deps (GnmNamedExpr *nexpr)
-{
-	GSList *ptr, *deps = NULL;
-
-	if (nexpr->dependents == NULL)
-		return NULL;
-
-	g_hash_table_foreach (nexpr->dependents, cb_collect_name_deps, &deps);
-
-	/* pull them out */
-	for (ptr = deps ; ptr != NULL ; ptr = ptr->next) {
-		GnmDependent *dep = ptr->data;
-		if (dependent_is_linked (dep))
-			dependent_unlink (dep, NULL);
-	}
-	return deps;
-}
-
 /**
  * expr_name_handle_references : register or unregister a name with
  *    all of the sheets it explicitly references.  This is necessary
- *    beacuse names are not dependents, and if they reference a deleted
+ *    because names are not dependents, and if they reference a deleted
  *    sheet we will not notice.
  */
 static void
@@ -187,7 +249,7 @@ expr_name_handle_references (GnmNamedExpr *nexpr, gboolean add)
 {
 	GSList *sheets, *ptr;
 
-	sheets = gnm_expr_referenced_sheets (nexpr->expr);
+	sheets = gnm_expr_top_referenced_sheets (nexpr->texpr);
 	for (ptr = sheets ; ptr != NULL ; ptr = ptr->next) {
 		Sheet *sheet = ptr->data;
 		GnmNamedExpr *found;
@@ -247,8 +309,8 @@ expr_name_lookup (GnmParsePos const *pp, char const *name)
  * expr_name_new :
  *
  * Creates a new name without linking it into any container.
- */
-static GnmNamedExpr *
+ **/
+GnmNamedExpr *
 expr_name_new (char const *name, gboolean is_placeholder)
 {
 	GnmNamedExpr *nexpr;
@@ -260,7 +322,7 @@ expr_name_new (char const *name, gboolean is_placeholder)
 	nexpr->ref_count	= 1;
 	nexpr->active		= FALSE;
 	nexpr->name		= gnm_string_get (name);
-	nexpr->expr		= NULL;
+	nexpr->texpr		= NULL;
 	nexpr->dependents	= NULL;
 	nexpr->is_placeholder	= is_placeholder;
 	nexpr->is_hidden	= FALSE;
@@ -274,42 +336,41 @@ expr_name_new (char const *name, gboolean is_placeholder)
  * NB. if we already have a circular reference in addition
  * to this one we are checking we will come to serious grief.
  */
-gboolean
-expr_name_check_for_loop (char const *name, GnmExpr const *expr)
+static gboolean
+do_expr_name_loop_check (char const *name, GnmExpr const *expr)
 {
-	g_return_val_if_fail (expr != NULL, TRUE);
-
-	switch (expr->any.oper) {
+	switch (GNM_EXPR_GET_OPER (expr)) {
 	case GNM_EXPR_OP_RANGE_CTOR:
 	case GNM_EXPR_OP_INTERSECT:
 	case GNM_EXPR_OP_ANY_BINARY:
-		return (expr_name_check_for_loop (name, expr->binary.value_a) ||
-			expr_name_check_for_loop (name, expr->binary.value_b));
+		return (do_expr_name_loop_check (name, expr->binary.value_a) ||
+			do_expr_name_loop_check (name, expr->binary.value_b));
 	case GNM_EXPR_OP_ANY_UNARY:
-		return expr_name_check_for_loop (name, expr->unary.value);
+		return do_expr_name_loop_check (name, expr->unary.value);
 	case GNM_EXPR_OP_NAME: {
 		GnmNamedExpr const *nexpr = expr->name.name;
 		if (!strcmp (nexpr->name->str, name))
 			return TRUE;
-
-		/* And look inside this name tree too */
-		return expr_name_check_for_loop (name, nexpr->expr);
+		if (nexpr->texpr != NULL) /* look inside this name tree too */
+			return expr_name_check_for_loop (name, nexpr->texpr);
+		return FALSE;
 	}
 	case GNM_EXPR_OP_FUNCALL: {
-		GnmExprList *l = expr->func.arg_list;
-		for (; l ; l = l->next)
-			if (expr_name_check_for_loop (name, l->data))
+		int i;
+		for (i = 0; i < expr->func.argc; i++)
+			if (do_expr_name_loop_check (name, expr->func.argv[i]))
 				return TRUE;
 		break;
 	}
 	case GNM_EXPR_OP_CONSTANT:
 	case GNM_EXPR_OP_CELLREF:
-	case GNM_EXPR_OP_ARRAY:
+	case GNM_EXPR_OP_ARRAY_CORNER:
+	case GNM_EXPR_OP_ARRAY_ELEM:
 		break;
 	case GNM_EXPR_OP_SET: {
-		GnmExprList *l = expr->set.set;
-		for (; l ; l = l->next)
-			if (expr_name_check_for_loop (name, l->data))
+		int i;
+		for (i = 0; i < expr->set.argc; i++)
+			if (do_expr_name_loop_check (name, expr->set.argv[i]))
 				return TRUE;
 		break;
 	}
@@ -317,15 +378,32 @@ expr_name_check_for_loop (char const *name, GnmExpr const *expr)
 	return FALSE;
 }
 
+gboolean
+expr_name_check_for_loop (char const *name, GnmExprTop const *texpr)
+{
+	g_return_val_if_fail (texpr != NULL, TRUE);
+
+	return do_expr_name_loop_check (name, texpr->expr);
+}
+
+static void
+expr_name_queue_deps (GnmNamedExpr *nexpr)
+{
+	if (nexpr->dependents)
+		g_hash_table_foreach (nexpr->dependents,
+				      (GHFunc)dependent_queue_recalc,
+				      NULL);
+}
+
 /**
  * expr_name_add:
  * @pp:
  * @name:
- * @expr: if expr == NULL then create a placeholder with value #NAME?
+ * @texpr: if texpr == NULL then create a placeholder with value #NAME?
  * @error_msg:
  * @link_to_container:
  *
- * Absorbs the reference to @expr.
+ * Absorbs the reference to @texpr.
  * If @error_msg is non NULL it may hold a pointer to a translated descriptive
  * string.  NOTE : caller is responsible for freeing the error message.
  *
@@ -338,8 +416,9 @@ expr_name_check_for_loop (char const *name, GnmExpr const *expr)
  **/
 GnmNamedExpr *
 expr_name_add (GnmParsePos const *pp, char const *name,
-	       GnmExpr const *expr, char **error_msg,
-	       gboolean link_to_container)
+	       GnmExprTop const *texpr, char **error_msg,
+	       gboolean link_to_container,
+	       GnmNamedExpr *stub)
 {
 	GnmNamedExpr *nexpr = NULL;
 	GnmNamedExprCollection *scope = NULL;
@@ -347,9 +426,10 @@ expr_name_add (GnmParsePos const *pp, char const *name,
 	g_return_val_if_fail (pp != NULL, NULL);
 	g_return_val_if_fail (pp->sheet != NULL || pp->wb != NULL, NULL);
 	g_return_val_if_fail (name != NULL, NULL);
+	g_return_val_if_fail (stub == NULL || stub->is_placeholder, NULL);
 
-	if (expr != NULL && expr_name_check_for_loop (name, expr)) {
-		gnm_expr_unref (expr);
+	if (texpr != NULL && expr_name_check_for_loop (name, texpr)) {
+		gnm_expr_top_unref (texpr);
 		if (error_msg)
 			*error_msg = g_strdup_printf (_("'%s' has a circular reference"), name);
 		return NULL;
@@ -360,7 +440,7 @@ expr_name_add (GnmParsePos const *pp, char const *name,
 		/* see if there was a place holder */
 		nexpr = g_hash_table_lookup (scope->placeholders, name);
 		if (nexpr != NULL) {
-			if (expr == NULL) {
+			if (texpr == NULL) {
 				/* there was already a placeholder for this */
 				expr_name_ref (nexpr);
 				return nexpr;
@@ -377,7 +457,7 @@ expr_name_add (GnmParsePos const *pp, char const *name,
 						? g_strdup_printf (_("'%s' is already defined in sheet"), name)
 						: g_strdup_printf (_("'%s' is already defined in workbook"), name);
 
-				gnm_expr_unref (expr);
+				gnm_expr_top_unref (texpr);
 				return NULL;
 			}
 		}
@@ -389,13 +469,21 @@ expr_name_add (GnmParsePos const *pp, char const *name,
 	if (error_msg)
 		*error_msg = NULL;
 
-	if (nexpr == NULL)
-		nexpr = expr_name_new (name, expr == NULL);
+	if (nexpr == NULL) {
+		if (stub != NULL) {
+			nexpr = stub;
+			stub->is_placeholder = FALSE;
+			gnm_string_unref (stub->name);
+			stub->name = gnm_string_get (name);
+		} else
+			nexpr = expr_name_new (name, texpr == NULL);
+	}
 	parse_pos_init (&nexpr->pos,
 		pp->wb, pp->sheet, pp->eval.col, pp->eval.row);
-	if (expr == NULL)
-		expr = gnm_expr_new_constant (value_new_error_NAME (NULL));
-	expr_name_set_expr (nexpr, expr);
+	if (texpr == NULL)
+		texpr = gnm_expr_top_new_constant
+			(value_new_error_NAME (NULL));
+	expr_name_set_expr (nexpr, texpr);
 	if (link_to_container)
 		gnm_named_expr_collection_insert (scope, nexpr);
 
@@ -425,7 +513,7 @@ expr_name_unref (GnmNamedExpr *nexpr)
 		nexpr->name = NULL;
 	}
 
-	if (nexpr->expr != NULL)
+	if (nexpr->texpr != NULL)
 		expr_name_set_expr (nexpr, NULL);
 
 	if (nexpr->dependents != NULL) {
@@ -444,7 +532,7 @@ expr_name_unref (GnmNamedExpr *nexpr)
  * @nexpr :
  *
  * Remove a @nexpr from its container and deactivate it.
- * NOTE : @nexpr may continue to exist if things still have refrences to it,
+ * NOTE : @nexpr may continue to exist if things still have references to it,
  * but they will evaluate to #REF!
  **/
 void
@@ -479,7 +567,7 @@ expr_name_as_string (GnmNamedExpr const *nexpr, GnmParsePos const *pp,
 {
 	if (pp == NULL)
 		pp = &nexpr->pos;
-	return gnm_expr_as_string (nexpr->expr, pp, fmt);
+	return gnm_expr_top_as_string (nexpr->texpr, pp, fmt);
 }
 
 GnmValue *
@@ -491,7 +579,7 @@ expr_name_eval (GnmNamedExpr const *nexpr, GnmEvalPos const *pos,
 	if (!nexpr)
 		return value_new_error_NAME (pos);
 
-	return gnm_expr_eval (nexpr->expr, pos, flags);
+	return gnm_expr_top_eval (nexpr->texpr, pos, flags);
 }
 
 /**
@@ -519,8 +607,9 @@ expr_name_downgrade_to_placeholder (GnmNamedExpr *nexpr)
 	g_hash_table_steal (scope->names, nexpr->name->str);
 
 	nexpr->is_placeholder = TRUE;
-	expr_name_set_expr (nexpr, 
-		gnm_expr_new_constant (value_new_error_NAME (NULL)));
+	expr_name_set_expr
+		(nexpr,
+		 gnm_expr_top_new_constant (value_new_error_NAME (NULL)));
 	gnm_named_expr_collection_insert (scope, nexpr);
 }
 
@@ -577,24 +666,48 @@ expr_name_set_scope (GnmNamedExpr *nexpr, Sheet *sheet)
  * Unrefs the current content of @nexpr and absorbs a ref to @new_expr.
  **/
 void
-expr_name_set_expr (GnmNamedExpr *nexpr, GnmExpr const *new_expr)
+expr_name_set_expr (GnmNamedExpr *nexpr, GnmExprTop const *texpr)
 {
-	GSList *deps = NULL;
+	GSList *good = NULL;
 
 	g_return_if_fail (nexpr != NULL);
 
-	if (new_expr == nexpr->expr)
+	if (texpr == nexpr->texpr)
 		return;
-	if (nexpr->expr != NULL) {
+	if (nexpr->texpr != NULL) {
+		GSList *deps = NULL, *junk = NULL;
+
 		deps = expr_name_unlink_deps (nexpr);
 		expr_name_handle_references (nexpr, FALSE);
-		gnm_expr_unref (nexpr->expr);
-	}
-	nexpr->expr = new_expr;
-	dependents_link (deps, NULL);
+		gnm_expr_top_unref (nexpr->texpr);
 
-	if (new_expr != NULL)
+		/*
+		 * We do not want to relink deps for sheets that are going
+		 * away.  This speeds up exit for workbooks with lots of
+		 * names defined.
+		 */
+		while (deps) {
+			GSList *next = deps->next;
+			GnmDependent *dep = deps->data;
+
+			if (dep->sheet && dep->sheet->being_invalidated)
+				deps->next = junk, junk = deps;
+			else
+				deps->next = good, good = deps;
+
+			deps = next;
+		}
+
+		g_slist_free (junk);
+	}
+	nexpr->texpr = texpr;
+	dependents_link (good);
+	g_slist_free (good);
+
+	if (texpr != NULL)
 		expr_name_handle_references (nexpr, TRUE);
+
+	expr_name_queue_deps (nexpr);
 }
 
 void
@@ -625,7 +738,8 @@ gboolean
 expr_name_is_placeholder (GnmNamedExpr const *nexpr)
 {
 	g_return_val_if_fail (nexpr != NULL, FALSE);
-	return gnm_expr_is_err (nexpr->expr, GNM_ERROR_NAME);
+
+	return gnm_expr_top_is_err (nexpr->texpr, GNM_ERROR_NAME);
 }
 
 int
@@ -646,7 +760,7 @@ expr_name_cmp_by_name (GnmNamedExpr const *a, GnmNamedExpr const *b)
 	}
 
 	if (res == 0)	/* By name.  */
-		res = gnm_utf8_collate_casefold (a->name->str, b->name->str);
+		res = go_utf8_collate_casefold (a->name->str, b->name->str);
 
 	return res;
 }
@@ -711,8 +825,13 @@ sheet_names_check (Sheet const *sheet, GnmRange const *r)
 	tmp = *r;
 	range_normalize (&tmp);
 	nexpr = gnm_named_expr_collection_check (sheet->names, sheet, &tmp);
-	if (nexpr == NULL)
+	if (nexpr == NULL) {
 		nexpr = gnm_named_expr_collection_check (sheet->workbook->names, sheet, &tmp);
+		/* The global name is not accessible if there is a local name (#306685) */
+		if (nexpr != NULL &&
+		    gnm_named_expr_collection_lookup (sheet->names, nexpr->name->str) != NULL)
+			return NULL;
+	}
 
 	return (nexpr != NULL) ? nexpr->name->str : NULL;
 }
