@@ -11,7 +11,7 @@ struct _GnmNamedExpr {
 	GnmString  *name;
 	GnmParsePos    pos;
 	GHashTable *dependents;
-	GnmExpr const *expr;
+	GnmExprTop const *texpr;
 	gboolean    active;
 	gboolean    is_placeholder;
 	gboolean    is_hidden;
@@ -19,7 +19,7 @@ struct _GnmNamedExpr {
 
 GnmNamedExpr *expr_name_lookup (GnmParsePos const *pos, char const *name);
 GnmNamedExpr *expr_name_add    (GnmParsePos const *pp, char const *name,
-				GnmExpr const *expr, char **error_msg,
+				GnmExprTop const *texpr, char **error_msg,
 				gboolean link_to_container,
 				GnmNamedExpr *stub);
 GnmNamedExpr *expr_name_new    (char const *name, gboolean is_placeholder);
@@ -32,14 +32,14 @@ GnmValue*expr_name_eval       (GnmNamedExpr const *ne, GnmEvalPos const *ep,
 char    *expr_name_as_string  (GnmNamedExpr const *ne, GnmParsePos const *pp,
 			       GnmExprConventions const *fmt);
 char    *expr_name_set_scope  (GnmNamedExpr *ne, Sheet *sheet);
-void	 expr_name_set_expr   (GnmNamedExpr *ne, GnmExpr const *new_expr);
+void	 expr_name_set_expr   (GnmNamedExpr *ne, GnmExprTop const *texpr);
 void	 expr_name_add_dep    (GnmNamedExpr *ne, GnmDependent *dep);
 void	 expr_name_remove_dep (GnmNamedExpr *ne, GnmDependent *dep);
 gboolean expr_name_is_placeholder (GnmNamedExpr const *ne);
 void	 expr_name_downgrade_to_placeholder (GnmNamedExpr *nexpr);
 
 int      expr_name_cmp_by_name    (GnmNamedExpr const *a, GnmNamedExpr const *b);
-gboolean expr_name_check_for_loop (char const *name, GnmExpr const *expr);
+gboolean expr_name_check_for_loop (char const *name, GnmExprTop const *texpr);
 
 GList	   *sheet_names_get_available (Sheet const *sheet);
 char const *sheet_names_check	      (Sheet const *sheet, GnmRange const *r);
