@@ -27,8 +27,8 @@
 #include "sheet.h"
 #include "value.h"
 #include "number-match.h"
-#include "gnm-format.h"
 #include "func-builtin.h"
+#include <goffice/utils/go-format.h>
 
 #include <string.h>
 #include <glib.h>
@@ -88,7 +88,7 @@ func_def_cmp (gconstpointer a, gconstpointer b)
 }
 
 static char const *
-check_name_match (char const *name, char const *description, char const*tag)
+check_name_match (char const *name, char const *description, char const *tag)
 {
 	unsigned tag_len = strlen (tag);
 	char const *tmp;
@@ -175,7 +175,7 @@ cb_dump_usage (gpointer key, Symbol *sym, FILE *out)
 }
 
 static char *
-split_at_colon (const char *s, char **rest)
+split_at_colon (char const *s, char **rest)
 {
 	char *dup = g_strdup (s);
 	char *colon = strchr (dup, ':');
@@ -533,7 +533,9 @@ extract_arg_types (GnmFunc *def)
 }
 
 static GnmValue *
-error_function_no_full_info (FunctionEvalInfo *ei, GnmExprList const *expr_node_list)
+error_function_no_full_info (FunctionEvalInfo *ei,
+			     int argc,
+			     GnmExprConstPtr const *argv)
 {
 	return value_new_error (ei->pos, _("Function implementation not available."));
 }
@@ -705,7 +707,9 @@ gnm_func_add (GnmFuncGroup *fn_group,
 
 /* Handle unknown functions on import without losing their names */
 static GnmValue *
-unknownFunctionHandler (FunctionEvalInfo *ei, GnmExprList const *expr_node_list)
+unknownFunctionHandler (FunctionEvalInfo *ei,
+			int argc,
+			GnmExprConstPtr const *argv)
 {
 	return value_new_error_NAME (ei->pos);
 }
@@ -791,7 +795,7 @@ gnm_func_add_placeholder (Workbook *scope,
 
 /* See type GnmParseFunctionHandler */
 GnmExpr const *
-gnm_func_placeholder_factory (const char *name,
+gnm_func_placeholder_factory (char const *name,
 			      GnmExprList *args,
 			      G_GNUC_UNUSED GnmExprConventions const *convs)
 {
@@ -938,13 +942,12 @@ function_def_get_arg_type_string (GnmFunc const *fn_def,
 		return _("Cell Range");
 	case 'A':
 		return _("Area");
-	case 'B':
-		return _("Scalar or Blank");
 	case 'E':
-		return _("Scalar or Error");
+		return _("Scalar, Blank, or Error");
 	case 'S':
 		return _("Scalar");
 	case '?':
+		/* Missing values will be NULL.  */
 		return _("Any");
 
 	default:
@@ -961,8 +964,7 @@ function_def_get_arg_type_string (GnmFunc const *fn_def,
  * Return value: the name of the argument (must be freed)
  **/
 char*
-function_def_get_arg_name (GnmFunc const *fn_def,
-                           int arg_idx)
+function_def_get_arg_name (GnmFunc const *fn_def, int arg_idx)
 {
 	char **names, **o_names;
 	char *name;
@@ -1021,14 +1023,14 @@ free_values (GnmValue **values, int top)
  * Returns the result.
  **/
 GnmValue *
-function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
-			 GnmExprEvalFlags flags)
+function_call_with_exprs (FunctionEvalInfo *ei,
+			  int argc, GnmExprConstPtr const *argv,
+			  GnmExprEvalFlags flags)
 {
 	GnmFunc const *fn_def;
-	int	  argc, i, iter_count, iter_width = 0, iter_height = 0;
+	int	  i, iter_count, iter_width = 0, iter_height = 0;
 	char	  arg_type;
 	GnmValue	 **args, *tmp = NULL;
-	GnmExpr  *expr;
 	int 	 *iter_item = NULL;
 
 	g_return_val_if_fail (ei != NULL, NULL);
@@ -1040,31 +1042,29 @@ function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
 
 	/* Functions that deal with ExprNodes */
 	if (fn_def->fn_type == GNM_FUNC_TYPE_NODES)
-		return fn_def->fn.nodes (ei, l);
+		return fn_def->fn.nodes (ei, argc, argv);
 
 	/* Functions that take pre-computed Values */
-	argc = gnm_expr_list_length (l);
 	if (argc > fn_def->fn.args.max_args ||
 	    argc < fn_def->fn.args.min_args)
-		return value_new_error (ei->pos,
-					_("Invalid number of arguments"));
+		return value_new_error_NA (ei->pos);
 
 	args = g_alloca (sizeof (GnmValue *) * fn_def->fn.args.max_args);
 	iter_count = (flags & GNM_EXPR_EVAL_PERMIT_NON_SCALAR) ? 0 : -1;
 
-	for (i = 0; l; l = l->next, ++i) {
-		arg_type = fn_def->fn.args.arg_types[i];
+	for (i = 0; i < argc; i++) {
+		char arg_type = fn_def->fn.args.arg_types[i];
 		/* expr is always non-null, missing args are encoded as
 		 * const = empty */
-		expr = l->data; 
+		GnmExpr const *expr = argv[i];
 
 		if (arg_type == 'A' || arg_type == 'r') {
-			if (expr->any.oper == GNM_EXPR_OP_CELLREF) {
+			if (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_CELLREF) {
 				GnmCellRef r;
 				gnm_cellref_make_abs (&r, &expr->cellref.ref, ei->pos);
 				args[i] = value_new_cellrange_unsafe (&r, &r);
 				/* TODO decide on the semantics of these argument types */
-#warning do we need to force an eval here ?
+#warning "Do we need to force an eval here?"
 			} else {
 				tmp = args[i] = gnm_expr_eval (expr, ei->pos,
 					GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
@@ -1106,13 +1106,12 @@ function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
 			if (iter_count > 0) {
 				if (iter_width != value_area_get_width (tmp, ei->pos) ||
 				    iter_height != value_area_get_height (tmp, ei->pos)) {
-					/* no need to free inter_vals, there is nothing there yet */
 					free_values (args, i + 1);
 					return value_new_error_VALUE (ei->pos);
 				}
 			} else {
 				if (iter_count < 0) {
-					g_warning ("Damn I though this was impossible");
+					g_warning ("Damn I thought this was impossible");
 					iter_count = 0;
 				}
 				iter_item = g_alloca (sizeof (int) * argc);
@@ -1128,8 +1127,20 @@ function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
 		/* All of these argument types must be scalars */
 		switch (arg_type) {
 		case 'b':
+			if (VALUE_IS_STRING (tmp)) {
+				gboolean err;
+				gboolean b = value_get_as_bool (tmp, &err);
+				if (err) {
+					free_values (args, i + 1);
+					return value_new_error_VALUE (ei->pos);
+				}
+				value_release (args[i]);
+				tmp = args[i] = value_new_bool (b);
+				break;
+			}
+			/* Fall through.  */
 		case 'f':
-			if (tmp->type == VALUE_STRING) {
+			if (VALUE_IS_STRING (tmp)) {
 				tmp = format_match_number (value_peek_string (tmp), NULL,
 					workbook_date_conv (ei->pos->sheet->workbook));
 				if (tmp == NULL) {
@@ -1138,31 +1149,26 @@ function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
 				}
 				value_release (args [i]);
 				args[i] = tmp;
-			} else if (tmp->type == VALUE_ERROR) {
+			} else if (VALUE_IS_ERROR (tmp)) {
 				free_values (args, i);
 				return tmp;
-			} else if (tmp->type == VALUE_EMPTY) {
+			} else if (VALUE_IS_EMPTY (tmp)) {
 				value_release (args [i]);
 				tmp = args[i] = value_new_int (0);
 			}
 
-			if (tmp->type != VALUE_INTEGER &&
-			    tmp->type != VALUE_FLOAT &&
-			    tmp->type != VALUE_BOOLEAN) {
-				free_values (args, i+1);
+			if (!VALUE_IS_NUMBER (tmp))
 				return value_new_error_VALUE (ei->pos);
-			}
 			break;
 
 		case 's':
 		case 'S':
-			if (tmp->type == VALUE_ERROR) {
+			if (VALUE_IS_ERROR (tmp)) {
 				free_values (args, i);
 				return tmp;
 			}
 			break;
 
-		case 'B': /* nothing necessary */
 		case 'E': /* nothing necessary */
 			break;
 
@@ -1197,7 +1203,7 @@ function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
 					if  (arg_type == 'b' || arg_type == 'f') {
 						if (VALUE_IS_EMPTY (elem))
 							elem = value_zero;
-						else if (elem->type == VALUE_STRING) {
+						else if (VALUE_IS_STRING (elem)) {
 							tmp = format_match_number (value_peek_string (elem), NULL,
 								workbook_date_conv (ei->pos->sheet->workbook));
 							if (tmp != NULL) {
@@ -1205,21 +1211,19 @@ function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
 								continue;
 							} else
 								break;
-						} else if (elem->type == VALUE_ERROR) {
+						} else if (VALUE_IS_ERROR (elem)) {
 							err = elem;
 							break;
-						} else if (elem->type != VALUE_INTEGER &&
-							   elem->type != VALUE_FLOAT &&
-							   elem->type != VALUE_BOOLEAN)
+						} else if (!VALUE_IS_NUMBER (elem))
 							break;
 					} else if (arg_type == 's') {
 						if (VALUE_IS_EMPTY (elem)) {
 							args [iter_item[i]] = iter_args [i] = value_new_string ("");
 							continue;
-						} else if (elem->type == VALUE_ERROR) {
+						} else if (VALUE_IS_ERROR (elem)) {
 							err = elem;
 							break;
-						} else if (tmp->type != VALUE_STRING)
+						} else if (!VALUE_IS_STRING (tmp))
 							break;
 					} else if (elem == NULL) {
 						args [iter_item[i]] = iter_args [i] = value_new_empty ();
@@ -1264,7 +1268,7 @@ function_call_with_values (GnmEvalPos const *ep, char const *fn_name,
 	/* FIXME : support workbook local functions */
 	fn_def = gnm_func_lookup (fn_name, NULL);
 	if (fn_def == NULL)
-		return value_new_error (ep, _("Function does not exist"));
+		return value_new_error_NAME (ep);
 	return function_def_call_with_values (ep, fn_def, argc, values);
 }
 
@@ -1288,25 +1292,17 @@ function_def_call_with_values (GnmEvalPos const *ep, GnmFunc const *fn_def,
 		 * If function deals with ExprNodes, create some
 		 * temporary ExprNodes with constants.
 		 */
-		GnmExprConstant *expr = NULL;
-		GnmExprList *l = NULL;
+		GnmExprConstant *expr = g_new (GnmExprConstant, argc);
+		GnmExprConstPtr *argv = g_new (GnmExprConstPtr, argc);
 		int i;
 
-		if (argc) {
-			expr = g_alloca (argc * sizeof (GnmExprConstant));
-			for (i = 0; i < argc; i++) {
-				expr [i].oper = GNM_EXPR_OP_CONSTANT;
-				expr [i].value = values [i];
-				expr [i].ref_count = 1;
-
-				l = gnm_expr_list_append (l, expr + i);
-			}
+		for (i = 0; i < argc; i++) {
+			gnm_expr_constant_init (expr + i, values[i]);
+			argv[i] = (GnmExprConstPtr)(expr + i);
 		}
-
-		retval = fn_def->fn.nodes (&fs, l);
-
-		if (l != NULL)
-			gnm_expr_list_free (l);
+		retval = fn_def->fn.nodes (&fs, argc, argv);
+		g_free (argv);
+		g_free (expr);
 	} else
 		retval = fn_def->fn.args.func (&fs, values);
 
@@ -1345,7 +1341,7 @@ cb_iterate_cellrange (Sheet *sheet, int col, int row,
 	}
 
 	if (data->ignore_subtotal && cell_has_expr (cell) &&
-	    gnm_expr_containts_subtotal (cell->base.expression))
+	    gnm_expr_top_contains_subtotal (cell->base.texpr))
 		return NULL;
 
 	cell_eval (cell);
@@ -1384,8 +1380,7 @@ function_iterate_do_value (GnmEvalPos const  *ep,
 
 	case VALUE_EMPTY:
 	case VALUE_BOOLEAN:
-	case VALUE_INTEGER:
-	case VALUE_FLOAT:
+	case VALUE_FLOAT:	
 	case VALUE_STRING:
 		res = (*callback)(ep, value, closure);
 		break;
@@ -1450,28 +1445,26 @@ GnmValue *
 function_iterate_argument_values (GnmEvalPos const	*ep,
 				  FunctionIterateCB	 callback,
 				  void			*callback_closure,
-				  GnmExprList const	*expr_node_list,
+				  int                    argc,
+				  GnmExprConstPtr const *argv,
 				  gboolean		 strict,
 				  CellIterFlags		 iter_flags)
 {
 	GnmValue *result = NULL;
-	GnmExpr const *expr;
-	GnmValue *val;
+	int a;
 
-	for (; result == NULL && expr_node_list;
-	     expr_node_list = expr_node_list->next) {
-		expr = expr_node_list->data;
+	for (a = 0; result == NULL && a < argc; a++) {
+		GnmExpr const *expr = argv[a];
+		GnmValue *val;
 
 		if (iter_flags & CELL_ITER_IGNORE_SUBTOTAL &&
-		    gnm_expr_containts_subtotal (expr))
+		    gnm_expr_contains_subtotal (expr))
 			continue;
 
-/* NOT UNTIL 1.7 */
-#if 0
 		/* need to drill down into names to handle things like
 		 * sum(name)  with name := (A:A,B:B) */
-		while (expr->any.oper == GNM_EXPR_OP_NAME) {
-			expr = expr->name.name->expr;
+		while (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_NAME) {
+			expr = expr->name.name->texpr->expr;
 			if (expr == NULL) {
 				if (strict)
 					return value_new_error_REF (ep);
@@ -1480,21 +1473,33 @@ function_iterate_argument_values (GnmEvalPos const	*ep,
 		}
 
 		/* Handle sets as a special case */
-		if (expr->any.oper == GNM_EXPR_OP_SET) {
-			result = function_iterate_argument_values (ep, callback,
-				callback_closure, expr->set.set, strict, iter_flags);
+		if (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_SET) {
+			result = function_iterate_argument_values
+				(ep, callback, callback_closure,
+				 expr->set.argc, expr->set.argv,
+				 strict, iter_flags);
 			continue;
 		}
-#endif
 
-		/* Permit empties and non scalars. We don't know what form the
-		 * function wants its arguments */
-		val = gnm_expr_eval (expr, ep, GNM_EXPR_EVAL_PERMIT_NON_SCALAR|GNM_EXPR_EVAL_PERMIT_EMPTY);
+		/* We need a cleaner model of what to do here.  For now it
+		 * seems as if var-arg functions treat explict ranges as special cases
+		 * 	SUM(Range)
+		 * will pass the range not do implicit intersection in non-array mode
+		 * 	SUM(Range=3)
+		 * will do implicit intersection in non-array mode */
+		if (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_CONSTANT)
+			val = value_dup (expr->constant.value);
+		else if (ep->array != NULL)
+			val = gnm_expr_eval (expr, ep,
+				GNM_EXPR_EVAL_PERMIT_EMPTY | GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
+		else
+			val = gnm_expr_eval (expr, ep,
+				GNM_EXPR_EVAL_PERMIT_EMPTY);
 
 		if (val == NULL)
 			continue;
 
-		if (strict && val->type == VALUE_ERROR) {
+		if (strict && VALUE_IS_ERROR (val)) {
 			/* Be careful not to make VALUE_TERMINATE into a real value */
 			/* FIXME : Make the new position of the error here */
 			return val;
@@ -1529,7 +1534,7 @@ tokenized_help_new (GnmFunc const *func)
 		gboolean seek_at = TRUE;
 		gboolean last_newline = TRUE;
 
-#warning fixme, use the rest of the pieces and get rid of this routine.
+#warning "fixme, use the rest of the pieces and get rid of this routine."
 		tok->help_is_localized = FALSE;
 		tok->help_copy = g_strdup (func->help [0].text);
 		tok->sections = g_ptr_array_new ();
