@@ -2,11 +2,10 @@
 /*
  * gnumeric-gconf.c:
  *
- *
  * Author:
  * 	Andreas J. Guelzow <aguelzow@taliesin.ca>
  *
- * (C) Copyright 2002-2004 Andreas J. Guelzow <aguelzow@taliesin.ca>
+ * (C) Copyright 2002-2005 Andreas J. Guelzow <aguelzow@taliesin.ca>
  *
  * Introduced the concept of "node" and implemented the win32 backend
  * by Ivan, Wong Yat Cheung <email@ivanwong.info>, 2005
@@ -34,6 +33,9 @@
 #include "gutils.h"
 #include "mstyle.h"
 #include <goffice/utils/go-glib-extras.h>
+#include <goffice/utils/go-format.h>
+#include <value.h>
+#include <number-match.h>
 
 static GnmAppPrefs prefs;
 GnmAppPrefs const *gnm_app_prefs = &prefs;
@@ -47,9 +49,7 @@ static GOConfNode *root = NULL;
 #endif
 
 #ifdef WITH_GNOME
-#include <goffice/utils/format.h>
-#include <value.h>
-#include <number-match.h>
+
 #include <gconf/gconf-client.h>
 
 struct _GOConfNode {
@@ -60,14 +60,14 @@ struct _GOConfNode {
 static GConfClient *gconf_client = NULL;
 
 static void
-go_conf_init ()
+go_conf_init (void)
 {
 	if (!gconf_client)
 		gconf_client = gconf_client_get_default ();
 }
 
 static void
-go_conf_shutdown ()
+go_conf_shutdown (void)
 {
 	if (gconf_client) {
 		g_object_unref (G_OBJECT (gconf_client));
@@ -500,9 +500,6 @@ go_conf_add_monitor (GOConfNode *node, gchar const *key,
 #elif defined G_OS_WIN32
 
 #include <windows.h>
-#include <goffice/utils/format.h>
-#include <value.h>
-#include <number-match.h>
 
 #ifndef ERANGE
 /* mingw has not defined ERANGE (yet), MSVC has it though */
@@ -515,12 +512,12 @@ struct _GOConfNode {
 };
 
 static void
-go_conf_init ()
+go_conf_init (void)
 {
 }
 
 static void
-go_conf_shutdown ()
+go_conf_shutdown (void)
 {
 }
 
@@ -741,13 +738,16 @@ void
 go_conf_set_str_list (GOConfNode *node, gchar const *key, GSList *list)
 {
 	GString *str_list;
+	GSList *list_node = list;
 
 	str_list = g_string_new ("");
-	while (list) {
-		g_string_append (str_list, g_strescape (list->data, NULL));
+	while (list_node) {
+		g_string_append (str_list, g_strescape (list_node->data, NULL));
 		g_string_append_c (str_list, '\n');
-		list = list->next;
+		list_node = list_node->next;
 	}
+	if (list)
+		g_string_truncate (str_list, str_list->len - 1);
 	go_conf_win32_set (node, key, REG_SZ, (guchar *) str_list->str,
 			   str_list->len + 1);
 	g_string_free (str_list, TRUE);
@@ -823,7 +823,7 @@ go_conf_get_str_list (GOConfNode *node, gchar const *key)
 		str_list = g_strsplit ((const gchar *) ptr, "\n", 0);
 		for (i = 0; str_list[i]; ++i)
 			list = g_slist_prepend (list, g_strcompress (str_list[i]));
-		g_slist_reverse (list);
+		list = g_slist_reverse (list);
 		g_strfreev (str_list);
 		g_free (ptr);
 	}
@@ -1063,73 +1063,301 @@ go_conf_add_monitor (GOConfNode *node, gchar const *key,
 
 #else
 
+#include <glib/gstdio.h>
+#include <errno.h>
+
+#define BOOL_GROUP    "Booleans"
+#define INT_GROUP     "Ints"
+#define DOUBLE_GROUP  "Doubles"
+#define STRING_GROUP  "Strings"
+#define STRLIST_GROUP "StringLists"
+
+struct _GOConfNode {
+	gchar *path;
+};
+
+static GKeyFile *key_file = NULL;
+
+static gchar *get_rc_filename (void)
+{
+	const gchar *home = g_get_home_dir ();
+	gchar *fname = NULL;
+
+	if (home != NULL)
+		fname = g_build_filename (home, ".gnumericrc", NULL);
+
+	return fname;
+}
+
+static void dump_key_data_to_file (void)
+{
+	FILE *fp = NULL;
+	gchar *rcfile = NULL;
+	gchar *key_data;
+
+	rcfile = get_rc_filename ();
+	if (rcfile == NULL) {
+		g_warning ("Couldn't determine the name of the configuration file");
+		return;
+	}
+
+	fp = g_fopen (rcfile, "w");
+	if (fp == NULL) {
+		g_warning ("Couldn't write configuration info to %s", rcfile);
+		g_free (rcfile);
+		return;
+	}
+
+	key_data = g_key_file_to_data (key_file, NULL, NULL);
+
+	if (key_data != NULL) {
+		fputs (key_data, fp);
+		g_free (key_data);
+	}
+
+	fclose (fp);
+	g_free (rcfile);
+}
+
+static gchar *
+go_conf_get_real_key (GOConfNode const *key, gchar const *subkey)
+{
+	return key ? g_strconcat ((key)->path, "/", subkey, NULL) :
+		     g_strdup (subkey);
+}
+
+static void
+go_conf_init (void)
+{
+	gchar *rcfile = get_rc_filename ();
+
+	if (rcfile != NULL) {
+		key_file = g_key_file_new ();
+		g_key_file_load_from_file (key_file, rcfile, G_KEY_FILE_NONE, NULL);
+		g_free (rcfile);
+	}
+}
+
+static void
+go_conf_shutdown (void)
+{
+	dump_key_data_to_file ();
+	g_key_file_free (key_file);
+	key_file = NULL;
+}
+
+GOConfNode *
+go_conf_get_node (GOConfNode *parent, gchar const *key)
+{
+	GOConfNode *node;
+
+	node = g_new (GOConfNode, 1);
+	node->path = go_conf_get_real_key (parent, key);
+
+	return node;
+}
+
+void
+go_conf_free_node (GOConfNode *node)
+{
+	if (node != NULL) {
+		g_free (node->path);
+		g_free (node);
+	}
+}
+
 void
 go_conf_set_bool (GOConfNode *node, gchar const *key, gboolean val)
 {
+	gchar *real_key = go_conf_get_real_key (node, key);
+	g_key_file_set_boolean (key_file, BOOL_GROUP, real_key, val);
+	g_free (real_key);
 }
 
 void
 go_conf_set_int (GOConfNode *node, gchar const *key, gint val)
 {
+	gchar *real_key = go_conf_get_real_key (node, key);
+	g_key_file_set_integer (key_file, INT_GROUP, real_key, val);
+	g_free (real_key);
 }
 
 void
 go_conf_set_double (GOConfNode *node, gchar const *key, gnm_float val)
 {
+	gchar *real_key = go_conf_get_real_key (node, key);
+	gchar str[G_ASCII_DTOSTR_BUF_SIZE];
+
+	g_ascii_dtostr (str, sizeof (str), val);
+	g_key_file_set_value (key_file, DOUBLE_GROUP, key, str);
+	g_free (real_key);
 }
 
 void
 go_conf_set_string (GOConfNode *node, gchar const *key, char const *str)
 {
+	gchar *real_key = go_conf_get_real_key (node, key);
+	g_key_file_set_string (key_file, STRING_GROUP, real_key, str);
+	g_free (real_key);
 }
 
 void
 go_conf_set_str_list (GOConfNode *node, gchar const *key, GSList *list)
 {
+	gchar *real_key;
+	gchar **strs = NULL;
+	int i, ns;
+
+	if (list == NULL)
+		return;
+
+	real_key = go_conf_get_real_key (node, key);
+	ns = g_slist_length (list);
+	strs = g_new (gchar *, ns);
+
+	for (i = 0; i < ns; i++) {
+		const gchar *lstr = list->data;
+		strs[i] = g_strdup (lstr);
+		list = list->next;
+	}
+
+	g_key_file_set_string_list (key_file, STRLIST_GROUP, real_key, 
+				    (const gchar **const) strs, ns);
+	g_free (real_key);
+
+	for (i = 0; i < ns; i++)
+		g_free (strs[i]);
+	g_free (strs);
 }
 
 gboolean
 go_conf_get_bool (GOConfNode *node, gchar const *key)
 {
-	return FALSE;
+	gboolean val;
+	gchar *real_key;
+
+	real_key = go_conf_get_real_key (node, key);
+	val = g_key_file_get_boolean (key_file, BOOL_GROUP, real_key, NULL);
+	g_free (real_key);
+
+	return val;
 }
 
 gint
 go_conf_get_int	(GOConfNode *node, gchar const *key)
 {
-	return 0;
+	gboolean val;
+	gchar *real_key;
+
+	real_key = go_conf_get_real_key (node, key);
+	val = g_key_file_get_integer (key_file, INT_GROUP, real_key, NULL);
+	g_free (real_key);
+
+	return val;
 }
 
 gdouble
 go_conf_get_double (GOConfNode *node, gchar const *key)
 {
-	return 0.;
+	gchar *ptr;
+	gchar *real_key;
+	gdouble val;
+
+	real_key = go_conf_get_real_key (node, key);
+	ptr = g_key_file_get_value (key_file, DOUBLE_GROUP, real_key, NULL);
+	g_free (real_key);
+	if (ptr) {
+		val = g_ascii_strtod (ptr, NULL);
+		g_free (ptr);
+		if (errno != ERANGE)
+			return val;
+	}
+
+	return 0.0;
 }
 
 gchar *
 go_conf_get_string (GOConfNode *node, gchar const *key)
 {
-	return g_strdup ("");
+	gchar *real_key;
+	gchar *val = NULL;
+
+	real_key = go_conf_get_real_key (node, key);
+	val = g_key_file_get_string (key_file, STRING_GROUP, real_key, NULL);
+	g_free (real_key);
+
+	return val;
 }
 
 GSList *
 go_conf_get_str_list (GOConfNode *node, gchar const *key)
 {
-	return NULL;
+	gchar *real_key;
+	GSList *list = NULL;
+	gchar **str_list;
+	gint i, nstrs;
+
+	real_key = go_conf_get_real_key (node, key);
+	str_list = g_key_file_get_string_list (key_file, STRLIST_GROUP, real_key, &nstrs, NULL);
+	g_free (real_key);
+
+	if (str_list != NULL) {
+		for (i = 0; i < nstrs; i++) {
+			if (str_list[i][0]) {
+				list = g_slist_append (list, g_strcompress (str_list[i]));
+			}
+		}
+		g_strfreev (str_list);
+	}
+
+	return list;
 }
 
 gboolean
-go_conf_load_bool (GOConfNode *node, gchar const *key,
-		   gboolean default_val)
+go_conf_load_bool (GOConfNode *node, gchar const *key, gboolean default_val)
 {
-	return default_val;
+	gchar *real_key;
+	gboolean val;
+	GError *err = NULL;
+
+	real_key = go_conf_get_real_key (node, key);
+	val = g_key_file_get_boolean (key_file, BOOL_GROUP, real_key, &err);
+	if (err) {
+		val = default_val;
+		g_error_free (err);
+#if 0
+		d (g_warning ("%s: using default value '%s'", real_key, default_val ? "true" : "false"));
+#endif
+	}
+
+	g_free (real_key);
+	return val;
 }
+
 int
 go_conf_load_int (GOConfNode *node, gchar const *key,
 		  gint minima, gint maxima,
 		  gint default_val)
 {
-	return default_val;
+	gchar *real_key;
+	int val;
+	GError *err = NULL;
+
+	real_key = go_conf_get_real_key (node, key);
+	val = g_key_file_get_integer (key_file, INT_GROUP, real_key, &err);
+
+	if (err) {
+		val = default_val;
+		g_error_free(err);
+#if 0
+		d (g_warning ("%s: using default value %d", real_key, default_val));
+#endif
+	} else if (val < minima || val > maxima) {
+		val = default_val;
+	}
+
+	g_free (real_key);
+	return val;
 }
 
 double
@@ -1137,18 +1365,56 @@ go_conf_load_double (GOConfNode *node, gchar const *key,
 		     gdouble minima, gdouble maxima,
 		     gdouble default_val)
 {
-	return default_val;
+	gchar *real_key;
+	gchar *ptr;
+	double val;
+	GError *err = NULL;
+
+	real_key = go_conf_get_real_key (node, key);
+	ptr = g_key_file_get_value (key_file, DOUBLE_GROUP, real_key, &err);
+
+	if (err) {
+		val = default_val;
+		g_error_free (err);
+	} else {		
+		val = g_ascii_strtod (ptr, NULL);
+		if (val < minima || val > maxima) {
+			val = default_val;
+		}
+	}
+
+	g_free(ptr);
+	g_free (real_key);
+	return val;
 }
+
 char *
 go_conf_load_string (GOConfNode *node, gchar const *key)
 {
-	return NULL;
+	gchar *real_key;
+	char *val = NULL;
+	GError *err = NULL;
+
+	real_key = go_conf_get_real_key (node, key);
+	val = g_key_file_get_string (key_file, STRING_GROUP, real_key, &err);
+
+	if (err) {
+#if 0
+		g_warning (err->message);
+#endif
+		g_error_free (err);
+	}	
+	
+	g_free (real_key);
+	return val;
 }
+
 GSList *
 go_conf_load_str_list (GOConfNode *node, gchar const *key)
 {
-	return NULL;
+	return go_conf_get_str_list (node, key);
 }
+
 char *
 go_conf_get_short_desc (GOConfNode *node, gchar const *key)
 {
@@ -1164,18 +1430,55 @@ go_conf_get_long_desc  (GOConfNode *node, gchar const *key)
 GType
 go_conf_get_type (GOConfNode *node, gchar const *key)
 {
-	return G_TYPE_NONE;
+	gchar **groups;
+	gchar *real_key;
+	GType type = G_TYPE_NONE;
+	int i, ng;
+
+	real_key = go_conf_get_real_key (node, key);
+	groups = g_key_file_get_groups (key_file, &ng);
+
+	if (groups != NULL) {
+		for (i = 0; i < ng; i++) {
+			if (g_key_file_has_key (key_file, groups[i], real_key, NULL)) {
+				if (!g_ascii_strcasecmp (groups[i], BOOL_GROUP)) {
+					type = G_TYPE_BOOLEAN;
+				} else if (!g_ascii_strcasecmp (groups[i], INT_GROUP)) {
+					type = G_TYPE_INT;
+				} else if (!g_ascii_strcasecmp (groups[i], DOUBLE_GROUP)) {
+					type = G_TYPE_DOUBLE;
+				} else if (!g_ascii_strcasecmp (groups[i], STRING_GROUP)) {
+					type = G_TYPE_STRING;
+				} else if (!g_ascii_strcasecmp (groups[i], STRLIST_GROUP)) {
+					type = G_TYPE_STRING;
+				}
+				break;
+			}
+		}
+		g_strfreev (groups);
+	}
+		    
+	g_free (real_key);
+
+	return type;
 }
 
 gchar *
 go_conf_get_value_as_str (GOConfNode *node, gchar const *key)
 {
-	return g_strdup ("");
+	gchar *val = NULL;
+	gchar *real_key = go_conf_get_real_key (node, key);
+	val = g_key_file_get_string (key_file, STRING_GROUP, real_key, NULL);
+	g_free (real_key);	
+	return val;
 }
 
 gboolean
 go_conf_set_value_from_str (GOConfNode *node, gchar const *key, gchar const *val_str)
 {
+	gchar *real_key = go_conf_get_real_key (node, key);
+	g_key_file_set_value (key_file, STRING_GROUP, real_key, val_str);
+	g_free (real_key);
 	return TRUE;
 }
 
@@ -1204,20 +1507,20 @@ gnm_conf_init_printer_decoration_font (void)
 	GOConfNode *node;
 	gchar *name;
 	if (prefs.printer_decoration_font == NULL)
-		prefs.printer_decoration_font = mstyle_new ();
+		prefs.printer_decoration_font = gnm_style_new ();
 
 	node = go_conf_get_node (root, PRINTSETUP_GCONF_DIR);
 	name = go_conf_load_string (node, PRINTSETUP_GCONF_HF_FONT_NAME);
 	if (name) {
-		mstyle_set_font_name (prefs.printer_decoration_font, name);
+		gnm_style_set_font_name (prefs.printer_decoration_font, name);
 		g_free (name);
 	} else
-		mstyle_set_font_name (prefs.printer_decoration_font, DEFAULT_FONT);
-	mstyle_set_font_size (prefs.printer_decoration_font,
+		gnm_style_set_font_name (prefs.printer_decoration_font, DEFAULT_FONT);
+	gnm_style_set_font_size (prefs.printer_decoration_font,
 		go_conf_load_double (node, PRINTSETUP_GCONF_HF_FONT_SIZE, 1., 100., DEFAULT_SIZE));
-	mstyle_set_font_bold (prefs.printer_decoration_font,
+	gnm_style_set_font_bold (prefs.printer_decoration_font,
 		go_conf_load_bool (node, PRINTSETUP_GCONF_HF_FONT_BOLD, FALSE));
-	mstyle_set_font_italic (prefs.printer_decoration_font,
+	gnm_style_set_font_italic (prefs.printer_decoration_font,
 		go_conf_load_bool (node, PRINTSETUP_GCONF_HF_FONT_ITALIC, FALSE));
 	go_conf_free_node (node);
 }
@@ -1270,6 +1573,14 @@ gnm_conf_init_essential (void)
 		  node, GNM_CONF_GUI_ED_AUTOCOMPLETE, TRUE);
 	prefs.live_scrolling = go_conf_load_bool (
 		  node, GNM_CONF_GUI_ED_LIVESCROLLING, TRUE);
+	prefs.toolbars = g_hash_table_new_full
+		(g_str_hash, g_str_equal,
+		 (GDestroyNotify)g_free,
+		 NULL);
+	prefs.toolbar_positions = g_hash_table_new_full
+		(g_str_hash, g_str_equal,
+		 (GDestroyNotify)g_free,
+		 NULL);
 	go_conf_free_node (node);
 
 	/* Unfortunately we need the printing stuff in essentials since the */
@@ -1289,8 +1600,8 @@ gnm_conf_init_essential (void)
 		(node, PRINTSETUP_GCONF_PRINT_BLACK_AND_WHITE, FALSE);
 	prefs.print_titles = go_conf_load_bool 
 		(node, PRINTSETUP_GCONF_PRINT_TITLES, FALSE);
-	prefs.print_order_right_then_down = go_conf_load_bool 
-		(node, PRINTSETUP_GCONF_RIGHT_THEN_DOWN, FALSE);
+	prefs.print_order_across_then_down = go_conf_load_bool 
+		(node, PRINTSETUP_GCONF_ACROSS_THEN_DOWN, FALSE);
 	prefs.print_scale_percentage = go_conf_load_bool 
 		(node, PRINTSETUP_GCONF_SCALE_PERCENTAGE, TRUE);
 	prefs.print_scale_percentage_value = go_conf_load_double 
@@ -1301,19 +1612,19 @@ gnm_conf_init_essential (void)
 		(node, PRINTSETUP_GCONF_SCALE_HEIGHT, 0, 100, 1);
 	prefs.print_repeat_top = go_conf_load_string (node, PRINTSETUP_GCONF_REPEAT_TOP);
 	prefs.print_repeat_left = go_conf_load_string (node, PRINTSETUP_GCONF_REPEAT_LEFT);
-	prefs.print_tb_margins.top.points = go_conf_load_double 
+	prefs.print_margin_top.points = go_conf_load_double 
 		(node, PRINTSETUP_GCONF_MARGIN_TOP, 0.0, 10000.0, 120.0);
-	prefs.print_tb_margins.bottom.points = go_conf_load_double 
+	prefs.print_margin_bottom.points = go_conf_load_double 
 		(node, PRINTSETUP_GCONF_MARGIN_BOTTOM, 0.0, 10000.0, 120.0);
 	{
 		/* Note: the desired display unit is stored in the  */
 		/* printer config. So we are never using this field */
 		/* inside the margin structure, but only setting it */
 		/* in various input routines.                       */
-		prefs.print_tb_margins.top.desired_display 
+		prefs.print_margin_top.desired_display 
 			= gnome_print_unit_get_by_abbreviation ("cm");
-		prefs.print_tb_margins.bottom.desired_display 
-			= prefs.print_tb_margins.top.desired_display;
+		prefs.print_margin_bottom.desired_display 
+			= prefs.print_margin_top.desired_display;
 	}
 	prefs.print_all_sheets = go_conf_load_bool (
 		node, PRINTSETUP_GCONF_ALL_SHEETS, TRUE);
@@ -1386,6 +1697,8 @@ gnm_conf_init_extras (void)
 	node = go_conf_get_node (root, GNM_CONF_SORT_DIR);
 	prefs.sort_default_by_case = go_conf_load_bool (
 		node, GNM_CONF_SORT_DEFAULT_BY_CASE, FALSE);
+	prefs.sort_default_has_header = go_conf_load_bool (
+		node, GNM_CONF_SORT_DEFAULT_HAS_HEADER, FALSE);
 	prefs.sort_default_retain_formats = go_conf_load_bool (
 		node, GNM_CONF_SORT_DEFAULT_RETAIN_FORM, TRUE);
 	prefs.sort_default_ascending = go_conf_load_bool (
@@ -1427,9 +1740,11 @@ void
 gnm_conf_shutdown (void)
 {
 	if (prefs.printer_decoration_font) {
-		mstyle_unref (prefs.printer_decoration_font);
+		gnm_style_unref (prefs.printer_decoration_font);
 		prefs.printer_decoration_font = NULL;
 	}
+	g_hash_table_destroy (prefs.toolbars);
+	g_hash_table_destroy (prefs.toolbar_positions);
 	go_conf_free_node (root);
 	go_conf_shutdown ();
 }
@@ -1635,9 +1950,9 @@ gnm_gconf_set_print_titles (gboolean val)
 }
 
 void     
-gnm_gconf_set_print_order_right_then_down (gboolean val)
+gnm_gconf_set_print_order_across_then_down (gboolean val)
 {
-	go_conf_set_bool (root, PRINTSETUP_GCONF_DIR "/" PRINTSETUP_GCONF_RIGHT_THEN_DOWN, val);
+	go_conf_set_bool (root, PRINTSETUP_GCONF_DIR "/" PRINTSETUP_GCONF_ACROSS_THEN_DOWN, val);
 }
 
 void     
@@ -1745,24 +2060,24 @@ gnm_gconf_set_hf_font (GnmStyle const *mstyle)
 	GOConfNode *node;
 	GnmStyle *old_style = (prefs.printer_decoration_font != NULL) ?
 		prefs.printer_decoration_font :
-		mstyle_new_default ();
+		gnm_style_new_default ();
 	
-	prefs.printer_decoration_font = mstyle_copy_merge (old_style, mstyle);
-	mstyle_unref (old_style);
+	prefs.printer_decoration_font = gnm_style_new_merged (old_style, mstyle);
+	gnm_style_unref (old_style);
 	
 	node = go_conf_get_node (root, PRINTSETUP_GCONF_DIR);
-	if (mstyle_is_element_set (mstyle, MSTYLE_FONT_SIZE))
+	if (gnm_style_is_element_set (mstyle, MSTYLE_FONT_SIZE))
 		go_conf_set_double (node, PRINTSETUP_GCONF_HF_FONT_SIZE,
-			mstyle_get_font_size (mstyle));
-	if (mstyle_is_element_set (mstyle, MSTYLE_FONT_NAME))
+			gnm_style_get_font_size (mstyle));
+	if (gnm_style_is_element_set (mstyle, MSTYLE_FONT_NAME))
 		go_conf_set_string (node, PRINTSETUP_GCONF_HF_FONT_NAME,
-			mstyle_get_font_name (mstyle));
-	if (mstyle_is_element_set (mstyle, MSTYLE_FONT_BOLD))
+			gnm_style_get_font_name (mstyle));
+	if (gnm_style_is_element_set (mstyle, MSTYLE_FONT_BOLD))
 		go_conf_set_bool (node, PRINTSETUP_GCONF_HF_FONT_BOLD,
-			mstyle_get_font_bold (mstyle));
-	if (mstyle_is_element_set (mstyle, MSTYLE_FONT_ITALIC))
+			gnm_style_get_font_bold (mstyle));
+	if (gnm_style_is_element_set (mstyle, MSTYLE_FONT_ITALIC))
 		go_conf_set_bool (node, PRINTSETUP_GCONF_HF_FONT_ITALIC,
-			mstyle_get_font_italic (mstyle));
+			gnm_style_get_font_italic (mstyle));
 	go_conf_free_node (node);
 }
 
@@ -1837,6 +2152,14 @@ gnm_gconf_set_sort_by_case (gboolean val)
 		root, GNM_CONF_SORT_DIR "/" GNM_CONF_SORT_DEFAULT_BY_CASE, val != FALSE);
 }
 
+void     
+gnm_gconf_set_sort_has_header (gboolean val)
+{
+	prefs.sort_default_has_header = val;
+	go_conf_set_bool (
+		root, GNM_CONF_SORT_DIR "/" GNM_CONF_SORT_DEFAULT_HAS_HEADER, val != FALSE);
+}
+
 void     
 gnm_gconf_set_sort_ascending (gboolean val)
 {
@@ -1901,6 +2224,92 @@ gnm_gconf_set_gui_resolution_v (gnm_float val)
 		root, GNM_CONF_GUI_DIR "/" GNM_CONF_GUI_RES_V, val);
 }
 
+gboolean
+gnm_gconf_get_toolbar_visible (const char *name)
+{
+	gpointer pval;
+	char *key = g_strconcat (GNM_CONF_GUI_DIR "/" GNM_CONF_GUI_TOOLBARS "/",
+				 name,
+				 NULL);
+	gboolean found, vis;
+
+	found = g_hash_table_lookup_extended (prefs.toolbars,
+					      key,
+					      NULL, &pval);
+	if (found) {
+		vis = GPOINTER_TO_INT (pval);
+	} else {
+		vis = go_conf_load_bool (root, key, TRUE);
+		g_hash_table_insert (prefs.toolbars,
+				     g_strdup (name),
+				     GINT_TO_POINTER (vis));
+	}
+
+	g_free (key);
+	return vis;
+}
+
+void
+gnm_gconf_set_toolbar_visible (const char *name, gboolean vis)
+{
+	char *key = g_strconcat (GNM_CONF_GUI_DIR "/" GNM_CONF_GUI_TOOLBARS "/",
+				 name,
+				 NULL);
+	vis = !!vis;
+	g_hash_table_replace (prefs.toolbars,
+			      g_strdup (name),
+			      GINT_TO_POINTER (vis));
+	go_conf_set_bool (root, key, vis);
+	g_free (key);
+}
+
+/*
+ * Actually returns a GtkPositionType.
+ */
+int
+gnm_gconf_get_toolbar_position (const char *name)
+{
+	gpointer pval;
+	char *key = g_strconcat (GNM_CONF_GUI_DIR "/" GNM_CONF_GUI_TOOLBARS "/",
+				 name, "-position",
+				 NULL);
+	gboolean found;
+	int pos;
+	static const int TOP = 2;
+
+	found = g_hash_table_lookup_extended (prefs.toolbar_positions,
+					      key,
+					      NULL, &pval);
+	if (found) {
+		pos = GPOINTER_TO_INT (pval);
+	} else {
+		pos = go_conf_load_int (root, key, 0, 3, TOP);
+		g_hash_table_insert (prefs.toolbar_positions,
+				     g_strdup (name),
+				     GINT_TO_POINTER (pos));
+	}
+
+	g_free (key);
+	return pos;
+}
+
+void
+gnm_gconf_set_toolbar_position (const char *name, int pos)
+{
+	char *key;
+
+	g_return_if_fail (pos >= 0 && pos <= 3);
+
+	key = g_strconcat (GNM_CONF_GUI_DIR "/" GNM_CONF_GUI_TOOLBARS "/",
+				 name, "-position",
+				 NULL);
+	g_hash_table_replace (prefs.toolbar_positions,
+			      g_strdup (name),
+			      GINT_TO_POINTER (pos));
+	go_conf_set_int (root, key, pos);
+	g_free (key);
+}
+
 void     
 gnm_gconf_set_unfocused_rs (gboolean val)
 {
@@ -1924,4 +2333,3 @@ gnm_gconf_set_prefer_clipboard  (gboolean val)
 	go_conf_set_bool (
 		root, GNM_CONF_CUTANDPASTE_DIR "/" GNM_CONF_CUTANDPASTE_PREFER_CLIPBOARD, val != FALSE);
 }
-
