@@ -135,10 +135,11 @@ get_top_left (ItemEdit const *ie, int *top, int *left)
 	*left = ((int)ie->item.x1) + ci->margin_a;
 	*top  = (int)ie->item.y1;
 
-	if (align == VALIGN_CENTER || align == VALIGN_BOTTOM) {
+	if (align == VALIGN_CENTER || align == VALIGN_DISTRIBUTED ||
+	    align == VALIGN_BOTTOM) {
 		int text_height, height = (int)(ie->item.y2 - ie->item.y1);
 		pango_layout_get_pixel_size (ie->layout, NULL, &text_height);
-		*top += (align == VALIGN_CENTER)
+		*top += (align != VALIGN_BOTTOM)
 			? (height - text_height + 1)/2
 			: (height - text_height + 1);
 	}
@@ -491,14 +492,16 @@ entry_changed (FooCanvasItem *item)
 }
 
 static void
-item_edit_finalize (GObject *gobject)
+item_edit_dispose (GObject *gobject)
 {
 	ItemEdit *ie = ITEM_EDIT (gobject);
 
 	item_edit_cursor_blink_stop (ie);
 	ie_destroy_feedback_range (ie);
-
+#if 0
+	/* Why?  */
 	scg_set_display_cursor (ie->scg);
+#endif
 
 	if (ie->gfont != NULL) {
 		style_font_unref (ie->gfont);
@@ -509,7 +512,7 @@ item_edit_finalize (GObject *gobject)
 		ie->style= NULL;
 	}
 
-	(G_OBJECT_CLASS (parent_class)->finalize) (gobject);
+	G_OBJECT_CLASS (parent_class)->dispose (gobject);
 }
 
 static int
@@ -571,6 +574,7 @@ item_edit_set_property (GObject *gobject, guint param_id,
 			sheet_style_get (sheet, ie->pos.col, ie->pos.row));
 		ie->gfont = gnm_style_get_font (ie->style,
 			sheet->context, sheet->last_zoom_factor_used);
+		style_font_ref (ie->gfont);
 
 		if (gnm_style_get_align_h (ie->style) == HALIGN_GENERAL)
 			gnm_style_set_align_h (ie->style, HALIGN_LEFT);
@@ -608,7 +612,7 @@ item_edit_class_init (GObjectClass *gobject_class)
 	parent_class = g_type_class_peek_parent (gobject_class);
 
 	gobject_class->set_property = item_edit_set_property;
-	gobject_class->finalize	   = item_edit_finalize;
+	gobject_class->dispose	   = item_edit_dispose;
 
 	g_object_class_install_property (gobject_class, ARG_SHEET_CONTROL_GUI,
 		g_param_spec_object ("SheetControlGUI", "SheetControlGUI",
