@@ -1,10 +1,9 @@
-/* vim: set sw=8: */
-
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * parse-util.c: Various utility routines to parse or produce
  *     string representations of common reference types.
  *
- * Copyright (C) 2000 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2000-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -39,13 +38,15 @@
 /* For def_expr_name_handler: */
 #include "expr-impl.h"
 #include "gutils.h"
+#include <goffice/app/go-doc.h>
+#include <goffice/utils/go-glib-extras.h>
+#include <goffice/utils/go-file.h>
 
 #include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <glib.h>
 #include <string.h>
-#include <goffice/utils/go-glib-extras.h>
 
 static void
 col_name_internal (GString *target, int col)
@@ -235,6 +236,21 @@ r1c1_add_index (GString *target, char type, int num, unsigned char relative)
 		g_string_append_printf (target, "%c%d", type, num + 1);
 }
 
+static char *
+wb_rel_uri (Workbook *wb, Workbook *ref_wb)
+{
+	const char *uri = go_doc_get_uri ((GODoc *)wb);
+	const char *ref_uri = go_doc_get_uri ((GODoc *)ref_wb);
+	char *rel_uri = go_url_make_relative (uri, ref_uri);
+
+	if (rel_uri == NULL || rel_uri[0] == '/') {
+		g_free (rel_uri);
+		return g_strdup (uri);
+	}
+
+	return rel_uri;
+}
+
 /**
  * cellref_as_string :
  * @ref :
@@ -261,10 +277,12 @@ cellref_as_string (GString *target, GnmExprConventions const *conv,
 		else if (pp->wb == NULL || sheet->workbook == pp->wb)
 			g_string_append (target, sheet->name_quoted);
 		else {
+			char *rel_uri = wb_rel_uri (sheet->workbook, pp->wb);
 			g_string_append_c (target, '[');
-			g_string_append (target, workbook_get_uri (sheet->workbook));
+			g_string_append (target, rel_uri);
 			g_string_append_c (target, ']');
 			g_string_append (target, sheet->name_quoted);
+			g_free (rel_uri);
 		}
 		g_string_append (target, conv->output_sheet_name_sep);
 	}
@@ -320,9 +338,11 @@ rangeref_as_string (GString *target, GnmExprConventions const *conv,
 
 	if (ref->a.sheet) {
 		if (pp->wb != NULL && ref->a.sheet->workbook != pp->wb) {
+			char *rel_uri = wb_rel_uri (ref->a.sheet->workbook, pp->wb);
 			g_string_append_c (target, '[');
-			g_string_append (target, workbook_get_uri (ref->a.sheet->workbook));
+			g_string_append (target, rel_uri);
 			g_string_append_c (target, ']');
+			g_free (rel_uri);
 		}
 		if (pp->wb == NULL && pp->sheet == NULL)
 			/* For the expression leak printer. */
@@ -407,6 +427,67 @@ rangeref_as_string (GString *target, GnmExprConventions const *conv,
 	}
 }
 
+/**
+ * gnm_1_0_rangeref_as_string :
+ * @ref :
+ * @pp :
+ *
+ * Simplified variant of rangeref_as_string that old version of gnumeric can
+ * read.  It drops support for full col/row references.  We can remap them on import.
+ **/
+void
+gnm_1_0_rangeref_as_string (GString *target, GnmExprConventions const *conv,
+			    GnmRangeRef const *ref, GnmParsePos const *pp)
+{
+	GnmRange r;
+
+	r.start.col = cellref_abs_col (&ref->a, pp);
+	r.end.col   = cellref_abs_col (&ref->b, pp);
+	r.start.row = cellref_abs_row (&ref->a, pp);
+	r.end.row   = cellref_abs_row (&ref->b, pp);
+
+	if (ref->a.sheet) {
+		if (pp->wb != NULL && ref->a.sheet->workbook != pp->wb) {
+			char *rel_uri = wb_rel_uri (ref->a.sheet->workbook, pp->wb);
+			g_string_append_c (target, '[');
+			g_string_append (target, rel_uri);
+			g_string_append_c (target, ']');
+			g_free (rel_uri);
+		}
+		if (pp->wb == NULL && pp->sheet == NULL)
+			/* For the expression leak printer. */
+			g_string_append (target, "'?'");
+		else if (ref->b.sheet == NULL || ref->a.sheet == ref->b.sheet)
+			g_string_append (target, ref->a.sheet->name_quoted);
+		else {
+			g_string_append (target, ref->a.sheet->name_quoted);
+			g_string_append_c (target, ':');
+			g_string_append (target, ref->b.sheet->name_quoted);
+		}
+		g_string_append (target, conv->output_sheet_name_sep);
+	}
+
+	if (!ref->a.col_relative)
+		g_string_append_c (target, '$');
+	col_name_internal (target, r.start.col);
+	if (!ref->a.row_relative)
+		g_string_append_c (target, '$');
+	row_name_internal (target, r.start.row);
+
+	if (r.start.col != r.end.col ||
+	    ref->a.col_relative != ref->b.col_relative ||
+	    r.start.row != r.end.row ||
+	    ref->a.row_relative != ref->b.row_relative) {
+		g_string_append_c (target, ':');
+		if (!ref->b.col_relative)
+			g_string_append_c (target, '$');
+		col_name_internal (target, r.end.col);
+		if (!ref->b.row_relative)
+			g_string_append_c (target, '$');
+		row_name_internal (target, r.end.row);
+	}
+}
+
 static char const *
 cellref_a1_get (GnmCellRef *out, char const *in, GnmCellPos const *pos)
 {
@@ -465,7 +546,7 @@ r1c1_get_index (char const *str, int *num, unsigned char *relative, gboolean is_
 			return NULL;
 		return end + 1;
 	} else {
-		if (*num <= 0 || *num >= colrow_max (is_col))
+		if (*num <= 0 || *num > colrow_max (is_col))
 			return NULL;
 		(*num)--;
 	}
@@ -623,8 +704,8 @@ gnm_expr_char_start_p (char const * c)
  *
  * @pos : If the string looks like an expression parse it at this location.
  * @text: The text to be parsed.
- * @val : Returns a GnmValue * if the text was a value, otherwise NULL.
- * @expr: Returns an GnmExpr * if the text was an expression, otherwise NULL.
+ * @val : Returns a GnmValue* if the text was a value, otherwise NULL.
+ * @texpr: Returns a GnmExprTop* if the text was an expression, otherwise NULL.
  * @cur_fmt : Optional, current number format.
  * @date_conv : Optional, date parse conventions
  *
@@ -633,13 +714,13 @@ gnm_expr_char_start_p (char const * c)
  */
 void
 parse_text_value_or_expr (GnmParsePos const *pos, char const *text,
-			  GnmValue **val, GnmExpr const **expr,
+			  GnmValue **val, GnmExprTop const **texpr,
 			  GOFormat *cur_fmt,
 			  GODateConventions const *date_conv)
 {
 	char const *expr_start;
 
-	*expr = NULL;
+	*texpr = NULL;
 
 	/* Does it match any formats?  */
 	*val = format_match (text, cur_fmt, date_conv);
@@ -649,11 +730,11 @@ parse_text_value_or_expr (GnmParsePos const *pos, char const *text,
 	/* If it does not match known formats, see if it is an expression */
 	expr_start = gnm_expr_char_start_p (text);
 	if (NULL != expr_start && *expr_start) {
-		*expr = gnm_expr_parse_str (expr_start, pos,
-					    GNM_EXPR_PARSE_DEFAULT,
-					    gnm_expr_conventions_default,
-					    NULL);
-		if (*expr != NULL)
+		*texpr = gnm_expr_parse_str (expr_start, pos,
+					     GNM_EXPR_PARSE_DEFAULT,
+					     gnm_expr_conventions_default,
+					     NULL);
+		if (*texpr != NULL)
 			return;
 	}
 
@@ -726,7 +807,7 @@ unquote (char *dst, char const *src, int n)
  *           Otherwise returns @start and does not modify @wb.
  * **/
 static char const *
-wbref_parse (char const *start, Workbook **wb)
+wbref_parse (char const *start, Workbook **wb, Workbook *ref_wb)
 {
 	/* Is this an external reference ? */
 	if (*start == '[') {
@@ -752,7 +833,9 @@ wbref_parse (char const *start, Workbook **wb)
 		} else
 			unquote (name, start+2, end-start-2);
 
-		tmp_wb = gnm_app_workbook_get_by_name (name);
+		tmp_wb = gnm_app_workbook_get_by_name
+			(name,
+			 ref_wb ? go_doc_get_uri ((GODoc *)ref_wb) : NULL);
 		if (tmp_wb == NULL)
 			return NULL;
 		*wb = tmp_wb;
@@ -790,10 +873,47 @@ sheetref_parse (char const *start, Sheet **sheet, Workbook const *wb,
 			return start;
 		}
 	} else {
-		for (end = start;
-		     g_unichar_isalnum (g_utf8_get_char (end));
-		     end = g_utf8_next_char (end))
-			; /* Nothing */
+		gboolean only_digits = TRUE;
+		end = start;
+
+		/*
+		 * Valid: Normal!a1
+		 * Valid: x.y!a1
+		 * Invalid: .y!a1
+		 *
+		 * Some names starting with digits are actually valid, but
+		 * unparse quoted. Things are quite tricky: most sheet names
+		 * starting with a digit are ok, but not those starting with
+		 * "[0-9]*\." or "[0-9]+[eE]".
+		 * 
+		 * Valid: 42!a1
+		 * Valid: 4x!a1
+		 * Invalid: 1.!a1
+		 * Invalid: 1e!a1
+		 */
+
+		while (1) {
+			gunichar uc = g_utf8_get_char (end);
+			if (g_unichar_isalpha (uc) || uc == '_') {
+				if (only_digits && end != start &&
+				    (uc == 'e' || uc == 'E')) {
+					end = start;
+					break;
+				}
+				only_digits = FALSE;
+				end = g_utf8_next_char (end);
+			} else if (g_unichar_isdigit (uc)) {
+				end = g_utf8_next_char (end);
+			} else if (uc == '.') {
+				/* Valid, except after only digits.  */
+				if (only_digits) {
+					end = start;
+					break;
+				}
+				end++;
+			} else 
+				break;
+		}
 
 		if (*end != '!' && (!allow_3d || *end != ':'))
 			return start;
@@ -836,7 +956,7 @@ r1c1_rangeref_parse (GnmRangeRef *res, char const *ptr, GnmParsePos const *pp)
 		if (ptr[0] != ':' || (ptr[1] != 'R' && ptr[1] != 'r') ||
 		    NULL == (tmp = r1c1_get_index (ptr+1, &res->b.row, &res->b.row_relative, FALSE)) ||
 		    (*tmp != 'C' && *tmp != 'c') ||
-		    NULL == (tmp = r1c1_get_index (tmp, &res->b.row, &res->b.row_relative, FALSE)))
+		    NULL == (tmp = r1c1_get_index (tmp, &res->b.col, &res->b.col_relative, FALSE)))
 			return ptr;
 		return tmp;
 	} else if (*ptr == 'C' || *ptr == 'c') { /* full col C[#] */
@@ -870,12 +990,14 @@ rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *pp,
 {
 	char const *ptr = start, *start_sheet, *tmp1, *tmp2;
 	Workbook *wb;
+	Workbook *ref_wb;
 
 	g_return_val_if_fail (start != NULL, start);
 	g_return_val_if_fail (pp != NULL, start);
 
 	wb = pp->wb;
-	start_sheet = wbref_parse (start, &wb);
+	ref_wb = wb ? wb : pp->sheet->workbook;
+	start_sheet = wbref_parse (start, &wb, ref_wb);
 	if (start_sheet == NULL)
 		return start; /* TODO error unknown workbook */
 	ptr = sheetref_parse (start_sheet, &res->a.sheet, wb, TRUE);
@@ -960,7 +1082,8 @@ rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *pp,
 	return tmp2;
 }
 
-
+#if 0
+/* Do we even need this anymore ? */
 char const *
 gnm_1_0_rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *pp,
 			G_GNUC_UNUSED GnmExprConventions const *convs)
@@ -972,7 +1095,7 @@ gnm_1_0_rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *
 	g_return_val_if_fail (pp != NULL, start);
 
 	wb = pp->wb;
-	ptr = wbref_parse (start, &wb);
+	ptr = wbref_parse (start, &wb, NULL);
 	if (ptr == NULL)
 		return start; /* TODO error unknown workbook */
 
@@ -1013,6 +1136,7 @@ gnm_1_0_rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *
 		res->b.row -= pp->eval.row;
 	return tmp2;
 }
+#endif
 
 /* ------------------------------------------------------------------------- */
 
@@ -1032,9 +1156,11 @@ def_expr_name_handler (GString *target,
 
 	if (name->optional_scope != NULL) {
 		if (name->optional_scope->workbook != pp->wb) {
+			char *rel_uri = wb_rel_uri (name->optional_wb_scope, pp->wb);
 			g_string_append_c (target, '[');
-			g_string_append (target, workbook_get_uri (name->optional_wb_scope));
+			g_string_append (target, rel_uri);
 			g_string_append_c (target, ']');
+			g_free (rel_uri);
 		} else {
 			g_string_append (target, name->optional_scope->name_quoted);
 			g_string_append (target, conv->output_sheet_name_sep);
@@ -1051,19 +1177,148 @@ def_expr_name_handler (GString *target,
 
 /* ------------------------------------------------------------------------- */
 
-GnmExprConventions *
-gnm_expr_conventions_new (void)
+static GString *
+std_sheet_name_quote (GnmExprConventions const *convs,
+		      char const *str)
 {
-	GnmExprConventions *res = g_new0 (GnmExprConventions, 1);
+	gunichar uc = g_utf8_get_char (str);
+	GString *res = g_string_sized_new (20);
+	const char *p;
+	int nletters;
+	int ndigits;
+
+	if (g_ascii_isalpha (uc)) {
+		nletters = 1;
+		ndigits = 0;
+		p = str + 1;
+	} else if (g_unichar_isalpha (uc) || uc == '_') {
+		nletters = -1;
+		ndigits = -1;
+		p = g_utf8_next_char (str);
+	} else
+		goto quoted;
+
+	/* FIXME: What about '?' and '\\'.  I cannot enter those.  */
+
+	for (; *p; p = g_utf8_next_char (p)) {
+		uc = g_utf8_get_char (p);
+
+		if (g_ascii_isalpha (uc)) {
+			if (ndigits == 0)
+				nletters++;
+		} else if (g_ascii_isdigit (uc)) {
+			if (ndigits >= 0)
+				ndigits++;
+		} else if (uc == '.' || uc == '_' || g_unichar_isalpha (uc))
+			nletters = ndigits = -1;
+		else
+			goto quoted;
+	}
+
+	if (ndigits > 0) {
+		/*
+		 * Excel also quotes things that look like cell references.
+		 * Precisely, check for a match against
+		 *    ([A-Za-z]+)0*([1-9][0-9]*)
+		 * where $1 is a valid column name and $2 is a valid row
+		 * number.  (The 0* is an Excel bug.)
+		 */
+
+		int col, row;
+		unsigned char col_relative, row_relative;
+		if (!col_parse (str, &col, &col_relative))
+			goto unquoted;
+
+		p = str + nletters;
+		while (*p == '0')
+			p++, ndigits--;
+		if (!row_parse (p, &row, &row_relative))
+			goto unquoted;
+
+		goto quoted;
+	}
+
+ unquoted:
+	g_string_append (res, str);
+	return res;
+
+ quoted:
+	g_string_append_c (res, '\'');
+	/* This is UTF-8 safe.  */
+	for (; *str; str++) {
+		gchar c = *str;
+		if (c == '\'' || c == '\\')
+			g_string_append_c (res, '\\');
+		g_string_append_c (res, c);
+	}
+	g_string_append_c (res, '\'');
+
+	return res;
+}
+
+static const char *
+std_name_parser (const char *str,
+		 G_GNUC_UNUSED GnmExprConventions const *convs)
+{
+	gunichar uc = g_utf8_get_char (str);
+
+	if (!g_unichar_isalpha (uc) && uc != '_' && uc != '\\')
+		return NULL;
+
+	do {
+		str = g_utf8_next_char (str);
+		uc = g_utf8_get_char (str);
+	} while (g_unichar_isalnum (uc) ||
+		 uc == '_' ||
+		 uc == '?' ||
+		 uc == '\\' ||
+		 uc == '.');
+
+	return str;
+}
+
+/**
+ * gnm_expr_conventions_new_full :
+ * @size :
+ *
+ * Construct a GnmExprConventions of @size.
+ *
+ * Returns a GnmExprConventions with default values.  Caller is responsible for
+ * freeing the result.
+ **/
+GnmExprConventions *
+gnm_expr_conventions_new_full (unsigned size)
+{
+	GnmExprConventions *res = g_malloc0 (size);
+
+	g_return_val_if_fail (size >= sizeof (GnmExprConventions), NULL);
 
 	res->expr_name_handler = def_expr_name_handler;
 	res->cell_ref_handler = cellref_as_string;
 	res->range_ref_handler = rangeref_as_string;
+	res->sheet_name_quote = std_sheet_name_quote;
+	res->name_parser = std_name_parser;
+	res->intersection_char = ' ';
 	res->output_sheet_name_sep = "!";
 	res->output_translated = TRUE;
 	return res;
 }
 
+/**
+ * gnm_expr_conventions_new :
+ *
+ * A convenience wrapper around gnm_expr_conventions_new_full
+ * that constructs a GnmExprConventions of std size.
+ *
+ * Returns a GnmExprConventions with default values.  Caller is responsible for
+ * freeing the result.
+ **/
+GnmExprConventions *
+gnm_expr_conventions_new (void)
+{
+	return gnm_expr_conventions_new_full (sizeof (GnmExprConventions));
+}
+
 void
 gnm_expr_conventions_free (GnmExprConventions *c)
 {
@@ -1197,18 +1452,17 @@ parse_util_init (void)
 
 	convs = gnm_expr_conventions_new ();
 	convs->ref_parser = rangeref_parse;
-	convs->range_sep_colon		= TRUE;
-	convs->sheet_sep_exclamation	= TRUE;
-	convs->dots_in_names		= TRUE;
-	gnm_expr_conventions_default = convs;
+	convs->range_sep_colon		 = TRUE;
+	convs->sheet_sep_exclamation	 = TRUE;
+	convs->r1c1_addresses		 = FALSE;
+	gnm_expr_conventions_default	 = convs;
 
 	convs = gnm_expr_conventions_new ();
 	convs->ref_parser = rangeref_parse;
-	convs->range_sep_colon		= TRUE;
-	convs->sheet_sep_exclamation	= TRUE;
-	convs->dots_in_names		= TRUE;
-	convs->r1c1_addresses		= TRUE;
-	gnm_expr_conventions_r1c1 = convs;
+	convs->range_sep_colon		 = TRUE;
+	convs->sheet_sep_exclamation	 = TRUE;
+	convs->r1c1_addresses		 = TRUE;
+	gnm_expr_conventions_r1c1	 = convs;
 }
 
 void
@@ -1220,7 +1474,7 @@ parse_util_shutdown (void)
 	gnm_expr_conventions_r1c1 = NULL;
 }
 
-GnmExpr const *
+GnmExprTop const *
 gnm_expr_parse_str_simple (char const *expr, GnmParsePos const *pp)
 {
 	return gnm_expr_parse_str (expr, pp, GNM_EXPR_PARSE_DEFAULT,
@@ -1228,3 +1482,24 @@ gnm_expr_parse_str_simple (char const *expr, GnmParsePos const *pp)
 }
 
 /* ------------------------------------------------------------------------- */
+/**
+ * gnm_expr_conv_quote:
+ * @convs : #GnmExprConventions
+ * @str   : string to quote
+ *
+ * Quotes @str according to the convention @conv if necessary. 
+ * or returns a literal copy of @str if no quoting was needed.
+ *
+ * Return value: caller is responsible for the resulting GString 
+ **/
+GString *
+gnm_expr_conv_quote (GnmExprConventions const *convs,
+		     char const *str)
+{
+	g_return_val_if_fail (convs != NULL, NULL);
+	g_return_val_if_fail (convs->sheet_name_quote != NULL, NULL);
+	g_return_val_if_fail (str != NULL, NULL);
+	g_return_val_if_fail (str[0] != 0, NULL);
+
+	return convs->sheet_name_quote (convs, str);
+}
