@@ -1,5 +1,5 @@
-#ifndef GNUMERIC_PARSE_UTIL_H
-# define GNUMERIC_PARSE_UTIL_H
+#ifndef GNM_PARSE_UTIL_H
+# define GNM_PARSE_UTIL_H
 
 #include "gnumeric.h"
 
@@ -14,16 +14,17 @@ char const *row_parse (char const *str, int *res, unsigned char *relative);
 char const *cellpos_as_string	(GnmCellPos const *pos);
 char const *cellpos_parse	(char const *cell_str, GnmCellPos *res,
 				 gboolean strict);
-void        cellref_as_string   (GString *target, const GnmExprConventions *conv,
+void        cellref_as_string   (GString *target, GnmExprConventions const *convs,
 				 GnmCellRef const *cell_ref,
 				 GnmParsePos const *pp, gboolean no_sheetname);
 char const *cellref_parse	(GnmCellRef *out, char const *in,
 				 GnmCellPos const *pos);
 
-void        rangeref_as_string (GString *target, GnmExprConventions const *conv,
+void        rangeref_as_string (GString *target, GnmExprConventions const *convs,
 				GnmRangeRef const *ref, GnmParsePos const *pp);
 char const *rangeref_parse	(GnmRangeRef *res, char const *in,
-				 GnmParsePos const *pp);
+				 GnmParsePos const *pp,
+				 GnmExprConventions const *convs);
 				 /* GError **err); */
 
 char const *sheetref_parse	(char const *start, Sheet **sheet,
@@ -32,10 +33,9 @@ char const *sheetref_parse	(char const *start, Sheet **sheet,
 char const *cell_coord_name	(int col, int row);
 char const *cell_name		(GnmCell const *cell);
 
-/* backwards compatibility versions that will move to a plugin */
-char	   *gnm_1_0_rangeref_as_string	(GnmRangeRef const *ref, GnmParsePos const *pp);
-char const *gnm_1_0_rangeref_parse	(GnmRangeRef *res, char const *in,
-					 GnmParsePos const *pp);
+/* backwards compatibility version */
+void gnm_1_0_rangeref_as_string (GString *target, GnmExprConventions const *conv,
+				 GnmRangeRef const *ref, GnmParsePos const *pp);
 
 typedef enum {
 	PERR_NONE,
@@ -77,8 +77,11 @@ typedef enum {
 } GnmExprParseFlags;
 
 typedef char const *(*GnmRangeRefParse) (GnmRangeRef *res, char const *in,
-					 GnmParsePos const *pp);
+					 GnmParsePos const *pp,
+					 GnmExprConventions const *convs);
 					 /* GError **err); */
+typedef char const *(*GnmNameParse) (const char *in,
+				     GnmExprConventions const *convs);
 
 /*
  * On success, this functions should return a non-NULL value and
@@ -86,7 +89,7 @@ typedef char const *(*GnmRangeRefParse) (GnmRangeRef *res, char const *in,
  */
 typedef GnmExpr const *(*GnmParseFunctionHandler) (const char *name,
 						   GnmExprList *args,
-						   GnmExprConventions *convs);
+						   GnmExprConventions const *convs);
 
 typedef void (*GnmParseExprNameHandler) (GString *target,
 					 GnmParsePos const *pp,
@@ -104,6 +107,9 @@ typedef void (*GnmParseRangeRefHandler) (GString *target,
 					 GnmRangeRef const *cell_ref,
 					 GnmParsePos const *pp);
 
+typedef GString * (*GnmSheetNameQuoteHandler) (GnmExprConventions const *convs,
+					       const char *name);
+
 struct _GnmExprConventions {
 #if 0
 	/* Not yet.  */
@@ -113,6 +119,9 @@ struct _GnmExprConventions {
 #endif
 	gboolean r1c1_addresses;
 
+	/* What character denotes range intersection?  */
+	gunichar intersection_char;
+
 	/* What characters are range separators?  */
 	gboolean range_sep_colon;  /* A1:B2 */
 	gboolean range_sep_dotdot; /* A1..B2 */
@@ -121,9 +130,6 @@ struct _GnmExprConventions {
 	gboolean sheet_sep_exclamation;  /* Sheet!... */
 	gboolean sheet_sep_colon; /* Sheet:... */
 
-	/* Is ERROR.TYPE allowed?  */
-	gboolean dots_in_names;
-
 	/* Formerly USE_APPLIX_CONVENTIONS.  */
 	gboolean ignore_whitespace;
 
@@ -148,6 +154,10 @@ struct _GnmExprConventions {
 	/* Called a lot for anything that might be a reference.  */
 	GnmRangeRefParse ref_parser;
 
+	/* Called a lot for anything that might be a function name or defined
+	   name.  */
+	GnmNameParse name_parser;
+
 	/*
 	 * Optional name->GnmParseFunctionHandler hash.  When a name is
 	 * present in this hash, unknown_function_handler will not be
@@ -169,39 +179,51 @@ struct _GnmExprConventions {
 	/* Called to make strings of range refs.  */
 	GnmParseRangeRefHandler range_ref_handler;
 
+	/* Called to optionally quote a sheet name.  */
+	GnmSheetNameQuoteHandler sheet_name_quote;
+
 	/* Used to separate sheet from name when both are needed.  */
 	char const *output_sheet_name_sep;
 
-	/* If non-null, used to separate elements in lists.  */
+	/* If non-null, used to separate elements in lists. */
 	char const *output_argument_sep;
 
-	/* If non-null, used to separate array columns.  */
+	/* If non-null, used to separate array columns. */
 	char const *output_array_col_sep;
 
+	/* If non-null, used to separate array columns.  */
+	char const *output_array_row_sep;
+
 	gboolean output_translated;
 };
 
-GnmExprConventions *gnm_expr_conventions_new (void);
+GnmExprConventions *gnm_expr_conventions_new	  (void);
+GnmExprConventions *gnm_expr_conventions_new_full (unsigned size);
 void gnm_expr_conventions_free (GnmExprConventions *c);
 
-extern GnmExprConventions *gnm_expr_conventions_default;
+extern GnmExprConventions const *gnm_expr_conventions_default;
+extern GnmExprConventions const *gnm_expr_conventions_r1c1;
 void parse_util_init (void);
 void parse_util_shutdown (void);
 
-GnmExpr const *gnm_expr_parse_str (char const *expr, GnmParsePos const *pp,
-				   GnmExprParseFlags flags,
-				   GnmExprConventions *conv,
-				   GnmParseError *error);
+GnmExprTop const *gnm_expr_parse_str (char const *expr, GnmParsePos const *pp,
+				      GnmExprParseFlags flags,
+				      GnmExprConventions const *convs,
+				      GnmParseError *error);
 
-GnmExpr const *gnm_expr_parse_str_simple (char const *expr, GnmParsePos const *pp);
+GnmExprTop const *gnm_expr_parse_str_simple (char const *expr,
+					     GnmParsePos const *pp);
 
 /* Is this string potentially the start of an expression */
 char const *gnm_expr_char_start_p (char const *c);
 
 void	    parse_text_value_or_expr (GnmParsePos const *pos,
 				      char const *text,
-				      GnmValue **val, GnmExpr const **expr,
-				      GnmFormat *current_format,
-				      GnmDateConventions const *date_conv);
+				      GnmValue **val,
+				      GnmExprTop const **texpr,
+				      GOFormat *current_format,
+				      GODateConventions const *date_conv);
 
-#endif /* GNUMERIC_PARSE_UTIL_H */
+GString	*gnm_expr_conv_quote (GnmExprConventions const *conv, char const *str);
+
+#endif /* GNM_PARSE_UTIL_H */
