@@ -275,21 +275,20 @@ print_page_repeated_intersect (PrintJobInfo const *pj, Sheet const *sheet,
 static PangoLayout *
 ensure_decoration_layout (PrintJobInfo *pj)
 {
-
 	if (!pj->decoration_layout) {
 		PangoLayout *layout = gnome_print_pango_create_layout (pj->print_context);
 		/*
-		 * Copy the style so we don't leave a cached GnmFont in the prefs
-		 * object.
+		 * Copy the style so we don't leave a cached GnmFont in the
+		 * prefs object.
 		 */
 		GnmStyle *style = gnm_style_dup (gnm_app_prefs->printer_decoration_font);
-		GnmFont *font = gnm_style_get_font (style,
-						 pango_layout_get_context (layout),
-						 1.);
+		GnmFont *font = gnm_style_get_font
+			(style,
+			 pango_layout_get_context (layout),
+			 1.);
 
 		pj->decoration_layout = layout;
 		pango_layout_set_font_description (layout, font->pango.font_descr);
-		style_font_unref (font);
 		gnm_style_unref (style);
 	}
 	return pj->decoration_layout;
@@ -763,7 +762,7 @@ compute_scale_fit_to (PrintJobInfo const *pj, Sheet const *sheet,
 
 static int
 print_range_down_then_across (PrintJobInfo const *pj, Sheet const *sheet,
-			     GnmRange const *r, gboolean output)
+			      GnmRange const *r, gboolean output)
 {
 	PrintInformation *pi = sheet->print_info;
 	double usable_x, usable_x_initial, usable_x_repeating;
