@@ -177,88 +177,8 @@ range_list_destroy (GSList *ranges)
 }
 
 
-/**
- * range_adjacent:
- * @a: First range
- * @b: Second range
- *
- * Detects whether a range of similar size is adjacent
- * to the other range. Similarity is determined by having
- * a shared side of equal length. NB. this will clearly
- * give odd results for overlapping regions.
- *
- * Return value: if they share a side of equal length
- **/
-gboolean
-range_adjacent (GnmRange const *a, GnmRange const *b)
-{
-	g_return_val_if_fail (a != NULL, FALSE);
-	g_return_val_if_fail (b != NULL, FALSE);
-
-	if ((a->start.col == b->start.col) &&
-	    (a->end.col   == b->end.col))
-		return (a->end.row + 1 == b->start.row ||
-			b->end.row + 1 == a->start.row);
-
-	if ((a->start.row == b->start.row) &&
-	    (a->end.row   == b->end.row))
-		return (a->end.col + 1 == b->start.col ||
-			b->end.col + 1 == a->start.col);
-
-	return FALSE;
-}
-
-/**
- * range_merge:
- * @a: GnmRange a.
- * @b: GnmRange b.
- *
- * This routine coalesces two adjacent regions, eg.
- * (A1, B1) would return A1:B1 or (A1:B2, C1:D2)) would
- * give A1:D2. NB. it is imperative that the regions are
- * actualy adjacent or unexpected results will ensue.
- *
- * Fully commutative.
- *
- * Return value: the merged range.
- **/
-GnmRange
-range_merge (GnmRange const *a, GnmRange const *b)
-{
-	GnmRange ans;
-
-	ans.start.col = 0;
-	ans.start.row = 0;
-	ans.end.col   = 0;
-	ans.end.row   = 0;
-
-	g_return_val_if_fail (a != NULL, ans);
-	g_return_val_if_fail (b != NULL, ans);
-
-/*      Useful perhaps but kills performance */
-/*	g_return_val_if_fail (range_adjacent (a, b), ans); */
-
-	if (a->start.row < b->start.row) {
-		ans.start.row = a->start.row;
-		ans.end.row   = b->end.row;
-	} else {
-		ans.start.row = b->start.row;
-		ans.end.row   = a->end.row;
-	}
-
-	if (a->start.col < b->start.col) {
-		ans.start.col = a->start.col;
-		ans.end.col   = b->end.col;
-	} else {
-		ans.start.col = b->start.col;
-		ans.end.col   = a->end.col;
-	}
-
-	return ans;
-}
-
 char const *
-range_name (GnmRange const *src)
+range_as_string (GnmRange const *src)
 {
 	static char buffer[(2 + 4 * sizeof (long)) * 2 + 1];
 
@@ -330,8 +250,8 @@ range_has_header (Sheet const *sheet, GnmRange const *src,
 
 		/* Look for style differences */
 		if (!ignore_styles &&
-		    !gnm_style_equal_header (cell_get_mstyle (a),
-					  cell_get_mstyle (b), top))
+		    !gnm_style_equal_header (cell_get_style (a),
+					     cell_get_style (b), top))
 			return TRUE;
 	}
 
@@ -784,21 +704,6 @@ range_is_full (GnmRange const *r, gboolean is_cols)
 		return (r->start.row <= 0 && r->end.row >= SHEET_MAX_ROWS - 1);
 }
 
-/**
- * range_is_infinite:
- * @r: the range.
- *
- * This determines whether @r completely spans a sheet
- * in either dimension ( semi-infinite )
- *
- * Return value: TRUE if it is infinite, FALSE otherwise.
- **/
-gboolean
-range_is_infinite (GnmRange const *r)
-{
-	return range_is_full (r, TRUE) || range_is_full (r, FALSE);
-}
-
 /**
  * range_clip_to_finite :
  * @range :
@@ -1027,18 +932,10 @@ gnm_sheet_range_overlap (GnmSheetRange const *a, GnmSheetRange const *b)
 	return FALSE;
 }
 
-GnmSheetRange *
-gnm_sheet_range_dup (GnmSheetRange const *src)
-{
-	g_return_val_if_fail (src != NULL, NULL);
-
-	return gnm_sheet_range_new (src->sheet, &src->range);
-}
-
 char *
 global_range_name (Sheet const *sheet, GnmRange const *r)
 {
-	char const * the_range_name = range_name (r);
+	char const * the_range_name = range_as_string (r);
 
 	if (sheet == NULL)
 		return g_strdup (the_range_name);
@@ -1079,11 +976,11 @@ undo_cell_pos_name (Sheet const *sheet, GnmCellPos const *pos)
  * @r:
  *
  * Returns the range name depending on the preference setting.
- */
+ **/
 char *
 undo_range_name (Sheet const *sheet, GnmRange const *r)
 {
-	char const * the_range_name = range_name (r);
+	char const *the_range_name = range_as_string (r);
 	guint max_width;
 
 	max_width = max_range_name_width ();
@@ -1110,7 +1007,7 @@ range_list_name_try (GString *names, Sheet const *sheet, GSList const *ranges,
 		     guint max_width)
 {
 	GSList const *l;
-	char const *n = range_name (ranges->data);
+	char const *n = range_as_string (ranges->data);
 
 	if (sheet == NULL)
 		g_string_assign (names, n);
@@ -1125,7 +1022,7 @@ range_list_name_try (GString *names, Sheet const *sheet, GSList const *ranges,
 	for (l = ranges->next; l != NULL; l = l->next) {
 		gsize new_len;
 
-		n = range_name (l->data);
+		n = range_as_string (l->data);
 
 		/* The string may not get too long. */
 		new_len = names->len + 2 + strlen (n);
@@ -1216,26 +1113,28 @@ GSList *
 global_range_list_parse (Sheet *sheet, char const *str)
 {
 	GnmParsePos  pp;
-	GnmExpr const *expr;
+	GnmExprTop const *texpr;
 	GSList   *ranges = NULL;
 	GnmValue	 *v;
 
 	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 	g_return_val_if_fail (str != NULL, NULL);
 
-	expr = gnm_expr_parse_str (str,
-		parse_pos_init_sheet (&pp, sheet),
-		GNM_EXPR_PARSE_FORCE_EXPLICIT_SHEET_REFERENCES |
-		GNM_EXPR_PARSE_PERMIT_MULTIPLE_EXPRESSIONS |
-		GNM_EXPR_PARSE_UNKNOWN_NAMES_ARE_STRINGS,
-		gnm_expr_conventions_default,
-		NULL);
+	texpr = gnm_expr_parse_str
+		(str,
+		 parse_pos_init_sheet (&pp, sheet),
+		 GNM_EXPR_PARSE_FORCE_EXPLICIT_SHEET_REFERENCES |
+		 GNM_EXPR_PARSE_PERMIT_MULTIPLE_EXPRESSIONS |
+		 GNM_EXPR_PARSE_UNKNOWN_NAMES_ARE_STRINGS,
+		 gnm_expr_conventions_default,
+		 NULL);
 
-	if (expr != NULL)  {
-		if (expr->any.oper == GNM_EXPR_OP_SET) {
-			GnmExprList *l;
-			for (l = expr->set.set; l != NULL; l = l->next) {
-				v = gnm_expr_get_range (l->data);
+	if (texpr != NULL)  {
+		if (GNM_EXPR_GET_OPER (texpr->expr) == GNM_EXPR_OP_SET) {
+			GnmExpr const *expr = texpr->expr;
+			int i;
+			for (i = 0; i < expr->set.argc; i++) {
+				v = gnm_expr_get_range (expr->set.argv[i]);
 				if (v == NULL) {
 					range_list_destroy (ranges);
 					ranges = NULL;
@@ -1244,11 +1143,11 @@ global_range_list_parse (Sheet *sheet, char const *str)
 					ranges = g_slist_prepend (ranges, v);
 			}
 		} else {
-			v = gnm_expr_get_range (expr);
+			v = gnm_expr_top_get_range (texpr);
 			if (v != NULL)
 				ranges = g_slist_prepend (ranges, v);
 		}
-		gnm_expr_unref (expr);
+		gnm_expr_top_unref (texpr);
 	}
 
 	return g_slist_reverse (ranges);
