@@ -3,7 +3,6 @@
 
 #include "gnumeric.h"
 #include <pango/pango.h>
-#include <gdk/gdkcolor.h>
 
 /**
  * RenderedValue:
@@ -17,24 +16,38 @@ struct _RenderedValue {
 	/* In pixels:  */
 	guint16 indent_left, indent_right;
 
-	/* See http://bugzilla.gnome.org/show_bug.cgi?id=105322 */
-	GdkColor color;
+	GOColor go_fore_color;
 
-	guint effective_halign : 8; /* 7 bits would be enough.  */
-	guint effective_valign : 8; /* 4 bits would be enough.  */
+	guint effective_halign : 8;
+	guint effective_valign : 5;
  	guint variable_width : 1;   /* result depends on the width of cell */
 	guint hfilled : 1;
 	guint vfilled : 1;
 	guint wrap_text : 1;
 	guint might_overflow : 1;   /* Subject to ####### treatment.  */
 	guint numeric_overflow : 1; /* ####### has happened.  */
+	guint noborders : 1;        /* Valid for rotated only.  */
+	guint drawn : 1;            /* Has drawing layout taken place?  */
 	signed int rotation : 10;
 };
 
-RenderedValue *rendered_value_new     (GnmCell const *cell, GnmStyle const *mstyle,
+struct _RenderedRotatedValue {
+	RenderedValue rv;
+	PangoMatrix rotmat;
+	int linecount;
+	struct RenderedRotatedValueInfo {
+		int dx, dy;
+	} *lines;	
+};
+
+RenderedValue *rendered_value_new     (GnmCell *cell, GnmStyle const *mstyle,
 				       gboolean variable_width,
-				       PangoContext *context, float zoom);
+				       PangoContext *context,
+				       double zoom);
 void           rendered_value_destroy (RenderedValue *rv);
+
+RenderedValue *rendered_value_recontext (RenderedValue *rv,
+					 PangoContext *context);
 void           rendered_value_remeasure (RenderedValue *rv);
 
 /* Return the value as a single string without format infomation.  */
