@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
@@ -78,7 +78,7 @@ sheet_merge_add (Sheet *sheet, GnmRange const *r, gboolean clear,
 		if (cc != NULL)
 			go_cmd_context_error (cc, g_error_new (go_error_invalid(), 0,
 				_("There is already a merged region that intersects\n%s!%s"),
-				sheet->name_unquoted, range_name (r)));
+				sheet->name_unquoted, range_as_string (r)));
 		g_slist_free (test);
 		return TRUE;
 	}
@@ -103,10 +103,10 @@ sheet_merge_add (Sheet *sheet, GnmRange const *r, gboolean clear,
 					    cc);
 
 		/* Apply the corner style to the entire region */
-		style = mstyle_copy (sheet_style_get (sheet, r->start.col,
+		style = gnm_style_dup (sheet_style_get (sheet, r->start.col,
 						      r->start.row));
 		for (i = MSTYLE_BORDER_TOP; i <= MSTYLE_BORDER_DIAGONAL; i++)
-			mstyle_unset_element (style, i);
+			gnm_style_unset_element (style, i);
 		sheet_style_apply_range (sheet, r, style);
 		sheet_region_queue_recalc (sheet, r);
 	}
@@ -132,7 +132,7 @@ sheet_merge_add (Sheet *sheet, GnmRange const *r, gboolean clear,
 			sv_set_edit_pos (sv, &r->start);
 	});
 
-	comment = cell_has_comment_pos (sheet, &r->start);
+	comment = sheet_get_comment (sheet, &r->start);
 	if (comment != NULL)
 		sheet_object_update_bounds (SHEET_OBJECT (comment), NULL);
 
@@ -180,7 +180,7 @@ sheet_merge_remove (Sheet *sheet, GnmRange const *r, GOCmdContext *cc)
 	if (cell != NULL)
 		cell->base.flags &= ~CELL_IS_MERGED;
 
-	comment = cell_has_comment_pos (sheet, &r->start);
+	comment = sheet_get_comment (sheet, &r->start);
 	if (comment != NULL)
 		sheet_object_update_bounds (SHEET_OBJECT (comment), NULL);
 
