@@ -31,11 +31,13 @@
 #include "sheet-object-graph.h"
 #include "sheet-object-image.h"
 #include "graph.h"
-#include "io-context.h"
+#include <goffice/app/io-context.h>
 #include "application.h"
+#include "gutils.h"
 
 #include <libxml/globals.h>
-#include <libfoocanvas/foo-canvas.h>
+#include <goffice/cut-n-paste/foocanvas/foo-canvas.h>
+#include <goffice/utils/go-glib-extras.h>
 #include <gsf/gsf-impl-utils.h>
 
 #include <string.h>
@@ -43,10 +45,6 @@
 /* Returns the class for a SheetObject */
 #define SO_CLASS(so) SHEET_OBJECT_CLASS(G_OBJECT_GET_CLASS(so))
 
-#define	SO_VIEW_SHEET_CONTROL_KEY	"SheetControl"
-#define	SO_VIEW_OBJECT_KEY		"SheetObject"
-#define	SO_VIEW_KEY			"Key"
-
 enum {
 	BOUNDS_CHANGED,
 	UNREALIZED,
@@ -54,6 +52,8 @@ enum {
 };
 static guint	     signals [LAST_SIGNAL] = { 0 };
 static GObjectClass *parent_klass;
+static GQuark	sov_so_quark;
+static GQuark	sov_container_quark;
 
 static void
 cb_so_pull_to_front (SheetObject *so, SheetControl *sc)
@@ -78,29 +78,36 @@ cb_so_push_to_back (SheetObject *so, SheetControl *sc)
 static void
 cb_so_delete (SheetObject *so, SheetControl *sc)
 {
-	cmd_object_delete (sc_wbc (sc), so, NULL);
+	cmd_objects_delete (sc_wbc (sc), 
+		go_slist_create (so, NULL), NULL);
 }
-static void
-cb_so_configure (SheetObject *so, SheetControl *sc)
+void
+sheet_object_get_editor (SheetObject *so, SheetControl *sc)
 {
-	SO_CLASS(so)->user_config (so, sc);
+	g_return_if_fail (IS_SHEET_OBJECT (so));
+	g_return_if_fail (SO_CLASS (so));
+
+	if (SO_CLASS(so)->user_config)
+		SO_CLASS(so)->user_config (so, sc);
 }
 static void
 cb_so_cut (SheetObject *so, SheetControl *sc)
 {
-	gnm_app_clipboard_cut_copy_obj (sc_wbc (sc), TRUE, sc_view (sc), so);
+	gnm_app_clipboard_cut_copy_obj (sc_wbc (sc), TRUE, sc_view (sc),
+		go_slist_create (so, NULL));
 }
 static void
 cb_so_copy (SheetObject *so, SheetControl *sc)
 {
-	gnm_app_clipboard_cut_copy_obj (sc_wbc (sc), FALSE, sc_view (sc), so);
+	gnm_app_clipboard_cut_copy_obj (sc_wbc (sc), FALSE, sc_view (sc),
+		go_slist_create (so, NULL));
 }
 
 static void
 sheet_object_populate_menu (SheetObject *so, GPtrArray *actions)
 {
 	static SheetObjectAction const so_actions [] = {
-		{ "gtk-properties",	NULL,		NULL,  0, cb_so_configure },
+		{ "gtk-properties",	NULL,		NULL,  0, sheet_object_get_editor },
 		{ NULL,	NULL, NULL, 0, NULL },
 		{ NULL,			N_("_Order"),	NULL,  1, NULL },
 			{ NULL,			N_("Pul_l to Front"),	NULL,  0, cb_so_pull_to_front },
@@ -129,7 +136,7 @@ static void
 sheet_objects_max_extent (Sheet *sheet)
 {
 	GnmCellPos max_pos = { 0, 0 };
-	GList *ptr;
+	GSList *ptr;
 
 	for (ptr = sheet->sheet_objects; ptr != NULL ; ptr = ptr->next ) {
 		SheetObject *so = SHEET_OBJECT (ptr->data);
@@ -162,10 +169,9 @@ sheet_object_init (GObject *object)
 	int i;
 	SheetObject *so = SHEET_OBJECT (object);
 
-	so->type = SHEET_OBJECT_ACTION_STATIC;
 	so->sheet = NULL;
-	so->is_visible = TRUE;
-	so->move_with_cells = TRUE;
+	so->flags = SHEET_OBJECT_IS_VISIBLE | SHEET_OBJECT_PRINT |
+		SHEET_OBJECT_MOVE_WITH_CELLS | SHEET_OBJECT_SIZE_WITH_CELLS;
 
 	/* Store the logical position as A1 */
 	so->anchor.cell_bound.start.col = so->anchor.cell_bound.start.row = 0;
@@ -204,14 +210,14 @@ sheet_object_class_init (GObjectClass *klass)
 		SHEET_OBJECT_TYPE,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (SheetObjectClass, bounds_changed),
-		(GSignalAccumulator) NULL, NULL,
+		NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
 	signals [UNREALIZED] = g_signal_new ("unrealized",
 		SHEET_OBJECT_TYPE,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (SheetObjectClass, unrealized),
-		(GSignalAccumulator) NULL, NULL,
+		NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
 }
@@ -220,37 +226,17 @@ GSF_CLASS (SheetObject, sheet_object,
 	   sheet_object_class_init, sheet_object_init,
 	   G_TYPE_OBJECT);
 
-SheetObject *
-sheet_object_view_obj (GObject *view)
-{
-	gpointer obj = g_object_get_data (view, SO_VIEW_OBJECT_KEY);
-	return SHEET_OBJECT (obj);
-}
-
-SheetControl *
-sheet_object_view_control (GObject *view)
-{
-	gpointer obj = g_object_get_data (view, SO_VIEW_SHEET_CONTROL_KEY);
-	return SHEET_CONTROL (obj);
-}
-
-gpointer
-sheet_object_view_key (GObject *view)
-{
-	return g_object_get_data (view, SO_VIEW_KEY);
-}
-
-GObject *
-sheet_object_get_view (SheetObject *so, gpointer key)
+SheetObjectView *
+sheet_object_get_view (SheetObject const *so, SheetObjectViewContainer *container)
 {
 	GList *l;
 
 	g_return_val_if_fail (IS_SHEET_OBJECT (so), NULL);
 
 	for (l = so->realized_list; l; l = l->next) {
-		GObject *obj = G_OBJECT (l->data);
-		if (key == g_object_get_data (obj, SO_VIEW_KEY))
-			return obj;
+		SheetObjectView *view = SHEET_OBJECT_VIEW (l->data);
+		if (container == g_object_get_qdata (G_OBJECT (view), sov_container_quark))
+			return view;
 	}
 
 	return NULL;
@@ -293,9 +279,9 @@ sheet_object_update_bounds (SheetObject *so, GnmCellPos const *pos)
 		i = so->anchor.cell_bound.start.row;
 		while (i <= end && is_hidden)
 			is_hidden &= sheet_row_is_hidden (so->sheet, i++);
-	}
-
-	so->is_visible = !is_hidden;
+		so->flags |= SHEET_OBJECT_IS_VISIBLE;
+	} else
+		so->flags &= ~SHEET_OBJECT_IS_VISIBLE;
 
 	g_signal_emit (so, signals [BOUNDS_CHANGED], 0);
 }
@@ -313,6 +299,15 @@ sheet_object_get_sheet (SheetObject const *so)
 	return so->sheet;
 }
 
+static int
+cb_create_views (SheetObject *so)
+{
+	g_object_set_data (G_OBJECT (so), "create_view_handler", NULL);
+	SHEET_FOREACH_CONTROL (so->sheet, view, control,
+		sc_object_create_view (control, so););
+	sheet_object_update_bounds (so, NULL);
+	return FALSE;
+}
 
 /**
  * sheet_object_set_sheet :
@@ -329,7 +324,7 @@ sheet_object_set_sheet (SheetObject *so, Sheet *sheet)
 	g_return_val_if_fail (IS_SHEET_OBJECT (so), TRUE);
 	g_return_val_if_fail (IS_SHEET (sheet), TRUE);
 	g_return_val_if_fail (so->sheet == NULL, TRUE);
-	g_return_val_if_fail (g_list_find (sheet->sheet_objects, so) == NULL, TRUE);
+	g_return_val_if_fail (g_slist_find (sheet->sheet_objects, so) == NULL, TRUE);
 
 	so->sheet = sheet;
 	if (SO_CLASS (so)->assign_to_sheet &&
@@ -339,14 +334,15 @@ sheet_object_set_sheet (SheetObject *so, Sheet *sheet)
 	}
 
 	g_object_ref (G_OBJECT (so));
-	sheet->sheet_objects = g_list_prepend (sheet->sheet_objects, so);
-	SHEET_FOREACH_CONTROL (so->sheet, view, control,
-		sc_object_create_view (control, so););
-	sheet_object_update_bounds (so, NULL);
-
+	sheet->sheet_objects = g_slist_prepend (sheet->sheet_objects, so);
 	/* FIXME : add a flag to sheet to have sheet_update do this */
 	sheet_objects_max_extent (sheet);
 
+	if (NULL == g_object_get_data (G_OBJECT (so), "create_view_handler")) {
+		guint id = g_idle_add ((GSourceFunc) cb_create_views, so);
+		g_object_set_data (G_OBJECT (so), "create_view_handler", GUINT_TO_POINTER (id));
+	}
+
 	return FALSE;
 }
 
@@ -362,27 +358,33 @@ sheet_object_set_sheet (SheetObject *so, Sheet *sheet)
 gboolean
 sheet_object_clear_sheet (SheetObject *so)
 {
-	GList *ptr;
+	GSList *ptr;
+	gpointer view_handler;
 
 	g_return_val_if_fail (IS_SHEET_OBJECT (so), TRUE);
 	g_return_val_if_fail (IS_SHEET (so->sheet), TRUE);
 
-	ptr = g_list_find (so->sheet->sheet_objects, so);
+	ptr = g_slist_find (so->sheet->sheet_objects, so);
 	g_return_val_if_fail (ptr != NULL, TRUE);
 
+	/* clear any pending attempts to create views */
+	view_handler = g_object_get_data (G_OBJECT (so), "create_view_handler");
+	if (NULL != view_handler) {
+		g_source_remove (GPOINTER_TO_UINT (view_handler));
+		g_object_set_data (G_OBJECT (so), "create_view_handler", NULL);
+	}
+
 	/* The views remove themselves from the list */
 	while (so->realized_list != NULL)
-		sc_object_destroy_view (
-			sheet_object_view_control (G_OBJECT (so->realized_list->data)),
-			so);
+		sheet_object_view_destroy (so->realized_list->data);
 	g_signal_emit (so, signals [UNREALIZED], 0);
 
 	if (SO_CLASS (so)->remove_from_sheet &&
 	    SO_CLASS (so)->remove_from_sheet (so))
 		return TRUE;
 
-	so->sheet->sheet_objects = g_list_remove_link (so->sheet->sheet_objects, ptr);
-	g_list_free (ptr);
+	so->sheet->sheet_objects = g_slist_remove_link (so->sheet->sheet_objects, ptr);
+	g_slist_free (ptr);
 
 	if (so->anchor.cell_bound.end.col == so->sheet->max_object_extent.col &&
 	    so->anchor.cell_bound.end.row == so->sheet->max_object_extent.row)
@@ -408,29 +410,40 @@ cb_sheet_object_view_finalized (SheetObject *so, GObject *view)
  *
  * Asks @so to create a view for the (@sc,@key) pair.
  **/
-void
-sheet_object_new_view (SheetObject *so, SheetControl *sc, gpointer key)
+SheetObjectView *
+sheet_object_new_view (SheetObject *so, SheetObjectViewContainer *container)
 {
-	GObject *view;
+	SheetObjectView *view;
 
-	g_return_if_fail (IS_SHEET_CONTROL (sc));
-	g_return_if_fail (IS_SHEET_OBJECT (so));
+	g_return_val_if_fail (IS_SHEET_OBJECT (so), NULL);
+	g_return_val_if_fail (NULL != container, NULL);
 
-	view = sheet_object_get_view (so, key);
+	view = sheet_object_get_view (so, container);
 	if (view != NULL)
-		return;
+		return NULL;
 
-	view = SO_CLASS (so)->new_view (so, sc, key);
+	view = SO_CLASS (so)->new_view (so, container);
 
-	g_return_if_fail (GTK_IS_OBJECT (view));
+	g_return_val_if_fail (IS_SHEET_OBJECT_VIEW (view), NULL);
 
 	/* Store some useful information */
-	g_object_set_data (G_OBJECT (view), SO_VIEW_OBJECT_KEY, so);
-	g_object_set_data (G_OBJECT (view), SO_VIEW_SHEET_CONTROL_KEY, sc);
-	g_object_set_data (G_OBJECT (view), SO_VIEW_KEY, key);
+	g_object_set_qdata (G_OBJECT (view), sov_so_quark, so);
+	g_object_set_qdata (G_OBJECT (view), sov_container_quark, container);
 	g_object_weak_ref (G_OBJECT (view),
 		(GWeakNotify) cb_sheet_object_view_finalized, so);
 	so->realized_list = g_list_prepend (so->realized_list, view);
+	sheet_object_update_bounds (so, NULL);
+
+	return view;
+}
+
+gboolean
+sheet_object_can_print (SheetObject const *so)
+{
+	g_return_val_if_fail (IS_SHEET_OBJECT (so), FALSE);
+	return  (so->flags & SHEET_OBJECT_IS_VISIBLE) &&
+		(so->flags & SHEET_OBJECT_PRINT) &&
+		SO_CLASS (so)->print != NULL;
 }
 
 void
@@ -478,53 +491,6 @@ sheet_object_anchor_cpy	(SheetObjectAnchor *dst, SheetObjectAnchor const *src)
 	memcpy (dst, src, sizeof (SheetObjectAnchor));
 }
 
-static double
-cell_offset_calc_pixel (Sheet const *sheet, int i, gboolean is_col,
-			SheetObjectAnchorType anchor_type, float offset)
-{
-	ColRowInfo const *cri = sheet_colrow_get_info (sheet, i, is_col);
-	/* TODO : handle other anchor types */
-	if (anchor_type == SO_ANCHOR_PERCENTAGE_FROM_COLROW_END)
-		return (1. - offset) * cri->size_pixels;
-	return offset * cri->size_pixels;
-}
-
-/**
- * sheet_object_position_pixels_get :
- * @so : The sheet object
- * @coords : array of 4 ints where we return the coordinates in pixels
- *
- * Calculate the position of the object @so in pixels from the logical position
- * in the object.
- **/
-void
-sheet_object_position_pixels_get (SheetObject const *so,
-				  SheetControl const *sc, double *coords)
-{
-	GnmRange const *r;
-
-	g_return_if_fail (IS_SHEET_OBJECT (so));
-	g_return_if_fail (IS_SHEET (so->sheet));
-
-	r = &so->anchor.cell_bound;
-
-	coords [0] = sc_colrow_distance_get (sc, TRUE, 0,  r->start.col);
-	coords [2] = coords [0] + sc_colrow_distance_get (sc, TRUE,
-		r->start.col, r->end.col);
-	coords [1] = sc_colrow_distance_get (sc, FALSE, 0, r->start.row);
-	coords [3] = coords [1] + sc_colrow_distance_get (sc, FALSE,
-		r->start.row, r->end.row);
-
-	coords [0] += cell_offset_calc_pixel (so->sheet, r->start.col,
-		TRUE, so->anchor.type [0], so->anchor.offset [0]);
-	coords [1] += cell_offset_calc_pixel (so->sheet, r->start.row,
-		FALSE, so->anchor.type [1], so->anchor.offset [1]);
-	coords [2] += cell_offset_calc_pixel (so->sheet, r->end.col,
-		TRUE, so->anchor.type [2], so->anchor.offset [2]);
-	coords [3] += cell_offset_calc_pixel (so->sheet, r->end.row,
-		FALSE, so->anchor.type [3], so->anchor.offset [3]);
-}
-
 static double
 cell_offset_calc_pt (Sheet const *sheet, int i, gboolean is_col,
 		     SheetObjectAnchorType anchor_type, float offset)
@@ -565,31 +531,38 @@ sheet_object_default_size (SheetObject *so, double *w, double *h)
  **/
 void
 sheet_object_position_pts_get (SheetObject const *so, double *coords)
+{
+	g_return_if_fail (IS_SHEET_OBJECT (so));
+	sheet_object_anchor_to_pts (&so->anchor, so->sheet, coords);
+}
+
+void
+sheet_object_anchor_to_pts (SheetObjectAnchor const *anchor,
+			    Sheet const *sheet, double *res_pts)
 {
 	GnmRange const *r;
 
-	g_return_if_fail (IS_SHEET_OBJECT (so));
-	g_return_if_fail (coords != NULL);
+	g_return_if_fail (res_pts != NULL);
 
-	r = &so->anchor.cell_bound;
+	r = &anchor->cell_bound;
 
-	coords [0] = sheet_col_get_distance_pts (so->sheet, 0,
+	res_pts [0] = sheet_col_get_distance_pts (sheet, 0,
 		r->start.col);
-	coords [2] = coords [0] + sheet_col_get_distance_pts (so->sheet,
+	res_pts [2] = res_pts [0] + sheet_col_get_distance_pts (sheet,
 		r->start.col, r->end.col);
-	coords [1] = sheet_row_get_distance_pts (so->sheet, 0,
+	res_pts [1] = sheet_row_get_distance_pts (sheet, 0,
 		r->start.row);
-	coords [3] = coords [1] + sheet_row_get_distance_pts (so->sheet,
+	res_pts [3] = res_pts [1] + sheet_row_get_distance_pts (sheet,
 		r->start.row, r->end.row);
 
-	coords [0] += cell_offset_calc_pt (so->sheet, r->start.col,
-		TRUE, so->anchor.type [0], so->anchor.offset [0]);
-	coords [1] += cell_offset_calc_pt (so->sheet, r->start.row,
-		FALSE, so->anchor.type [1], so->anchor.offset [1]);
-	coords [2] += cell_offset_calc_pt (so->sheet, r->end.col,
-		TRUE, so->anchor.type [2], so->anchor.offset [2]);
-	coords [3] += cell_offset_calc_pt (so->sheet, r->end.row,
-		FALSE, so->anchor.type [3], so->anchor.offset [3]);
+	res_pts [0] += cell_offset_calc_pt (sheet, r->start.col,
+		TRUE, anchor->type [0], anchor->offset [0]);
+	res_pts [1] += cell_offset_calc_pt (sheet, r->start.row,
+		FALSE, anchor->type [1], anchor->offset [1]);
+	res_pts [2] += cell_offset_calc_pt (sheet, r->end.col,
+		TRUE, anchor->type [2], anchor->offset [2]);
+	res_pts [3] += cell_offset_calc_pt (sheet, r->end.row,
+		FALSE, anchor->type [3], anchor->offset [3]);
 }
 
 /**
@@ -598,14 +571,16 @@ sheet_object_position_pts_get (SheetObject const *so, double *coords)
  * @rinfo : details on what should be moved.
  * @update : Should we do the bound_update now, or leave it for later.
  * 		if FALSE honour the move_with_cells flag.
+ * @undo : if non-NULL add dropped objects to ::objects
  *
  * Uses the relocation info and the anchors to decide whether or not, and how
  * to relocate objects when the grid moves (eg ins/del col/row).
  **/
 void
-sheet_objects_relocate (GnmExprRelocateInfo const *rinfo, gboolean update)
+sheet_objects_relocate (GnmExprRelocateInfo const *rinfo, gboolean update,
+			GnmRelocUndo *undo)
 {
-	GList   *ptr, *next;
+	GSList   *ptr, *next;
 	GnmRange	 dest;
 	gboolean clear, change_sheets;
 
@@ -619,16 +594,17 @@ sheet_objects_relocate (GnmExprRelocateInfo const *rinfo, gboolean update)
 
 	/* Clear the destination range on the target sheet */
 	if (change_sheets) {
-		GList *copy = g_list_copy (rinfo->target_sheet->sheet_objects);
+		GSList *copy = g_slist_copy (rinfo->target_sheet->sheet_objects);
 		for (ptr = copy; ptr != NULL ; ptr = ptr->next ) {
 			SheetObject *so = SHEET_OBJECT (ptr->data);
 			GnmRange const *r  = &so->anchor.cell_bound;
 			if (range_contains (&dest, r->start.col, r->start.row)) {
-				/* lost_objs = g_slist_prepend (lost_objs, g_object_ref (so)); */
+				if (NULL != undo)
+					undo->objs = g_slist_prepend (undo->objs, g_object_ref (so));
 				sheet_object_clear_sheet (so);
 			}
 		}
-		g_list_free (copy);
+		g_slist_free (copy);
 	}
 
 	ptr = rinfo->origin_sheet->sheet_objects;
@@ -637,14 +613,15 @@ sheet_objects_relocate (GnmExprRelocateInfo const *rinfo, gboolean update)
 		GnmRange       *r  = &so->anchor.cell_bound;
 
 		next = ptr->next;
-		if (update && !so->move_with_cells)
+		if (update && 0 == (so->flags & SHEET_OBJECT_MOVE_WITH_CELLS))
 			continue;
 		if (range_contains (&rinfo->origin,
 				    r->start.col, r->start.row)) {
 			/* FIXME : just moving the range is insufficent for all anchor types */
 			/* Toss any objects that would be clipped. */
 			if (range_translate (r, rinfo->col_offset, rinfo->row_offset)) {
-				/* lost_objs = g_slist_prepend (lost_objs, g_object_ref (so)); */
+				if (NULL != undo)
+					undo->objs = g_slist_prepend (undo->objs, g_object_ref (so));
 				sheet_object_clear_sheet (so);
 				continue;
 			}
@@ -657,7 +634,8 @@ sheet_objects_relocate (GnmExprRelocateInfo const *rinfo, gboolean update)
 				sheet_object_update_bounds (so, NULL);
 		} else if (!change_sheets &&
 			   range_contains (&dest, r->start.col, r->start.row)) {
-			/* lost_objs = g_slist_prepend (lost_objs, g_object_ref (so)); */
+			if (NULL != undo)
+				undo->objs = g_slist_prepend (undo->objs, g_object_ref (so));
 			sheet_object_clear_sheet (so);
 			continue;
 		}
@@ -683,7 +661,7 @@ GSList *
 sheet_objects_get (Sheet const *sheet, GnmRange const *r, GType t)
 {
 	GSList *res = NULL;
-	GList *ptr;
+	GSList *ptr;
 
 	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 
@@ -710,7 +688,7 @@ sheet_objects_get (Sheet const *sheet, GnmRange const *r, GType t)
 void
 sheet_objects_clear (Sheet const *sheet, GnmRange const *r, GType t)
 {
-	GList *ptr, *next;
+	GSList *ptr, *next;
 
 	g_return_if_fail (IS_SHEET (sheet));
 
@@ -727,20 +705,6 @@ sheet_objects_clear (Sheet const *sheet, GnmRange const *r, GType t)
 	}
 }
 
-void
-sheet_objects_init (void)
-{
-	GNM_SO_LINE_TYPE;
-	GNM_SO_FILLED_TYPE;
-	SHEET_OBJECT_GRAPH_TYPE;
-	SHEET_OBJECT_IMAGE_TYPE;
-	GNM_GO_DATA_SCALAR_TYPE;
-	GNM_GO_DATA_VECTOR_TYPE;
-	CELL_COMMENT_TYPE;
-
-	sheet_object_widget_register ();
-}
-
 /**
  * sheet_object_dup:
  * @so: a #SheetObject to duplicate
@@ -761,7 +725,7 @@ sheet_object_dup (SheetObject const *so)
 	g_return_val_if_fail (new_so != NULL, NULL);
 
 	SO_CLASS (so)->copy (new_so, so);
-	new_so->type = so->type;
+	new_so->flags = so->flags;
 	sheet_object_anchor_cpy (&new_so->anchor, &so->anchor);
 
 	return new_so;
@@ -781,13 +745,12 @@ sheet_object_clone_sheet (Sheet const *src, Sheet *dst, GnmRange *range)
 {
 	SheetObject *so;
 	SheetObject *new_so;
-	GList *list;
+	GSList *list;
 
 	g_return_if_fail (IS_SHEET (dst));
 	g_return_if_fail (dst->sheet_objects == NULL);
 
-	list = src->sheet_objects;
-	for (; list != NULL; list = list->next) {
+	for (list = src->sheet_objects; list != NULL; list = list->next) {
 		so = (SheetObject *) list->data;
 		if (range == NULL || range_overlap (range, &so->anchor.cell_bound)) {
 			new_so = sheet_object_dup (so);
@@ -798,7 +761,7 @@ sheet_object_clone_sheet (Sheet const *src, Sheet *dst, GnmRange *range)
 		}
 	}
 
-	dst->sheet_objects = g_list_reverse (dst->sheet_objects);
+	dst->sheet_objects = g_slist_reverse (dst->sheet_objects);
 }
 
 
@@ -838,7 +801,7 @@ sheet_object_direction_set (SheetObject *so, gdouble const *coords)
 gboolean
 sheet_object_rubber_band_directly (SheetObject const *so)
 {
-	return SO_CLASS (so)->rubber_band_directly;
+	return FALSE;
 }
 
 /**
@@ -870,10 +833,10 @@ sheet_object_anchor_init (SheetObjectAnchor *anchor,
 
 	if (types == NULL) {
 		static SheetObjectAnchorType const defaultVal [4] = {
-			SO_ANCHOR_PTS_FROM_COLROW_START,
-			SO_ANCHOR_PTS_FROM_COLROW_START,
-			SO_ANCHOR_PTS_FROM_COLROW_START,
-			SO_ANCHOR_PTS_FROM_COLROW_START
+			SO_ANCHOR_PERCENTAGE_FROM_COLROW_START,
+			SO_ANCHOR_PERCENTAGE_FROM_COLROW_START,
+			SO_ANCHOR_PERCENTAGE_FROM_COLROW_START,
+			SO_ANCHOR_PERCENTAGE_FROM_COLROW_START
 		};
 		types = defaultVal;
 	}
@@ -886,6 +849,22 @@ sheet_object_anchor_init (SheetObjectAnchor *anchor,
 
 /*****************************************************************************/
 
+gint
+sheet_object_get_stacking (SheetObject *so)
+{
+	GList *l = so->realized_list;
+
+	if (l != NULL) {
+		FooCanvasItem *item = FOO_CANVAS_ITEM (l->data);
+		FooCanvasGroup *parent = FOO_CANVAS_GROUP (item->parent);
+		GList *link = g_list_find (parent->item_list, item);
+		return g_list_position (parent->item_list, link);
+	}
+	return -1;
+}
+
+/*****************************************************************************/
+
 gint
 sheet_object_adjust_stacking (SheetObject *so, gint positions)
 {
@@ -907,3 +886,156 @@ sheet_object_adjust_stacking (SheetObject *so, gint positions)
 	}
 	return ((before == -1 || after == -1) ? positions : (after - before));
 }
+
+/*****************************************************************************/
+
+void
+sheet_object_view_set_bounds (SheetObjectView *sov,
+			      double const *coords, gboolean visible)
+{
+	SheetObjectViewIface *iface;
+	
+	g_return_if_fail (IS_SHEET_OBJECT_VIEW (sov));
+	iface = SHEET_OBJECT_VIEW_GET_CLASS (sov);
+	if (NULL != iface->set_bounds)
+		(iface->set_bounds) (sov, coords, visible);
+}
+
+SheetObject *
+sheet_object_view_get_so (SheetObjectView *view)
+{
+	return g_object_get_qdata (G_OBJECT (view), sov_so_quark);
+}
+
+void
+sheet_object_view_destroy (SheetObjectView *sov)
+{
+	SheetObjectViewIface *iface = SHEET_OBJECT_VIEW_GET_CLASS (sov);
+	g_return_if_fail (iface != NULL);
+	if (iface->destroy)
+		iface->destroy (sov);
+}
+
+GType
+sheet_object_view_get_type (void)
+{
+	static GType type = 0;
+
+	if (!type) {
+		static GTypeInfo const type_info = {
+			sizeof (SheetObjectViewIface),	/* class_size */
+			NULL,				/* base_init */
+			NULL,				/* base_finalize */
+		};
+
+		type = g_type_register_static (G_TYPE_INTERFACE,
+			"SheetObjectView", &type_info, 0);
+	}
+
+	return type;
+}
+
+/*****************************************************************************/
+
+GType
+sheet_object_imageable_get_type (void)
+{
+	static GType type = 0;
+
+	if (!type) {
+		static GTypeInfo const type_info = {
+			sizeof (SheetObjectImageableIface), /* class_size */
+			NULL,				/* base_init */
+			NULL,				/* base_finalize */
+		};
+
+		type = g_type_register_static (G_TYPE_INTERFACE,
+			"SheetObjectImageable", &type_info, 0);
+	}
+
+	return type;
+}
+
+#define SHEET_OBJECT_IMAGEABLE_CLASS(o)	(G_TYPE_INSTANCE_GET_INTERFACE ((o), SHEET_OBJECT_IMAGEABLE_TYPE, SheetObjectImageableIface))
+
+GtkTargetList *
+sheet_object_get_target_list (SheetObject const *so)
+{
+	if (!IS_SHEET_OBJECT_IMAGEABLE (so))
+		return NULL;
+
+	return SHEET_OBJECT_IMAGEABLE_CLASS (so)->get_target_list (so);
+}
+
+void
+sheet_object_write_image (SheetObject const *so, const char *format, double resolution,
+			  GsfOutput *output, GError **err)
+{
+	g_return_if_fail (IS_SHEET_OBJECT_IMAGEABLE (so));
+
+	SHEET_OBJECT_IMAGEABLE_CLASS (so)->write_image (so, format, resolution, 
+							output, err);
+
+}
+
+/*****************************************************************************/
+
+GType
+sheet_object_exportable_get_type (void)
+{
+	static GType type = 0;
+
+	if (!type) {
+		static GTypeInfo const type_info = {
+			sizeof (SheetObjectExportableIface), /* class_size */
+			NULL,				/* base_init */
+			NULL,				/* base_finalize */
+		};
+
+		type = g_type_register_static (G_TYPE_INTERFACE,
+			"SheetObjectExportable", &type_info, 0);
+	}
+
+	return type;
+}
+
+#define SHEET_OBJECT_EXPORTABLE_CLASS(o)	(G_TYPE_INSTANCE_GET_INTERFACE ((o), SHEET_OBJECT_EXPORTABLE_TYPE, SheetObjectExportableIface))
+
+GtkTargetList *
+sheet_object_exportable_get_target_list (SheetObject const *so)
+{
+	if (!IS_SHEET_OBJECT_EXPORTABLE (so))
+		return NULL;
+
+	return SHEET_OBJECT_EXPORTABLE_CLASS (so)->get_target_list (so);
+}
+
+void
+sheet_object_write_object (SheetObject const *so, const char *format,
+			  GsfOutput *output, GError **err)
+{
+	g_return_if_fail (IS_SHEET_OBJECT_EXPORTABLE (so));
+
+	SHEET_OBJECT_EXPORTABLE_CLASS (so)->write_object (so, format, 
+							output, err);
+
+}
+
+/*****************************************************************************/
+
+void
+sheet_objects_init (void)
+{
+	GNM_SO_LINE_TYPE;
+	GNM_SO_FILLED_TYPE;
+	SHEET_OBJECT_GRAPH_TYPE;
+	SHEET_OBJECT_IMAGE_TYPE;
+	GNM_GO_DATA_SCALAR_TYPE;
+	GNM_GO_DATA_VECTOR_TYPE;
+	GNM_GO_DATA_MATRIX_TYPE;
+	CELL_COMMENT_TYPE;
+
+	sheet_object_widget_register ();
+	sov_so_quark = g_quark_from_static_string ("SheetObject");
+	sov_container_quark = g_quark_from_static_string ("SheetObjectViewContainer");
+}
