@@ -28,10 +28,10 @@ struct _Sheet {
 	Workbook    *workbook;
 	gboolean    being_invalidated;
 	struct {
-		/* An alternating list of ref'd names and expressions.  */
-		GSList *name_exprs;
-		/* An alternating list of deps and ref'd expressions.  */
-		GSList *dep_exprs;
+		/* An alternating list of ref'd names and texpressions.  */
+		GSList *name_texprs;
+		/* An alternating list of deps and ref'd texpressions.  */
+		GSList *dep_texprs;
 		/* A list of deps to relink.  */
 		GSList *relink;
 	} revive;
@@ -56,9 +56,6 @@ struct _Sheet {
 	GSList      *sheet_objects;	/* List of objects in this sheet */
 	GnmCellPos   max_object_extent;
 
-	gboolean    pristine;
-	gboolean    modified;
-
 	/* Sheet level preferences */
 	gboolean    r1c1_addresses;
 	gboolean    display_formulas;
@@ -136,6 +133,9 @@ gboolean    sheet_is_cell_empty 	      (Sheet *sheet, int col, int row);
 gboolean    sheet_col_is_hidden		   (Sheet const *sheet, int col);
 gboolean    sheet_row_is_hidden		   (Sheet const *sheet, int row);
 
+GnmComment *sheet_get_comment		   (Sheet const *sheet,
+					    GnmCellPos const *pos);
+
 /* Create new ColRowInfos from the default sheet style */
 ColRowInfo *sheet_col_new                  (Sheet *sheet);
 ColRowInfo *sheet_row_new                  (Sheet *sheet);
@@ -186,6 +186,7 @@ void        sheet_row_add                 (Sheet *sheet, ColRowInfo *cp);
  */
 /* Col width */
 double  sheet_col_get_distance_pts	  (Sheet const *sheet, int from_col, int to_col);
+int     sheet_col_get_distance_pixels     (Sheet const *sheet, int from_col, int to_col);
 void    sheet_col_set_size_pts		  (Sheet *sheet, int col, double width_pts,
 					   gboolean set_by_user);
 void    sheet_col_set_size_pixels	  (Sheet *sheet, int col, int width_pixels,
@@ -208,8 +209,12 @@ void    sheet_row_set_default_size_pts	  (Sheet *sheet, double height_pts);
 void    sheet_row_set_default_size_pixels (Sheet *sheet, int height_pixels);
 
 /* Find minimum pixel size to display contents (including margins and far grid line) */
-int     sheet_col_size_fit_pixels    (Sheet *sheet, int col);
-int     sheet_row_size_fit_pixels    (Sheet *sheet, int row);
+int     sheet_col_size_fit_pixels    (Sheet *sheet, int col,
+				      int srow, int erow,
+				      gboolean ignore_strings);
+int     sheet_row_size_fit_pixels    (Sheet *sheet, int row,
+				      int scol, int ecol,
+				      gboolean ignore_strings);
 
 gboolean sheet_colrow_can_group	     (Sheet *sheet, GnmRange const *r,
 				      gboolean is_cols);
@@ -247,19 +252,16 @@ void	 sheet_update_only_grid		(Sheet const *s);
 void     sheet_update                   (Sheet const *s);
 void	 sheet_scrollbar_config		(Sheet const *s);
 
-void     sheet_set_dirty	(Sheet *sheet, gboolean is_dirty);
-gboolean sheet_is_pristine	(Sheet const *sheet);
+void     sheet_mark_dirty	(Sheet *sheet);
 GnmRange    sheet_get_extent	(Sheet const *sheet,
 				 gboolean spans_and_merges_extend);
 
-char	*sheet_name_quote	(char const *unquoted_name);
-
 /*
  * Utilities to set cell contents, queueing recalcs,
  * redraws and rendering as required.  Does NOT check for
  * division of arrays.
  */
-void	     sheet_cell_set_expr    (GnmCell *cell, GnmExpr const *expr);
+void	     sheet_cell_set_expr    (GnmCell *cell, GnmExprTop const *texpr);
 void	     sheet_cell_set_value   (GnmCell *cell, GnmValue *v);
 void	     sheet_cell_set_text    (GnmCell *cell, char const *str,
 				     PangoAttrList *markup);
