@@ -2,7 +2,7 @@
  * tabulate.c:
  *
  * Authors:
- *   COPYRIGHT (C) 2003 Morten Welinder (terra@diku.dk)
+ *   COPYRIGHT (C) 2003 Morten Welinder (terra@gnome.org)
  *   COPYRIGHT (C) 2003 Andreas J. Guelzow  <aguelzow@taliesin.ca>
  *
  *
@@ -66,7 +66,7 @@ tabulation_eval (Workbook *wb, int dims, gnm_float const *x,
 static GOFormat const *
 my_get_format (GnmCell const *cell)
 {
-	GOFormat const *format = gnm_style_get_format (cell_get_mstyle (cell));
+	GOFormat const *format = gnm_style_get_format (cell_get_style (cell));
 
 	if (go_format_is_general (format) &&
 	    cell->value != NULL && VALUE_FMT (cell->value) != NULL)
@@ -78,7 +78,7 @@ GSList *
 do_tabulation (WorkbookControl *wbc,
 	       GnmTabulateInfo *data)
 {
-	Workbook *wb = wb_control_workbook (wbc);
+	Workbook *wb = wb_control_get_workbook (wbc);
 	GSList *sheet_idx = NULL;
 	Sheet *sheet = NULL;
 	gboolean sheetdim = (!data->with_coordinates && data->dims >= 3);
