@@ -58,11 +58,9 @@ static struct {
 	{ N_("#NAME?"), NULL, NULL },
 	{ N_("#NUM!"), NULL, NULL },
 	{ N_("#N/A"), NULL, NULL },
-	{ N_("#RECALC!"), NULL, NULL },
 	{ N_("#UNKNOWN!"), NULL, NULL }
 };
 
-
 GnmValue *
 value_new_empty (void)
 {
@@ -71,12 +69,12 @@ value_new_empty (void)
 	return (GnmValue *)&v;
 }
 
-/* Memory pool for ints and bools.  */
-static GOMemChunk *value_int_pool;
+/* Memory pool for bools.  */
+static GOMemChunk *value_bool_pool;
 GnmValue *
 value_new_bool (gboolean b)
 {
-	GnmValueBool *v = CHUNK_ALLOC (GnmValueBool, value_int_pool);
+	GnmValueBool *v = CHUNK_ALLOC (GnmValueBool, value_bool_pool);
 	*((GnmValueType *)&(v->type)) = VALUE_BOOLEAN;
 	v->fmt = NULL;
 	v->val = b;
@@ -86,11 +84,7 @@ value_new_bool (gboolean b)
 GnmValue *
 value_new_int (int i)
 {
-	GnmValueInt *v = CHUNK_ALLOC (GnmValueInt, value_int_pool);
-	*((GnmValueType *)&(v->type)) = VALUE_INTEGER;
-	v->fmt = NULL;
-	v->val = i;
-	return (GnmValue *)v;
+	return value_new_float (i);
 }
 
 static GOMemChunk *value_float_pool;
@@ -183,12 +177,6 @@ value_new_error_NA (GnmEvalPos const *pos)
 	return value_new_error_str (pos, standard_errors[GNM_ERROR_NA].locale_name_str);
 }
 
-GnmValue *
-value_new_error_RECALC (GnmEvalPos const *pos)
-{
-	return value_new_error_str (pos, standard_errors[GNM_ERROR_RECALC].locale_name_str);
-}
-
 char const *
 value_error_name (GnmStdError err, gboolean translated)
 {
@@ -212,7 +200,7 @@ GnmValue *
 value_error_set_pos (GnmValueErr *err, GnmEvalPos const *pos)
 {
     g_return_val_if_fail (err != NULL, NULL);
-    g_return_val_if_fail (err->type == VALUE_ERROR, NULL);
+    g_return_val_if_fail (VALUE_IS_ERROR (err), NULL);
 
     err->src = *pos;
     return (GnmValue *)err;
@@ -225,7 +213,7 @@ value_error_classify (GnmValue const *v)
 
 	g_return_val_if_fail (v != NULL, GNM_ERROR_UNKNOWN);
 
-	if (v->type != VALUE_ERROR)
+	if (!VALUE_IS_ERROR (v))
 		return GNM_ERROR_UNKNOWN;
 
 	for (i = 0; i < G_N_ELEMENTS (standard_errors); i++)
@@ -364,21 +352,21 @@ GnmValue *
 value_new_cellrange_str (Sheet *sheet, char const *str)
 {
 	GnmParsePos  pp;
-	GnmExpr const *expr;
+	GnmExprTop const *texpr;
 
 	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 	g_return_val_if_fail (str != NULL, NULL);
 
-	expr = gnm_expr_parse_str (str,
+	texpr = gnm_expr_parse_str (str,
 		parse_pos_init_sheet (&pp, sheet),
 		GNM_EXPR_PARSE_FORCE_EXPLICIT_SHEET_REFERENCES |
 		GNM_EXPR_PARSE_UNKNOWN_NAMES_ARE_STRINGS,
 		gnm_expr_conventions_default,
 		NULL);
 
-	if (expr != NULL)  {
-		GnmValue *value = gnm_expr_get_range (expr);
-		gnm_expr_unref (expr);
+	if (texpr != NULL)  {
+		GnmValue *value = gnm_expr_top_get_range (texpr);
+		gnm_expr_top_unref (texpr);
 		return value;
 	}
 
@@ -427,6 +415,31 @@ value_new_array_empty (guint cols, guint rows)
 	return (GnmValue *)v;
 }
 
+/*
+ * Returns TRUE, FALSE, or -1.
+ */
+static int
+value_parse_boolean (const char *str, gboolean translated)
+{
+	if (translated) {
+		/* FIXME: ascii???  */
+		if (0 == g_ascii_strcasecmp (str, format_boolean (TRUE)))
+			return TRUE;
+		else if (0 == g_ascii_strcasecmp (str, format_boolean (FALSE)))
+			return FALSE;
+		else
+			return -1;
+	} else {
+		if (0 == g_ascii_strcasecmp (str, "TRUE"))
+			return TRUE;
+		else if (0 == g_ascii_strcasecmp (str, "FALSE"))
+			return FALSE;
+		else
+			return -1;
+	}
+}
+
+
 GnmValue *
 value_new_from_string (GnmValueType t, char const *str, GOFormat *sf,
 		       gboolean translated)
@@ -437,32 +450,14 @@ value_new_from_string (GnmValueType t, char const *str, GOFormat *sf,
 		res = value_new_empty ();
 		break;
 
-	case VALUE_BOOLEAN:
-		if (translated) {
-			/* FIXME: ascii???  */
-			if (0 == g_ascii_strcasecmp (str, format_boolean (TRUE)))
-				res = value_new_bool (TRUE);
-			else if (0 == g_ascii_strcasecmp (str, format_boolean (FALSE)))
-				res = value_new_bool (FALSE);
-		} else {
-			if (0 == g_ascii_strcasecmp (str, "TRUE"))
-				res = value_new_bool (TRUE);
-			else if (0 == g_ascii_strcasecmp (str, "FALSE"))
-				res = value_new_bool (FALSE);
-		}
-		break;
-
-	case VALUE_INTEGER: {
-		char *end;
-		long l;
-
-		errno = 0;
-		l = strtol (str, &end, 10);
-		if (str != end && *end == '\0' && errno != ERANGE)
-			res = value_new_int ((int)l);
+	case VALUE_BOOLEAN: {
+		int i = value_parse_boolean (str, translated);
+		if (i != -1)
+			res = value_new_bool ((gboolean)i);
 		break;
 	}
 
+	case VALUE_INTEGER:
 	case VALUE_FLOAT: {
 		char *end;
 		gnm_float d;
@@ -522,8 +517,7 @@ value_release (GnmValue *value)
 		return;
 
 	case VALUE_BOOLEAN:
-	case VALUE_INTEGER:
-		CHUNK_FREE (value_int_pool, value);
+		CHUNK_FREE (value_bool_pool, value);
 		return;
 
 	case VALUE_FLOAT:
@@ -601,10 +595,6 @@ value_dup (GnmValue const *src)
 		res = value_new_bool (src->v_bool.val);
 		break;
 
-	case VALUE_INTEGER:
-		res = value_new_int (src->v_int.val);
-		break;
-
 	case VALUE_FLOAT:
 		res = value_new_float (src->v_float.val);
 		break;
@@ -664,23 +654,14 @@ value_cmp (void const *ptr_a, void const *ptr_b)
 	case IS_GREATER : return  1;
 	default :
 		break;
-	};
+	}
 	return a->type - b->type;
 }
 
 int
 value_cmp_reverse (void const *ptr_a, void const *ptr_b)
 {
-	GnmValue const *a = *(GnmValue const **)ptr_a;
-	GnmValue const *b = *(GnmValue const **)ptr_b;
-	switch (value_compare (a, b, TRUE)) {
-	case IS_EQUAL :   return  0;
-	case IS_LESS :	  return  1;
-	case IS_GREATER : return -1;
-	default :
-		break;
-	};
-	return b->type - a->type;
+	return -value_cmp (ptr_a, ptr_b);
 }
 
 gboolean
@@ -699,9 +680,6 @@ value_equal (GnmValue const *a, GnmValue const *b)
 	case VALUE_ERROR:
 		return a->v_err.mesg == b->v_err.mesg;
 
-	case VALUE_INTEGER:
-		return a->v_int.val == b->v_int.val;
-
 	case VALUE_FLOAT:
 		return a->v_float.val == b->v_float.val;
 
@@ -746,9 +724,6 @@ value_hash (GnmValue const *v)
 	case VALUE_ERROR:
 		return g_str_hash (v->v_err.mesg->str);
 
-	case VALUE_INTEGER:
-		return (guint)(v->v_int.val);
-
 	case VALUE_FLOAT: {
 		int expt;
 		gnm_float mant = gnm_frexp (gnm_abs (v->v_float.val), &expt);
@@ -804,17 +779,21 @@ value_get_as_bool (GnmValue const *v, gboolean *err)
 	case VALUE_BOOLEAN:
 		return v->v_bool.val;
 
-	case VALUE_STRING:
-		return v->v_str.val->str[0] != '\0';
-
-	case VALUE_INTEGER:
-		return v->v_int.val != 0;
+	case VALUE_STRING: {
+		int i = value_parse_boolean (value_peek_string (v), FALSE);
+		if (i == -1) {
+			if (err)
+				*err = TRUE;
+			return FALSE;
+		}
+		return (gboolean)i;
+	}
 
 	case VALUE_FLOAT:
 		return v->v_float.val != 0.0;
 
 	default:
-		g_warning ("Unhandled value in value_get_boolean.");
+		g_warning ("Unhandled value in value_get_as_bool.");
 
 	case VALUE_CELLRANGE:
 	case VALUE_ARRAY:
@@ -874,10 +853,6 @@ value_get_as_gstring (GnmValue const *v, GString *target,
 		g_string_append (target, v->v_str.val->str);
 		return;
 
-	case VALUE_INTEGER:
-		g_string_append_printf (target, "%d", v->v_int.val);
-		return;
-
 	case VALUE_FLOAT:
 		g_string_append_printf (target, "%.*" GNM_FORMAT_g, GNM_DIG,
 					v->v_float.val);
@@ -916,7 +891,7 @@ value_get_as_gstring (GnmValue const *v, GString *target,
 					g_string_append (target, col_sep);
 
 				/* quote strings */
-				if (val->type == VALUE_STRING)
+				if (VALUE_IS_STRING (val))
 					go_strescape (target, val->v_str.val->str);
 				else
 					value_get_as_gstring (val, target, conv);
@@ -972,9 +947,9 @@ value_peek_string (GnmValue const *v)
 {
 	g_return_val_if_fail (v, "");
 
-	if (v->type == VALUE_STRING)
+	if (VALUE_IS_STRING (v))
 		return v->v_str.val->str;
-	else if (v->type == VALUE_ERROR)
+	else if (VALUE_IS_ERROR (v))
 		return v->v_err.mesg->str;
 	else {
 		static char *cache[2] = { NULL, NULL };
@@ -1007,9 +982,6 @@ value_get_as_int (GnmValue const *v)
 		g_warning ("Getting range as a int: what to do?");
 		return 0;
 
-	case VALUE_INTEGER:
-		return v->v_int.val;
-
 	case VALUE_ARRAY:
 		return 0;
 
@@ -1049,9 +1021,6 @@ value_get_as_float (GnmValue const *v)
 		g_warning ("Getting range as a double: what to do?");
 		return 0.0;
 
-	case VALUE_INTEGER:
-		return (gnm_float) v->v_int.val;
-
 	case VALUE_ARRAY:
 		return 0.0;
 
@@ -1093,14 +1062,14 @@ value_coerce_to_number (GnmValue *v, gboolean *valid, GnmEvalPos const *ep)
 	g_return_val_if_fail (v != NULL, NULL);
 
 	*valid = FALSE;
-	if (v->type == VALUE_STRING) {
+	if (VALUE_IS_STRING (v)) {
 		GnmValue *tmp = format_match_number (value_peek_string (v), NULL,
 			workbook_date_conv (ep->sheet->workbook));
 		value_release (v);
 		if (tmp == NULL)
 			return value_new_error_VALUE (ep);
 		v = tmp;
-	} else if (v->type == VALUE_ERROR)
+	} else if (VALUE_IS_ERROR (v))
 		return v;
 
 	if (!VALUE_IS_NUMBER (v)) {
@@ -1127,39 +1096,6 @@ value_array_set (GnmValue *array, int col, int row, GnmValue *v)
 	array->v_array.vals[col][row] = v;
 }
 
-void
-value_array_resize (GnmValue *v, int width, int height)
-{
-	int x, y, xcpy, ycpy;
-	GnmValue *newval;
-	GnmValue ***tmp;
-
-	g_warning ("Totally untested");
-	g_return_if_fail (v);
-	g_return_if_fail (v->type == VALUE_ARRAY);
-
-	newval = value_new_array (width, height);
-
-	xcpy = MIN (width, v->v_array.x);
-	ycpy = MIN (height, v->v_array.y);
-
-	for (x = 0; x < xcpy; x++)
-		for (y = 0; y < ycpy; y++) {
-			value_array_set (newval, x, y, v->v_array.vals[x][y]);
-			v->v_array.vals[x][y] = NULL;
-		}
-
-	tmp = v->v_array.vals;
-	v->v_array.vals = newval->v_array.vals;
-	newval->v_array.vals = tmp;
-	newval->v_array.x = v->v_array.x;
-	newval->v_array.y = v->v_array.y;
-	v->v_array.x = width;
-	v->v_array.y = height;
-
-	value_release (newval);
-}
-
 static GnmValDiff
 compare_bool_bool (GnmValue const *va, GnmValue const *vb)
 {
@@ -1171,19 +1107,6 @@ compare_bool_bool (GnmValue const *va, GnmValue const *vb)
 	return b ? IS_LESS : IS_EQUAL;
 }
 
-static GnmValDiff
-compare_int_int (GnmValue const *va, GnmValue const *vb)
-{
-	int const a = value_get_as_int (va);
-	int const b = value_get_as_int (vb);
-	if (a == b)
-		return IS_EQUAL;
-	else if (a < b)
-		return IS_LESS;
-	else
-		return IS_GREATER;
-}
-
 static GnmValDiff
 compare_float_float (GnmValue const *va, GnmValue const *vb)
 {
@@ -1212,7 +1135,7 @@ value_diff (GnmValue const *a, GnmValue const *b)
 {
 	GnmValueType ta, tb;
 
-	/* Handle trivial and double NULL case */
+	/* Handle trivial (including empty/empty) and double NULL */
 	if (a == b)
 		return 0.;
 
@@ -1235,7 +1158,7 @@ value_diff (GnmValue const *a, GnmValue const *b)
 			if (t == 0)
 				return 0.;
 		}
-		case VALUE_INTEGER : case VALUE_FLOAT : case VALUE_BOOLEAN :
+		case VALUE_FLOAT: case VALUE_BOOLEAN:
 		default :
 			return DBL_MAX;
 		}
@@ -1247,16 +1170,16 @@ value_diff (GnmValue const *a, GnmValue const *b)
 			if (*b->v_str.val->str == '\0')
 				return 0.;
 
-		case VALUE_INTEGER : case VALUE_FLOAT : case VALUE_BOOLEAN :
+		case VALUE_FLOAT : case VALUE_BOOLEAN :
 		default :
 			return DBL_MAX;
 		}
 	}
 
 	/* Booleans > all numbers (Why did excel do this ?? ) */
-	if (ta == VALUE_BOOLEAN && (tb == VALUE_INTEGER || tb == VALUE_FLOAT))
+	if (ta == VALUE_BOOLEAN && tb == VALUE_FLOAT)
 		return DBL_MAX;
-	if (tb == VALUE_BOOLEAN && (ta == VALUE_INTEGER || ta == VALUE_FLOAT))
+	if (tb == VALUE_BOOLEAN && ta == VALUE_FLOAT)
 		return DBL_MAX;
 
 	switch ((ta > tb) ? ta : tb) {
@@ -1266,12 +1189,6 @@ value_diff (GnmValue const *a, GnmValue const *b)
 	case VALUE_BOOLEAN:
 		return (compare_bool_bool (a, b) == IS_EQUAL) ? 0. : DBL_MAX;
 
-	case VALUE_INTEGER: {
-		int const ia = value_get_as_int (a);
-		int const ib = value_get_as_int (b);
-		return abs (ia - ib);
-	}
-
 	case VALUE_FLOAT: {
 		gnm_float const da = value_get_as_float (a);
 		gnm_float const db = value_get_as_float (b);
@@ -1311,7 +1228,7 @@ value_compare (GnmValue const *a, GnmValue const *b, gboolean case_sensitive)
 			if (*a->v_str.val->str == '\0')
 				return IS_EQUAL;
 
-		case VALUE_INTEGER : case VALUE_FLOAT :
+		case VALUE_FLOAT:
 			return IS_GREATER;
 
 		/* Strings are < FALSE ?? */
@@ -1351,7 +1268,7 @@ value_compare (GnmValue const *a, GnmValue const *b, gboolean case_sensitive)
 			if (*b->v_str.val->str == '\0')
 				return IS_EQUAL;
 
-		case VALUE_INTEGER : case VALUE_FLOAT :
+		case VALUE_FLOAT :
 			return IS_LESS;
 
 		/* Strings are < FALSE ?? */
@@ -1364,9 +1281,9 @@ value_compare (GnmValue const *a, GnmValue const *b, gboolean case_sensitive)
 	}
 
 	/* Booleans > all numbers (Why did excel do this ?? ) */
-	if (ta == VALUE_BOOLEAN && (tb == VALUE_INTEGER || tb == VALUE_FLOAT))
+	if (ta == VALUE_BOOLEAN && tb == VALUE_FLOAT)
 		return IS_GREATER;
-	if (tb == VALUE_BOOLEAN && (ta == VALUE_INTEGER || ta == VALUE_FLOAT))
+	if (tb == VALUE_BOOLEAN && ta == VALUE_FLOAT)
 		return IS_LESS;
 
 	switch ((ta > tb) ? ta : tb) {
@@ -1376,9 +1293,6 @@ value_compare (GnmValue const *a, GnmValue const *b, gboolean case_sensitive)
 	case VALUE_BOOLEAN:
 		return compare_bool_bool (a, b);
 
-	case VALUE_INTEGER:
-		return compare_int_int (a, b);
-
 	case VALUE_FLOAT:
 		return compare_float_float (a, b);
 	default:
@@ -1406,8 +1320,8 @@ criteria_test_equal (GnmValue const *x, GnmValue const *y)
         if (VALUE_IS_NUMBER (x) && VALUE_IS_NUMBER (y))
 	        return (value_get_as_float (x) == value_get_as_float (y));
 	else
-		return (x->type == VALUE_STRING &&
-			y->type == VALUE_STRING &&
+		return (VALUE_IS_STRING (x) &&
+			VALUE_IS_STRING (y) &&
 			g_ascii_strcasecmp (x->v_str.val->str, y->v_str.val->str) == 0);
 }
 
@@ -1422,8 +1336,8 @@ criteria_test_unequal (GnmValue const *x, GnmValue const *y)
 	        return (value_get_as_float (x) != value_get_as_float (y));
 	else
 		/* Hmm...  Is this really right?  number vs string, not unequal?  */
-		return (x->type == VALUE_STRING &&
-			y->type == VALUE_STRING &&
+		return (VALUE_IS_STRING (x) &&
+			VALUE_IS_STRING (y) &&
 			g_ascii_strcasecmp (x->v_str.val->str, y->v_str.val->str) != 0);
 }
 
@@ -1486,10 +1400,10 @@ find_column_of_field (GnmEvalPos const *ep,
 
 	offset = database->v_range.cell.a.col;
 
-	if (field->type == VALUE_INTEGER)
+	if (VALUE_IS_FLOAT (field))
 	        return value_get_as_int (field) + offset - 1;
 
-	if (field->type != VALUE_STRING)
+	if (!VALUE_IS_STRING (field))
 	        return -1;
 
 	sheet = eval_sheet (database->v_range.cell.a.sheet, ep->sheet);
@@ -1770,7 +1684,7 @@ filter_row:
 /****************************************************************************/
 
 GnmValueErr const value_terminate_err = { VALUE_ERROR, NULL, NULL };
-static GnmValueInt const the_value_zero = { VALUE_INTEGER, NULL, 0 };
+static GnmValueFloat const the_value_zero = { VALUE_FLOAT, NULL, 0 };
 GnmValue const *value_zero = (GnmValue const *)&the_value_zero;
 
 void
@@ -1785,10 +1699,9 @@ value_init (void)
 	}
 
 #if USE_VALUE_POOLS
-	/* GnmValueInt and GnmValueBool ought to have the same size.  */
-	value_int_pool =
-		go_mem_chunk_new ("value int/bool pool",
-				   MAX (sizeof (GnmValueInt), sizeof (GnmValueBool)),
+	value_bool_pool =
+		go_mem_chunk_new ("value bool pool",
+				   sizeof (GnmValueBool),
 				   16 * 1024 - 128);
 
 	value_float_pool =
@@ -1829,8 +1742,8 @@ value_shutdown (void)
 	}
 
 #if USE_VALUE_POOLS
-	go_mem_chunk_destroy (value_int_pool, FALSE);
-	value_int_pool = NULL;
+	go_mem_chunk_destroy (value_bool_pool, FALSE);
+	value_bool_pool = NULL;
 
 	go_mem_chunk_destroy (value_float_pool, FALSE);
 	value_float_pool = NULL;
