@@ -13,8 +13,7 @@ typedef enum {
 	 * in the xml
 	 */
 	VALUE_EMPTY	= 10,
-	VALUE_BOOLEAN	= 20, /* Keep bool < int < float */
-	VALUE_INTEGER	= 30,
+	VALUE_BOOLEAN	= 20,
 	VALUE_FLOAT	= 40,
 	VALUE_ERROR	= 50,
 	VALUE_STRING	= 60,
@@ -22,6 +21,13 @@ typedef enum {
 	VALUE_ARRAY	= 80
 } GnmValueType;
 
+/*
+ * This one lives only in old XML files and is understood by
+ * value_new_from_string.
+ */
+#define	VALUE_INTEGER ((GnmValueType)30)
+
+
 typedef struct {
 	GnmValueType const type;
 	GOFormat *fmt;
@@ -31,11 +37,6 @@ struct _GnmValueBool {
 	GOFormat *fmt;
 	gboolean val;
 };
-struct _GnmValueInt {
-	GnmValueType const type;
-	GOFormat *fmt;
-	int val;
-};
 struct _GnmValueFloat {
 	GnmValueType const type;
 	GOFormat *fmt;
@@ -65,12 +66,10 @@ struct _GnmValueArray {
 	GnmValue ***vals;  /* Array [x][y] */
 };
 
-/* FIXME */
 union _GnmValue {
 	GnmValueType const type;
 	GnmValueAny	v_any;
 	GnmValueBool	v_bool;
-	GnmValueInt	v_int;
 	GnmValueFloat	v_float;
 	GnmValueErr	v_err;
 	GnmValueStr	v_str;
@@ -78,14 +77,15 @@ union _GnmValue {
 	GnmValueArray	v_array;
 };
 
-#define	VALUE_TYPE(v)			((v)->v_any.type)
 #define	VALUE_FMT(v)			((v)->v_any.fmt)
 #define VALUE_IS_EMPTY(v)		(((v) == NULL) || ((v)->type == VALUE_EMPTY))
 #define VALUE_IS_EMPTY_OR_ERROR(v)	(VALUE_IS_EMPTY(v) || (v)->type == VALUE_ERROR)
 #define VALUE_IS_STRING(v)		((v)->type == VALUE_STRING)
-#define VALUE_IS_NUMBER(v)		(((v)->type == VALUE_INTEGER) || \
-					 ((v)->type == VALUE_FLOAT) || \
+#define VALUE_IS_BOOLEAN(v)		((v)->type == VALUE_BOOLEAN)
+#define VALUE_IS_ERROR(v)		((v)->type == VALUE_ERROR)
+#define VALUE_IS_NUMBER(v)		(((v)->type == VALUE_FLOAT) || \
 					 ((v)->type == VALUE_BOOLEAN))
+#define VALUE_IS_FLOAT(v)		((v)->type == VALUE_FLOAT)
 
 typedef enum {
 	IS_EQUAL,
@@ -108,7 +108,6 @@ GnmValue *value_new_error_REF        (GnmEvalPos const *pos);
 GnmValue *value_new_error_NAME       (GnmEvalPos const *pos);
 GnmValue *value_new_error_NUM        (GnmEvalPos const *pos);
 GnmValue *value_new_error_NA         (GnmEvalPos const *pos);
-GnmValue *value_new_error_RECALC     (GnmEvalPos const *pos);
 GnmValue *value_new_string           (char const *str);
 GnmValue *value_new_string_nocopy    (char *str);
 GnmValue *value_new_string_str       (GnmString *str);
@@ -175,7 +174,6 @@ extern GnmValueErr const value_terminate_err;
 #define VALUE_TERMINATE ((GnmValue *)&value_terminate_err)
 
 void value_array_set       (GnmValue *array, int col, int row, GnmValue *v);
-void value_array_resize    (GnmValue *v, int width, int height);
 
 /* FIXME: this stuff below ought to go elsewhere.  */
 typedef gboolean (*GnmCriteriaFunc) (GnmValue const *x, GnmValue const *y);
