@@ -15,8 +15,6 @@ WorkbookControl *workbook_control_gui_new	(WorkbookView *optional_view,
 						 Workbook *optional_wb,
 						 GdkScreen *optional_screen);
 
-int      wbcg_sheet_to_page_index (WorkbookControlGUI *wbcg, Sheet *sheet,
-				   SheetControlGUI **res);
 GtkWindow	*wbcg_toplevel	  (WorkbookControlGUI *wbcg);
 void	         wbcg_set_transient (WorkbookControlGUI *wbcg,
 				     GtkWindow *window);
@@ -39,7 +37,6 @@ void       wbcg_toggle_visibility (WorkbookControlGUI *wbcg,
 void       wbcg_copy_toolbar_visibility (WorkbookControlGUI *new_wbcg,
 					 WorkbookControlGUI *wbcg);
 
-void       wbcg_toggle_end_mode   (WorkbookControlGUI *wbcg);
 void       wbcg_set_end_mode      (WorkbookControlGUI *wbcg, gboolean flag);
 
 void	   wbcg_set_transient_for (WorkbookControlGUI *wbcg, GtkWindow *window);
