@@ -38,6 +38,11 @@
 #include <goffice/utils/go-file.h>
 #include <gsf/gsf-impl-utils.h>
 
+enum {
+	PROP_0,
+	PROP_VIEW
+};
+
 #define WBC_CLASS(o) WORKBOOK_CONTROL_CLASS (G_OBJECT_GET_CLASS (o))
 #define WBC_VIRTUAL_FULL(func, handle, arglist, call)		\
 void wb_control_ ## func arglist				\
@@ -64,33 +69,6 @@ wb_control_wrapper_new (WorkbookControl *wbc, WorkbookView *wbv, Workbook *wb,
 	return NULL;
 }
 
-/**
- * wb_control_update_title :
- * @wbc : #WorkbookControl
- *
- * Set the controls title to @title.  Additionally notify the application that
- * the list of windows should be updated.
- **/
-void
-wb_control_update_title (WorkbookControl *wbc)
-{
-	WorkbookControlClass *wbc_class = WBC_CLASS (wbc);
-
-	g_return_if_fail (wbc_class != NULL);
-
-	if (wbc_class != NULL && wbc_class->set_title != NULL) {
-		Workbook const *wb = wb_control_workbook (wbc);
-		char *basename = go_basename_from_uri (wb->uri);
-		char *title = g_strconcat
-			(workbook_is_dirty (wb) ? "*" : "",
-			 basename ? basename : wb->uri,
-			 NULL);
-		wbc_class->set_title (wbc, title);
-		g_free (title);
-		g_free (basename);
-	}
-}
-
 WBC_VIRTUAL (style_feedback,
 	(WorkbookControl *wbc, GnmStyle const *changes), (wbc, changes))
 WBC_VIRTUAL (edit_line_set,
@@ -193,11 +171,17 @@ wb_control_view (WorkbookControl const *wbc)
 	return wbc->wb_view;
 }
 
+GODoc *
+wb_control_get_doc (WorkbookControl const *wbc)
+{
+	return GO_DOC (wb_control_get_workbook (wbc));
+}
+
 Workbook *
-wb_control_workbook (WorkbookControl const *wbc)
+wb_control_get_workbook (WorkbookControl const *wbc)
 {
 	g_return_val_if_fail (IS_WORKBOOK_CONTROL (wbc), NULL);
-	return wb_view_workbook (wbc->wb_view);
+	return wbc->wb_view ? wb_view_get_workbook (wbc->wb_view) : NULL;
 }
 
 Sheet *
@@ -250,16 +234,16 @@ wb_control_parse_and_jump (WorkbookControl *wbc, char const *text)
 				b.row = r->end.row;
 				a.col_relative = a.row_relative = b.col_relative = b.row_relative = FALSE;
 				pp.sheet = NULL; /* make it a global name */
-				if (cellref_equal (&a, &b))
+				if (gnm_cellref_equal (&a, &b))
 					target_range = gnm_expr_new_cellref (&a);
 				else
 					target_range = gnm_expr_new_constant (
 						value_new_cellrange_unsafe (&a, &b));
-				cmd_define_name (wbc, text, &pp, target_range);
+				cmd_define_name (wbc, text, &pp, gnm_expr_top_new (target_range));
 			}
 			return FALSE;
 		} else {
-			target = gnm_expr_get_range (nexpr->expr);
+			target = gnm_expr_top_get_range (nexpr->texpr);
 			if (target == NULL) {
 				go_cmd_context_error_invalid (GO_CMD_CONTEXT (wbc),
 					_("Address"), text);
@@ -291,6 +275,42 @@ cb_wbc_clipboard_modified (GnmApp *app, WorkbookControl *wbc)
 
 /*****************************************************************************/
 
+static void
+wbc_get_property (GObject     *object,
+		  guint        property_id,
+		  GValue      *value,
+		  GParamSpec  *pspec)
+{
+	WorkbookControl *wbc = (WorkbookControl *)object;
+
+	switch (property_id) {
+	case PROP_VIEW:
+		g_value_set_object (value, wbc->wb_view);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+static void
+wbc_set_property (GObject      *object,
+		  guint         property_id,
+		  GValue const *value,
+		  GParamSpec   *pspec)
+{
+	WorkbookControl *wbc = (WorkbookControl *)object;
+
+	switch (property_id) {
+	case PROP_VIEW:
+		wbc->wb_view = g_value_get_object (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
 static GObjectClass *parent_klass;
 static void
 wbc_finalize (GObject *obj)
@@ -310,6 +330,18 @@ workbook_control_class_init (GObjectClass *object_class)
 {
 	parent_klass = g_type_class_peek_parent (object_class);
 	object_class->finalize = wbc_finalize;
+	object_class->get_property = wbc_get_property;
+	object_class->set_property = wbc_set_property;
+
+	g_object_class_install_property
+		(object_class,
+		 PROP_VIEW,
+		 g_param_spec_object ("view",
+				      _("View"),
+				      _("The workbook view being controlled."),
+				      WORKBOOK_VIEW_TYPE,
+				      GSF_PARAM_STATIC |
+				      G_PARAM_READWRITE));
 }
 
 static void
@@ -334,7 +366,7 @@ GSF_CLASS (WorkbookControl, workbook_control,
 
 void
 wb_control_set_view (WorkbookControl *wbc,
-			   WorkbookView *opt_view, Workbook *opt_wb)
+		     WorkbookView *opt_view, Workbook *opt_wb)
 {
 	WorkbookView *wbv;
 
@@ -360,7 +392,7 @@ wb_control_init_state (WorkbookControl *wbc)
 
 	/* Add views all all existing sheets */
 	wbv = wb_control_view (wbc);
-	sheets = workbook_sheets (wb_control_workbook (wbc));
+	sheets = workbook_sheets (wb_control_get_workbook (wbc));
 	for (ptr = sheets; ptr != NULL ; ptr = ptr->next) {
 		sheet = ptr->data;
 		SHEET_FOREACH_VIEW (sheet, view, {
