@@ -28,6 +28,7 @@ struct _WorkbookView {
 	GnmExpr const *auto_expr;
 	char	  *auto_expr_desc;
 	char	  *auto_expr_value_as_string;
+	gboolean  auto_expr_use_max_precision;
 
 	/* selection */
 	char	  *selection_description;
@@ -53,7 +54,9 @@ void		 wb_view_attach_control	  (WorkbookView *wbv, WorkbookControl *wbc);
 void		 wb_view_detach_control	  (WorkbookControl *wbc);
 
 /* Information */
-Workbook	*wb_view_workbook	  (WorkbookView const *wbv);
+GODoc		*wb_view_get_doc	  (WorkbookView const *wbv);
+Workbook	*wb_view_get_workbook	  (WorkbookView const *wbv);
+int		 wb_view_get_index_in_wb  (WorkbookView const *wbv);
 Sheet		*wb_view_cur_sheet	  (WorkbookView const *wbv);
 SheetView	*wb_view_cur_sheet_view	  (WorkbookView const *wbv);
 void		 wb_view_sheet_focus	  (WorkbookView *wbv, Sheet *sheet);
@@ -65,7 +68,6 @@ void         	 wb_view_set_attribute	  (WorkbookView *wbv, char const *name,
 					   char const *value);
 void		 wb_view_preferred_size	  (WorkbookView *wbv,
 					   int w_pixels, int h_pixels);
-void		 wb_view_prefs_update	  (WorkbookView *wbv);
 void		 wb_view_format_feedback  (WorkbookView *wbv, gboolean display);
 void             wb_view_menus_update     (WorkbookView *wbv);
 void		 wb_view_selection_desc   (WorkbookView *wbv, gboolean use_pos,
@@ -76,19 +78,21 @@ void		 wb_view_auto_expr_recalc (WorkbookView *wbv, gboolean display);
 void		 wb_view_auto_expr	  (WorkbookView *wbv,
 					   char const *name,
 					   char const *func_name);
+void             wb_view_auto_expr_precision (WorkbookView *wbv,
+					      gboolean use_max_precision);
 
 /* I/O routines */
-gboolean wb_view_save_as (WorkbookView *wbv, GnmFileSaver *fs,
-			  char const *file_name, GnmCmdContext *cc);
-gboolean wb_view_save	 (WorkbookView *wbv, GnmCmdContext *cc);
-gboolean wb_view_sendto	 (WorkbookView *wbv, GnmCmdContext *cc);
+gboolean wb_view_save_as (WorkbookView *wbv, GOFileSaver *fs,
+			  char const *file_name, GOCmdContext *cc);
+gboolean wb_view_save	 (WorkbookView *wbv, GOCmdContext *cc);
+gboolean wb_view_sendto	 (WorkbookView *wbv, GOCmdContext *cc);
 
 WorkbookView *wb_view_new_from_input  (GsfInput *input,
-				       GnmFileOpener const *optional_format,
+				       GOFileOpener const *optional_format,
 				       IOContext *io_context,
 				       gchar const *optional_encoding);
 WorkbookView *wb_view_new_from_uri  (char const *uri,
-				     GnmFileOpener const *optional_format,
+				     GOFileOpener const *optional_format,
 				     IOContext *io_context, 
 				     gchar const *optional_encoding);
 
