@@ -59,6 +59,7 @@
 #include <goffice/utils/go-glib-extras.h>
 #include <goffice/app/error-info.h>
 #include <gsf/gsf-libxml.h>
+#include <gsf/gsf-doc-meta-data.h>
 #include <gsf/gsf-input.h>
 #include <gsf/gsf-input-gzip.h>
 #include <gsf/gsf-output.h>
@@ -97,7 +98,6 @@ xml_io_conventions (void)
 	res->ref_parser = rangeref_parse;
 	res->range_sep_colon = TRUE;
 	res->sheet_sep_exclamation = TRUE;
-	res->dots_in_names = TRUE;
 	res->output_sheet_name_sep = "!";
 	res->output_argument_sep = ",";
 	res->output_array_col_sep = ",";
@@ -129,7 +129,7 @@ xml_parse_ctx_new (xmlDocPtr     doc,
 	ctxt->expr_map     = g_hash_table_new (g_direct_hash, g_direct_equal);
 	ctxt->shared_exprs = g_ptr_array_new ();
 	ctxt->wb_view      = wb_view;
-	ctxt->wb	   = (wb_view != NULL) ? wb_view_workbook (wb_view) : NULL;
+	ctxt->wb	   = (wb_view != NULL) ? wb_view_get_workbook (wb_view) : NULL;
 	ctxt->exprconv     = xml_io_conventions ();
 
 	return ctxt;
@@ -299,10 +299,7 @@ xml_read_selection_info (XmlParseContext *ctxt, xmlNodePtr tree)
 	sv_selection_reset (sv);
 	for (sel = selections->xmlChildrenNode; sel; sel = sel->next)
 		if (!xmlIsBlankNode (sel) && xml_node_get_range (sel, &r))
-			sv_selection_add_range (sv,
-						r.start.col, r.start.row,
-						r.start.col, r.start.row,
-						r.end.col, r.end.row);
+			sv_selection_add_range (sv, &r);
 
 	if (xml_node_get_int (selections, "CursorCol", &pos.col) &&
 	    xml_node_get_int (selections, "CursorRow", &pos.row))
@@ -360,12 +357,12 @@ xml_read_names (XmlParseContext *ctxt, xmlNodePtr tree,
 	xmlNode *name = e_xml_get_child_by_name (tree, CC2XML ("Names"));
 	xmlChar *name_str;
 	xmlChar *expr_str;
-	GnmExpr const *expr;
 
 	if (name == NULL)
 		return;
 
 	for (name = name->xmlChildrenNode; name ; name = name->next) {
+		GnmExprTop const *texpr;
 		GnmParseError  perr;
 		GnmParsePos    pp;
 
@@ -398,19 +395,21 @@ xml_read_names (XmlParseContext *ctxt, xmlNodePtr tree,
 		}
 
 		parse_error_init (&perr);
-		expr = gnm_expr_parse_str (CXML2C (expr_str), &pp,
+		texpr = gnm_expr_parse_str (CXML2C (expr_str), &pp,
 					   GNM_EXPR_PARSE_DEFAULT,
 					   ctxt->exprconv, &perr);
 		/* See http://bugzilla.gnome.org/show_bug.cgi?id=317427 */
-		if (!expr)
-			expr = gnm_expr_parse_str (CXML2C (expr_str), &pp,
+		if (!texpr)
+			texpr = gnm_expr_parse_str (CXML2C (expr_str), &pp,
 						   GNM_EXPR_PARSE_DEFAULT,
 						   gnm_expr_conventions_default,
 						   NULL);
 
-		if (expr != NULL) {
+		if (texpr != NULL) {
 			char *err = NULL;
-			expr_name_add (&pp, CXML2C (name_str), expr, &err, TRUE, NULL);
+			expr_name_add (&pp, CXML2C (name_str),
+				       texpr,
+				       &err, TRUE, NULL);
 			if (err != NULL) {
 				gnm_io_warning (ctxt->io_context, err);
 				g_free (err);
@@ -425,52 +424,22 @@ xml_read_names (XmlParseContext *ctxt, xmlNodePtr tree,
 }
 
 static void
-xml_read_summary (XmlParseContext *ctxt, xmlNodePtr tree, SummaryInfo *summary_info)
+dom_read_meta_data (XmlParseContext *ctxt, xmlNodePtr tree, GsfDocMetaData *meta_data)
 {
 	xmlNodePtr child;
+	xmlChar *val_str, *type_str, *name;
 
-	g_return_if_fail (ctxt != NULL);
-	g_return_if_fail (tree != NULL);
-	g_return_if_fail (summary_info != NULL);
+	for (child = tree->xmlChildrenNode; child != NULL ; child = child->next) {
+		if (xmlIsBlankNode (child) || NULL == child->name || strcmp (child->name, "Property") ||
+		    NULL == (name	= xml_node_get_cstr (child, "name")) ||
+		    NULL == (type_str	= xml_node_get_cstr (child, "type")) ||
+		    NULL == (val_str	= xml_node_get_cstr (child, NULL)))
+			continue;
 
-	for (child = tree->xmlChildrenNode; child != NULL ; child = child->next)
-		if (!xmlIsBlankNode (child) && child->name && !strcmp (child->name, "Item")) {
-			xmlNodePtr bits;
-			xmlChar *name = NULL;
-
-			for (bits = child->xmlChildrenNode; bits != NULL ; bits = bits->next) {
-				SummaryItem *sit = NULL;
-
-				if (xmlIsBlankNode (bits))
-					continue;
-
-				if (!strcmp (bits->name, "name")) {
-					name = xml_node_get_cstr (bits, NULL);
-				} else {
-					xmlChar *txt;
-					g_return_if_fail (name);
-
-					txt = xml_node_get_cstr (bits, NULL);
-					if (txt != NULL){
-						if (!strcmp (bits->name, "val-string"))
-							sit = summary_item_new_string (CXML2C (name),
-										       CXML2C (txt),
-										       TRUE);
-						else if (!strcmp (bits->name, "val-int"))
-							sit = summary_item_new_int (CXML2C (name),
-										    atoi (CXML2C (txt)));
-
-						if (sit)
-							summary_info_add (summary_info, sit);
-						xmlFree (txt);
-					}
-				}
-			}
-			if (name) {
-				xmlFree (name);
-				name = NULL;
-			}
-		}
+		xmlFree (val_str);
+		xmlFree (type_str);
+		xmlFree (name);
+	}
 }
 
 static void
@@ -807,9 +776,9 @@ xml_read_style (XmlParseContext *ctxt, xmlNodePtr tree)
 		gnm_style_set_wrap_text (style, val);
 
 	if (xml_node_get_int (tree, "Locked", &val))
-		gnm_style_set_content_locked (style, val);
+		gnm_style_set_contents_locked (style, val);
 	if (xml_node_get_int (tree, "Hidden", &val))
-		gnm_style_set_content_hidden (style, val);
+		gnm_style_set_contents_hidden (style, val);
 
 	if (xml_node_get_int (tree, "VAlign", &val))
 		gnm_style_set_align_v (style, val);
@@ -914,7 +883,8 @@ xml_read_style (XmlParseContext *ctxt, xmlNodePtr tree)
 			ValidationOp op = VALIDATION_OP_NONE;
 			xmlChar *title, *msg;
 			gboolean allow_blank, use_dropdown;
-			GnmExpr const *expr0 = NULL, *expr1 = NULL;
+			GnmExprTop const *texpr0 = NULL;
+			GnmExprTop const *texpr1 = NULL;
 
 			if (xml_node_get_int (child, "Style", &val))
 				vstyle = val;
@@ -934,21 +904,29 @@ xml_read_style (XmlParseContext *ctxt, xmlNodePtr tree)
 			parse_pos_init_sheet (&pp, ctxt->sheet);
 			if (NULL != (e_node = e_xml_get_child_by_name (child, CC2XML ("Expression0"))) &&
 			    NULL != (content = xml_node_get_cstr (e_node, NULL))) {
-				expr0 = gnm_expr_parse_str (CXML2C (content), &pp,
-					GNM_EXPR_PARSE_DEFAULT, ctxt->exprconv, NULL);
+				texpr0 = gnm_expr_parse_str
+					(CXML2C (content), &pp,
+					 GNM_EXPR_PARSE_DEFAULT,
+					 ctxt->exprconv, NULL);
 				xmlFree (content);
 			}
 			if (NULL != (e_node = e_xml_get_child_by_name (child, CC2XML ("Expression1"))) &&
 			    NULL != (content = xml_node_get_cstr (e_node, NULL))) {
-				expr1 = gnm_expr_parse_str (CXML2C (content), &pp,
-					GNM_EXPR_PARSE_DEFAULT, ctxt->exprconv, NULL);
+				texpr1 = gnm_expr_parse_str
+					(CXML2C (content), &pp,
+					 GNM_EXPR_PARSE_DEFAULT,
+					 ctxt->exprconv, NULL);
 				xmlFree (content);
 			}
 
-			gnm_style_set_validation (style,
-				validation_new (vstyle, type, op, CXML2C (title),
-					CXML2C (msg), expr0, expr1, allow_blank,
-					use_dropdown));
+			gnm_style_set_validation
+				(style,
+				 validation_new
+				 (vstyle, type, op,
+				  CXML2C (title), CXML2C (msg),
+				  texpr0,
+				  texpr1,
+				  allow_blank, use_dropdown));
 
 			xmlFree (msg);
 			xmlFree (title);
@@ -974,18 +952,22 @@ xml_read_style (XmlParseContext *ctxt, xmlNodePtr tree)
 			parse_pos_init_sheet (&pp, ctxt->sheet);
 			if (NULL != (e_node = e_xml_get_child_by_name (child, CC2XML ("Expression0"))) &&
 			    NULL != (content = xml_node_get_cstr (e_node, NULL))) {
-				cond.expr[0] = gnm_expr_parse_str (CXML2C (content), &pp,
-					GNM_EXPR_PARSE_DEFAULT, ctxt->exprconv, NULL);
+				cond.texpr[0] = gnm_expr_parse_str
+					(CXML2C (content), &pp,
+					 GNM_EXPR_PARSE_DEFAULT,
+					 ctxt->exprconv, NULL);
 				xmlFree (content);
 			} else
-				cond.expr[0] = NULL;
+				cond.texpr[0] = NULL;
 			if (NULL != (e_node = e_xml_get_child_by_name (child, CC2XML ("Expression1"))) &&
 			    NULL != (content = xml_node_get_cstr (e_node, NULL))) {
-				cond.expr[1] = gnm_expr_parse_str (CXML2C (content), &pp,
-					GNM_EXPR_PARSE_DEFAULT, ctxt->exprconv, NULL);
+				cond.texpr[1] = gnm_expr_parse_str
+					(CXML2C (content), &pp,
+					 GNM_EXPR_PARSE_DEFAULT,
+					 ctxt->exprconv, NULL);
 				xmlFree (content);
 			} else
-				cond.expr[1] = NULL;
+				cond.texpr[1] = NULL;
 			if (NULL != (e_node = e_xml_get_child_by_name (child, CC2XML ("Style"))))
 				cond.overlay = xml_read_style (ctxt, e_node);
 			if (NULL == sc)
@@ -1062,17 +1044,17 @@ xml_cell_set_array_expr (XmlParseContext *ctxt,
 			 int const rows, int const cols)
 {
 	GnmParsePos pp;
-	GnmExpr const *expr =
+	GnmExprTop const *texpr =
 		gnm_expr_parse_str (text,
 				    parse_pos_init_cell (&pp, cell),
 				    GNM_EXPR_PARSE_DEFAULT,
 				    ctxt->exprconv, NULL);
 
-	g_return_if_fail (expr != NULL);
+	g_return_if_fail (texpr != NULL);
 	cell_set_array_formula (cell->base.sheet,
 				cell->pos.col, cell->pos.row,
 				cell->pos.col + cols-1, cell->pos.row + rows-1,
-				expr);
+				texpr);
 }
 
 /**
@@ -1277,17 +1259,17 @@ xml_read_cell (XmlParseContext *ctxt, xmlNodePtr tree)
 				 * until after ValueType was added.
 				 */
 				GnmParsePos pos;
-				GnmExpr const *expr = NULL;
+				GnmExprTop const *texpr = NULL;
 				char const *expr_start = gnm_expr_char_start_p (CXML2C (content));
 				if (NULL != expr_start && *expr_start) {
-					expr = gnm_expr_parse_str (expr_start,
-								   parse_pos_init_cell (&pos, cell),
-								   GNM_EXPR_PARSE_DEFAULT,
-								   ctxt->exprconv, NULL);
+					texpr = gnm_expr_parse_str (expr_start,
+								    parse_pos_init_cell (&pos, cell),
+								    GNM_EXPR_PARSE_DEFAULT,
+								    ctxt->exprconv, NULL);
 				}
-				if (expr != NULL) {
-					cell_set_expr (cell, expr);
-					gnm_expr_unref (expr);
+				if (texpr != NULL) {
+					cell_set_expr (cell, texpr);
+					gnm_expr_top_unref (texpr);
 				} else
 					cell_set_text (cell, CXML2C (content));
 			}
@@ -1298,11 +1280,13 @@ xml_read_cell (XmlParseContext *ctxt, xmlNodePtr tree)
 				if (!cell_has_expr (cell)) {
 					g_warning ("XML-IO: Shared expression with no expession? id = %d\ncontent ='%s'",
 						   shared_expr_index, CXML2C (content));
-					cell_set_expr (cell,
-						gnm_expr_new_constant (value_dup (cell->value)));
+					cell_set_expr
+						(cell,
+						 gnm_expr_top_new_constant
+						 (value_dup (cell->value)));
 				}
 				g_ptr_array_add (ctxt->shared_exprs,
-						 (gpointer) cell->base.expression);
+						 (gpointer)cell->base.texpr);
 			} else {
 				g_warning ("XML-IO: Duplicate or invalid shared expression: %d",
 					   shared_expr_index);
@@ -1311,9 +1295,10 @@ xml_read_cell (XmlParseContext *ctxt, xmlNodePtr tree)
 		xmlFree (content);
 	} else if (shared_expr_index > 0) {
 		if (shared_expr_index <= (int)ctxt->shared_exprs->len + 1) {
-			GnmExpr *expr = g_ptr_array_index (ctxt->shared_exprs,
-							    shared_expr_index - 1);
-			cell_set_expr (cell, expr);
+			GnmExprTop *texpr =
+				g_ptr_array_index (ctxt->shared_exprs,
+						   shared_expr_index - 1);
+			cell_set_expr (cell, texpr);
 		} else {
 			g_warning ("XML-IO: Missing shared expression");
 		}
@@ -1654,7 +1639,7 @@ xml_read_sheet_object (XmlParseContext const *ctxt, xmlNodePtr tree)
 
 	/* Class renamed between 1.0.x and 1.2.x */
 	else if (!strcmp (tree->name, "GnmGraph"))
-		so = sheet_object_graph_new (FALSE);
+		so = sheet_object_graph_new (NULL);
 
 	/* Class renamed in 1.2.2 */
 	else if (!strcmp (tree->name, "CellComment"))
@@ -1939,6 +1924,8 @@ xml_sheet_read (XmlParseContext *ctxt, xmlNodePtr tree)
 		g_object_set (sheet, "display-outlines-right", tmp, NULL);
 	if (xml_node_get_bool (tree, "RTL_Layout", &tmp))
 		g_object_set (sheet, "text-is-rtl", tmp, NULL);
+	if (xml_node_get_bool (tree, "Protected", &tmp))
+		g_object_set (sheet, "protected", tmp, NULL);
 	if (xml_node_get_enum (tree, "Visibility", GNM_SHEET_VISIBILITY_TYPE, &tmpi))
 		g_object_set (sheet, "visibility", tmpi, NULL);
 	sheet->tab_color = xml_node_get_color (tree, "TabColor");
@@ -2040,11 +2027,12 @@ xml_read_clipboard_cell (XmlParseContext *ctxt, xmlNodePtr tree,
 		if (is_post_52_array) {
 			g_return_if_fail (content[0] == '=');
 
-			cc->expr = gnm_expr_parse_str (CXML2C (content), &pp,
-						   GNM_EXPR_PARSE_DEFAULT,
-						   ctxt->exprconv, NULL);
+			cc->texpr =
+				gnm_expr_parse_str (CXML2C (content), &pp,
+						    GNM_EXPR_PARSE_DEFAULT,
+						    ctxt->exprconv, NULL);
 
-			g_return_if_fail (cc->expr != NULL);
+			g_return_if_fail (cc->texpr != NULL);
 #warning TODO : arrays
 		} else if (is_value)
 			cc->val = value_new_from_string (value_type,
@@ -2053,25 +2041,28 @@ xml_read_clipboard_cell (XmlParseContext *ctxt, xmlNodePtr tree,
 			GODateConventions const *date_conv =
 				ctxt->wb ? workbook_date_conv (ctxt->wb) : NULL;
 			parse_text_value_or_expr (&pp, CXML2C (content),
-				&cc->val, &cc->expr, value_fmt, date_conv);
+				&cc->val, &cc->texpr, value_fmt, date_conv);
 		}
 
 		if (shared_expr_index > 0) {
 			if (shared_expr_index == (int)ctxt->shared_exprs->len + 1) {
-				if (NULL == cc->expr) {
+				if (NULL == cc->texpr) {
 					/* The parse failed, but we know it is
 					 * an expression.  this can happen
 					 * until we get proper interworkbook
 					 * linkages.  Force the content into
 					 * being an expression.
 					 */
-					cc->expr = gnm_expr_new_constant (value_new_string (
-						gnm_expr_char_start_p (CXML2C (content))));
+					cc->texpr =
+						gnm_expr_top_new_constant
+						(value_new_string
+						 (gnm_expr_char_start_p
+						  (CXML2C (content))));
 					value_release (cc->val);
 					cc->val = value_new_empty ();
 				}
 				g_ptr_array_add (ctxt->shared_exprs,
-						 (gpointer) cc->expr);
+						 (gpointer)cc->texpr);
 			} else {
 				g_warning ("XML-IO: Duplicate or invalid shared expression: %d",
 					   shared_expr_index);
@@ -2080,9 +2071,9 @@ xml_read_clipboard_cell (XmlParseContext *ctxt, xmlNodePtr tree,
 		xmlFree (content);
 	} else if (shared_expr_index > 0) {
 		if (shared_expr_index <= (int)ctxt->shared_exprs->len + 1) {
-			cc->expr = g_ptr_array_index (ctxt->shared_exprs,
-				shared_expr_index - 1);
-			gnm_expr_ref (cc->expr);
+			cc->texpr = g_ptr_array_index (ctxt->shared_exprs,
+						       shared_expr_index - 1);
+			gnm_expr_top_ref (cc->texpr);
 		} else {
 			g_warning ("XML-IO: Missing shared expression");
 		}
@@ -2090,7 +2081,7 @@ xml_read_clipboard_cell (XmlParseContext *ctxt, xmlNodePtr tree,
 	}
 	go_format_unref (value_fmt);
 
-	cr->content = g_slist_prepend (cr->content, cc);
+	cr->contents = g_slist_prepend (cr->contents, cc);
 }
 
 /**
@@ -2110,23 +2101,31 @@ xml_cellregion_read (WorkbookControl *wbc, Sheet *sheet, guchar const *buffer, i
 	XmlParseContext *ctxt;
 	xmlNode	   *l, *clipboard;
 	xmlDoc	   *doc;
-	GnmCellRegion *cr;
+	GnmCellRegion *cr = NULL;
 	int dummy;
+	char *old_num_locale, *old_monetary_locale;
 
 	g_return_val_if_fail (buffer != NULL, NULL);
 
+	old_num_locale = g_strdup (go_setlocale (LC_NUMERIC, NULL));
+	go_setlocale (LC_NUMERIC, "C");
+	old_monetary_locale = g_strdup (go_setlocale (LC_MONETARY, NULL));
+	go_setlocale (LC_MONETARY, "C");
+	go_set_untranslated_bools ();
+
 	doc = xmlParseDoc ((guchar *) buffer);
+
 	if (doc == NULL) {
 		go_cmd_context_error_import (GO_CMD_CONTEXT (wbc),
 			_("Unparsable xml in clipboard"));
-		return NULL;
+		goto err;
 	}
 	clipboard = doc->xmlRootNode;
 	if (clipboard == NULL || strcmp (clipboard->name, "ClipboardRange")) {
 		xmlFreeDoc (doc);
 		go_cmd_context_error_import (GO_CMD_CONTEXT (wbc),
 			_("Clipboard is in unknown format"));
-		return NULL;
+		goto err;
 	}
 
 	/* ctxt->sheet must == NULL or copying objects will break */
@@ -2138,7 +2137,7 @@ xml_cellregion_read (WorkbookControl *wbc, Sheet *sheet, guchar const *buffer, i
 	xml_node_get_int (clipboard, "BaseCol", &cr->base.col);
 	xml_node_get_int (clipboard, "BaseRow", &cr->base.row);
 	/* if it exists it is TRUE */
-	cr->not_as_content = xml_node_get_int (clipboard, "NotAsContent", &dummy);
+	cr->not_as_contents = xml_node_get_int (clipboard, "NotAsContent", &dummy);
 
 	l = e_xml_get_child_by_name (clipboard, CC2XML ("Styles"));
 	if (l != NULL)
@@ -2177,6 +2176,13 @@ xml_cellregion_read (WorkbookControl *wbc, Sheet *sheet, guchar const *buffer, i
 	xml_parse_ctx_destroy (ctxt);
 	xmlFreeDoc (doc);
 
+ err:
+	/* go_setlocale restores bools to locale translation */
+	go_setlocale (LC_MONETARY, old_monetary_locale);
+	g_free (old_monetary_locale);
+	go_setlocale (LC_NUMERIC, old_num_locale);
+	g_free (old_num_locale);
+
 	return cr;
 }
 
@@ -2317,9 +2323,9 @@ xml_workbook_read (IOContext *context,
 	go_setlocale (LC_MONETARY, "C");
 	go_set_untranslated_bools ();
 
-	child = e_xml_get_child_by_name (tree, CC2XML ("Summary"));
+	child = e_xml_get_child_by_name (tree, CC2XML ("MetaData"));
 	if (child)
-		xml_read_summary (ctxt, child, workbook_metadata (ctxt->wb));
+		dom_read_meta_data (ctxt, child, go_doc_get_meta_data (GO_DOC (ctxt->wb)));
 
 	child = e_xml_get_child_by_name (tree, CC2XML ("DateConvention"));
 	if (child != NULL) {
@@ -2629,7 +2635,7 @@ gnumeric_xml_read_workbook (GOFileOpener const *fo,
 	GnumericXMLVersion    version;
 	guint8 const *buf;
 	gsf_off_t size;
-	size_t len;
+	gsf_off_t len;
 
 	g_return_if_fail (input != NULL);
 
@@ -2692,7 +2698,7 @@ gnumeric_xml_read_workbook (GOFileOpener const *fo,
 	ctxt = xml_parse_ctx_new (res, gmr, wb_view);
 	ctxt->version = version;
 	xml_workbook_read (context, ctxt, res->xmlRootNode);
-	workbook_set_saveinfo (wb_view_workbook (ctxt->wb_view),
+	workbook_set_saveinfo (wb_view_get_workbook (ctxt->wb_view),
 		FILE_FL_AUTO, go_file_saver_for_id ("Gnumeric_xml_sax:xml_sax"));
 
 	xml_parse_ctx_destroy (ctxt);
